@@ -28,17 +28,16 @@
 // AI interface stuff
 #include "ExternalAI/Interface/SSkirmishAILibrary.h"
 #include "ExternalAI/Interface/SSkirmishAICallback.h"
-#include "../AI/Wrappers/LegacyCpp/AIGlobalAI.h"
+#include "LegacyCpp/AIGlobalAI.h"
 #include "Game/GameVersion.h"
 
 // E323AI stuff
 #include "AIExport.h"
 #include "CE323AI.h"
 
-// NOTE: myAIs is not static cause we need to count AI instances from outside
 
 // teamId -> AI map
-std::map<int, CAIGlobalAI*> myAIs;
+static std::map<int, CAIGlobalAI*> myAIs;
 // filled in init() of the first instance of this AI
 static const char* aiVersion = NULL;
 // callbacks for all the teams controlled by this Skirmish AI
