@@ -6,6 +6,7 @@
 #include <cctype>
 
 #include "CAI.h"
+#include "CRNG.h"
 #include "CConfigParser.h"
 #include "GameMap.hpp"
 #include "CUnitTable.h"
@@ -21,13 +22,12 @@
 #include "CGroup.h"
 #include "CScopedTimer.h"
 #include "Util.hpp"
+#include "CCoverageHandler.h"
 #include "ReusableObjectFactory.hpp"
 
-#include "../AI/Wrappers/LegacyCpp/AIGlobalAI.h"
+class IGlobalAICallback;
+#include "LegacyCpp/IAICallback.h"
 
-extern std::map<int, CAIGlobalAI*> myAIs;
-
-int CE323AI::instances = 0;
 
 CE323AI::CE323AI() {
 	isRunning = false;
@@ -41,14 +41,8 @@ void CE323AI::InitAI(IGlobalAICallback* callback, int team) {
   
 	CLogger::logLevel loggingLevel = CLogger::VERBOSE;
 	
-	instances++;
-	
-	ai                = new AIClasses();
-	ai->cb            = callback->GetAICallback();
-	ai->cbc           = callback->GetCheatInterface();
-	ai->team          = team;
-	ai->allyTeam      = ai->cb->GetMyAllyTeam();
-	
+	ai = new AIClasses(callback);
+
 	std::map<std::string, std::string> options = ai->cb->GetMyOptionValues();
 	if (options.find(optionDifficulty) != options.end()) {
 		ai->difficulty = static_cast<difficultyLevel>(atoi(options[optionDifficulty].c_str()));
@@ -56,22 +50,13 @@ void CE323AI::InitAI(IGlobalAICallback* callback, int team) {
 	if (options.find(optionLoggingLevel) != options.end()) {
 		loggingLevel = static_cast<CLogger::logLevel>(atoi(options[optionLoggingLevel].c_str()));
 	}
-	
-	ai->logger        = new CLogger(ai, /*CLogger::LOG_STDOUT |*/ CLogger::LOG_FILE, loggingLevel);
-	ai->cfgparser     = new CConfigParser(ai);
-	ai->unittable     = new CUnitTable(ai);
 
-	ai->allyAITeam    = 1;
-#if !defined(BUILDING_AI_FOR_SPRING_0_81_2)
-	std::map<int, CAIGlobalAI*>::iterator it;
-	for (it = myAIs.begin(); it != myAIs.end(); ++it) {
-		if (it->first != team) {
-			if (ai->cb->GetTeamAllyTeam(it->first) == ai->allyTeam)
-				ai->allyAITeam++;
-		}
-	}
-#endif
-	LOG_II("CE323AI::InitAI allyAITeam = " << ai->allyAITeam)
+	ai->logger = new CLogger(ai, /*CLogger::LOG_STDOUT |*/ CLogger::LOG_FILE, loggingLevel);
+	
+	LOG_II("CE323AI::InitAI allyIndex = " << ai->allyIndex)
+
+	ai->cfgparser = new CConfigParser(ai);
+	ai->unittable = new CUnitTable(ai);
 	
 	std::string configfile = ai->cfgparser->getFilename(GET_CFG);
 	ai->cfgparser->parseConfig(configfile);
@@ -106,29 +91,24 @@ void CE323AI::InitAI(IGlobalAICallback* callback, int team) {
 	ai->intel         = new CIntel(ai);
 	ai->military      = new CMilitary(ai);
 	ai->defensematrix = new CDefenseMatrix(ai);
+	ai->coverage      = new CCoverageHandler(ai);
 
 #if !defined(BUILDING_AI_FOR_SPRING_0_81_2)	
 	/* Set the new graph stuff */
 	ai->cb->DebugDrawerSetGraphPos(-0.4f, -0.4f);
 	ai->cb->DebugDrawerSetGraphSize(0.8f, 0.6f);
 #endif
-
-	/*
-	ai->uploader->AddString("aiversion", AI_VERSION_NR);
-	ai->uploader->AddString("ainame",    AI_NAME);
-	ai->uploader->AddString("modname",   ai->cb->GetModName());
-	ai->uploader->AddString("mapname",   ai->cb->GetMapName());
-	*/
 }
 
 void CE323AI::ReleaseAI() {
-	instances--;
 	
-	if (instances == 0) {
+	if (ai->isSole()) {
 		ReusableObjectFactory<CGroup>::Shutdown();
 		ReusableObjectFactory<CUnit>::Shutdown();
+		ReusableObjectFactory<CCoverageCell>::Shutdown();
 	}
 
+	delete ai->coverage;
 	delete ai->defensematrix;
 	delete ai->military;
 	delete ai->intel;
@@ -137,8 +117,8 @@ void CE323AI::ReleaseAI() {
 	delete ai->tasks;
 	delete ai->wishlist;
 	delete ai->economy;
-	delete ai->unittable;
 	delete ai->gamemap;
+	delete ai->unittable;
 	delete ai->cfgparser;
 	delete ai->logger;
 	delete ai;
@@ -150,41 +130,47 @@ void CE323AI::ReleaseAI() {
 
 /* Called when units are spawned in a factory or when game starts */
 void CE323AI::UnitCreated(int uid, int bid) {
-	CUnit *unit = ai->unittable->requestUnit(uid, bid);
+	CUnit* unit = ai->unittable->requestUnit(uid, bid);
 	
 	LOG_II("CE323AI::UnitCreated " << (*unit))
 
-	// unit->def->isCommander
-	if ((unit->type->cats&COMMANDER) && !ai->economy->isInitialized()) {
+	if ((unit->type->cats&COMMANDER).any() && !ai->economy->isInitialized()) {
 		ai->economy->init(*unit);
 	}
 
 	// HACK: for metal extractors & geoplants only
 	ai->economy->addUnitOnCreated(*unit);
 
+	ai->coverage->addUnit(unit);
+
 	if (bid < 0)
 		return; // unit was spawned from nowhere (e.g. commander), or given by another player
 
-	unsigned int c = unit->type->cats;
-	if (c&MOBILE) {
-		CUnit *builder = ai->unittable->getUnit(bid);
+	unitCategory c = unit->type->cats;
+	if ((c&MOBILE).any()) {
+		CUnit* builder = ai->unittable->getUnit(bid);
 		// if builder is a mobile unit (like Consul in BA) then do not 
 		// assign move command...
-		if (builder->type->cats&STATIC) {
+		if ((builder->type->cats&STATIC).any()) {
 			// NOTE: factories should be already rotated in proper direction 
 			// to prevent units going outside the map
-			if (c&AIR) {
-				if (c&ANTIAIR)
+			if ((c&AIR).any()) {
+				if ((c&ANTIAIR).any())
 					unit->guard(bid, true);
 				else
 					unit->moveRandom(450.0f, true);
 			}
-			else if (c&BUILDER)
+			else if ((c&BUILDER).any())
 				unit->moveForward(200.0f);
 			else
 				unit->moveForward(400.0f);
 		}
 	}
+	else {
+		if ((c&NANOTOWER).any()) {
+			unit->patrol(unit->getForwardPos(100.0f), true);
+		}
+	}
 
 	// TODO: check if UnitIdle for factory/builder is called after 
 	// UnitCreated then we do not need "unitsUnderConstruction" map
@@ -197,7 +183,7 @@ void CE323AI::UnitCreated(int uid, int bid) {
 
 /* Called when units are finished in a factory and able to move */
 void CE323AI::UnitFinished(int uid) {
-	CUnit *unit = ai->unittable->getUnit(uid);
+	CUnit* unit = ai->unittable->getUnit(uid);
 	
 	if(!unit) {
 		const UnitDef *ud = ai->cb->GetUnitDef(uid);
@@ -209,10 +195,10 @@ void CE323AI::UnitFinished(int uid) {
 
 	// NOTE: commanders and static units should start actions earlier than
 	// usual units
-	if (unit->builtBy == -1 || (unit->type->cats&STATIC))
-		ai->unittable->unitsAliveTime[uid] = IDLE_UNIT_TIMEOUT;
+	if (unit->builtBy == -1 || (unit->type->cats&STATIC).any())
+		unit->aliveFrames = IDLE_UNIT_TIMEOUT;
 	else
-		ai->unittable->unitsAliveTime[uid] = 0;
+		unit->aliveFrames = 0; // reset time at which unit was building
 			
 	ai->unittable->idle[uid] = true;
 
@@ -221,15 +207,13 @@ void CE323AI::UnitFinished(int uid) {
 		ai->unittable->builders[unit->builtBy] = true;
 	}
 
-	/* Eco unit */
-	if (unit->isEconomy())
+	if (unit->isEconomy()) {
 		ai->economy->addUnitOnFinished(*unit);
-	/* Military unit */
-	else if(unit->type->cats&ATTACKER)
-		ai->military->addUnit(*unit);
-	else
-		LOG_WW("CE323AI::UnitFinished invalid unit " << *unit)
-
+	}
+	else if(!ai->military->addUnit(*unit)) {
+		LOG_WW("CE323AI::UnitFinished unit " << (*unit) << " is NOT under AI control")
+	}
+	
 	// NOTE: very important to place this line AFTER registering a unit in
 	// either economy or military blocks
 	ai->unittable->unitsUnderConstruction.erase(uid);
@@ -238,7 +222,7 @@ void CE323AI::UnitFinished(int uid) {
 /* Called on a destroyed unit */
 void CE323AI::UnitDestroyed(int uid, int attacker) {
 	ai->tasks->onUnitDestroyed(uid, attacker);
-	
+
 	CUnit *unit = ai->unittable->getUnit(uid);
 	if (unit) {
 		LOG_II("CE323AI::UnitDestroyed " << (*unit))
@@ -248,7 +232,7 @@ void CE323AI::UnitDestroyed(int uid, int attacker) {
 
 /* Called when unit is idle */
 void CE323AI::UnitIdle(int uid) {
-	CUnit *unit = ai->unittable->getUnit(uid);
+	CUnit* unit = ai->unittable->getUnit(uid);
 	
 	if (unit == NULL) {
 		const UnitDef *ud = ai->cb->GetUnitDef(uid);
@@ -256,7 +240,7 @@ void CE323AI::UnitIdle(int uid) {
 		return;
 	}
 
-	if(ai->unittable->unitsUnderPlayerControl.find(uid) != ai->unittable->unitsUnderPlayerControl.end()) {
+	if (ai->unittable->unitsUnderPlayerControl.find(uid) != ai->unittable->unitsUnderPlayerControl.end()) {
 		ai->unittable->unitsUnderPlayerControl.erase(uid);
 		assert(unit->group == NULL);
 		LOG_II("CE323AI::UnitIdle " << (*unit) << " is under AI control again")
@@ -267,7 +251,7 @@ void CE323AI::UnitIdle(int uid) {
 	
 	ai->unittable->idle[uid] = true;
 	
-	if (unit->type->cats&(BUILDER|FACTORY))
+	if ((unit->type->cats&(BUILDER|FACTORY)).any())
 		ai->unittable->unitsBuilding.erase(uid);
 }
 
@@ -326,8 +310,8 @@ void CE323AI::UnitDamaged(int damaged, int attacker, float damage, float3 dir) {
 
 /* Called on move fail e.g. can't reach the point */
 void CE323AI::UnitMoveFailed(int uid) {
-	CUnit *unit = ai->unittable->getUnit(uid);
-	if (unit && (unit->type->cats&(LAND|SEA))) {
+	CUnit* unit = ai->unittable->getUnit(uid);
+	if (unit && (unit->type->cats&(LAND|SEA)).any()) {
 		// if unit is inside a factory then force moving...
 		float3 pos = ai->cb->GetUnitPos(unit->key);
 		std::map<int, CUnit*>::iterator it;
@@ -335,8 +319,8 @@ void CE323AI::UnitMoveFailed(int uid) {
 			float distance = ai->cb->GetUnitPos(it->first).distance2D(pos);
 			if (distance < 16.0) {
 				unit->moveForward(200.0f);
-				if (ai->unittable->unitsAliveTime[uid] <= IDLE_UNIT_TIMEOUT)
-					ai->unittable->unitsAliveTime[uid] = 0;
+				if (!unit->canPerformTasks())
+					unit->aliveFrames = 0; // prolong idle timeout
 			}
 		}
 	}
@@ -359,13 +343,17 @@ void CE323AI::EnemyEnterRadar(int enemy) {
 void CE323AI::EnemyLeaveRadar(int enemy) {
 }
 
+void CE323AI::EnemyCreated(int enemy) {
+	ai->intel->onEnemyCreated(enemy);
+}
+
 void CE323AI::EnemyDestroyed(int enemy, int attacker) {
 	ai->military->onEnemyDestroyed(enemy, attacker);
 	ai->tasks->onEnemyDestroyed(enemy, attacker);
+	ai->intel->onEnemyDestroyed(enemy, attacker);
 }
 
 void CE323AI::EnemyDamaged(int damaged, int attacker, float damage, float3 dir) {
-
 }
 
 
@@ -381,7 +369,8 @@ void CE323AI::GotChatMsg(const char* msg, int player) {
 		modEco[] = "economy",
 		modPF[] = "pathfinder",
 		modPG[] = "pathgraph",
-		modDM[] = "defensematrix";
+		modDM[] = "defensematrix",
+		modCL[] = "coverage";
 
 	// NOTE: accept AI commands from spectators only
 	if (ai->cb->GetPlayerTeam(player) >= 0)
@@ -397,8 +386,8 @@ void CE323AI::GotChatMsg(const char* msg, int player) {
 			if (ai->isMaster()) {
 				ai->cb->SendTextMsg("Usage: !e323ai <module>", 0);
 				ai->cb->SendTextMsg("where", 0);
-				ai->cb->SendTextMsg("\t<module> ::= pathfinder|pathgraph|military|threatmap|defensematrix", 0);
-				ai->cb->SendTextMsg("\t<module> ::= pf|pg|mil|tm|dm", 0);
+				ai->cb->SendTextMsg("\t<module> ::= pathfinder|pathgraph|military|threatmap|defensematrix|coverage", 0);
+				ai->cb->SendTextMsg("\t<module> ::= pf|pg|mil|tm|dm|cl", 0);
 			}
 			return;
 		}
@@ -424,9 +413,62 @@ void CE323AI::GotChatMsg(const char* msg, int player) {
 			isDebugOn = ai->defensematrix->switchDebugMode();
 			cmd.assign(modDM);
 		}
+		else if (cmd == modCL || cmd == "cl") {
+			isDebugOn = ai->coverage->toggleVisualization();
+			cmd.assign(modCL);
+		}
 		else {
-			line.assign("Module \"" + cmd + "\" is unknown or unsupported for visual debugging");
-			ai->cb->SendTextMsg(line.c_str(), 0);
+			if (cmd == "unit") {
+				// dump selected unit info...
+				line.clear();
+
+				if (ai->cb->GetSelectedUnits(&ai->unitIDs[0], 1) > 0) {
+					std::stringstream buffer;
+					CUnit* unit = ai->unittable->getUnit(ai->unitIDs[0]);
+					if (unit) {
+						buffer << ".id = " << unit->key << "\n";
+						buffer << ".humanName = " << unit->def->humanName << "\n";
+						buffer << ".isMicroing = " << unit->isMicroing() << "\n";
+						buffer << ".microingFrames = " << unit->microingFrames << "\n";
+						buffer << ".aliveFrames = " << unit->aliveFrames << "\n";
+						buffer << ".waiting = " << unit->waiting << "\n";
+						buffer << ".idle = " << ai->unittable->idle[unit->key] << "\n";
+						
+						ai->cb->SendTextMsg(buffer.str().c_str(), 0);
+					}
+				}
+			}
+			else if (cmd == "tmv") {
+				// dump threat value at mouse cursor...
+				float value = 0.0f;
+				float3 pos = ai->cb->GetMousePos();
+				CUnit* unit = NULL;				
+				if (ai->cb->GetSelectedUnits(&ai->unitIDs[0], 1) > 0) {
+					unit = ai->unittable->getUnit(ai->unitIDs[0]);
+					if (unit && unit->group) {
+						value = ai->threatmap->getThreat(pos, 0.0f, unit->group);
+					}
+				}
+				else {
+					value = ai->threatmap->getThreat(pos, 0.0f);
+				}
+				
+				std::stringstream buffer;
+				buffer << "Threat value";
+				if (unit)
+					buffer << " for " << unit->def->humanName;
+				buffer << " at position (" << pos.x << "," << pos.z << ") is " << value;
+				
+				ai->cb->SendTextMsg(buffer.str().c_str(), 0);				
+			}
+			else {
+				line.assign("Module \"" + cmd + "\" is unknown or unsupported for visual debugging");
+				
+			}
+
+			if (!line.empty())
+				ai->cb->SendTextMsg(line.c_str(), 0);
+			
 			return;
 		}
 		
@@ -563,7 +605,7 @@ void CE323AI::Update() {
 	/* Make sure we shift the multiplexer for each instance of E323AI */
 	int aiframe = localFrame + ai->team;
 	
-	// Make sure we start playing since "eco-incomes" update
+	// Make sure we start playing from "eco-incomes" update
 	if(!isRunning) {
 		isRunning = aiframe % MULTIPLEXER == 0;
 	}
@@ -605,6 +647,7 @@ void CE323AI::Update() {
 		case 5: { /* update defense matrix */
 			PROFILE(defensematrix)
 			ai->defensematrix->update();
+			ai->coverage->update();
 		}
 
 		case 6: { /* update military */
@@ -615,7 +658,7 @@ void CE323AI::Update() {
 
 		case 7: { /* update economy */
 			PROFILE(economy)
-			ai->economy->update(localFrame);
+			ai->economy->update();
 		}
 		break;
 
