@@ -7,7 +7,6 @@
 #include <limits>
 
 #include "CUnitTable.h"
-#include "CWishList.h"
 #include "CPathfinder.h"
 #include "CUnit.h"
 #include "CGroup.h"
@@ -18,7 +17,7 @@
 #include "CDefenseMatrix.h"
 
 
-CTaskHandler::CTaskHandler(AIClasses *ai): ARegistrar(500, std::string("taskhandler")) {
+CTaskHandler::CTaskHandler(AIClasses *ai): ARegistrar(500) {
 	this->ai = ai;
 
 	statsMaxActiveTasks = 0;
@@ -36,7 +35,7 @@ CTaskHandler::~CTaskHandler() {
 
 void CTaskHandler::remove(ARegistrar &obj) {
 	switch(obj.regtype()) {
-		case REG_TASK: {
+		case ARegistrar::TASK: {
 			ATask *task = dynamic_cast<ATask*>(&obj);
 			LOG_II("CTaskHandler::remove " << (*task))
 			for(std::list<CGroup*>::iterator it = task->groups.begin(); it != task->groups.end(); ++it) {
@@ -50,14 +49,14 @@ void CTaskHandler::remove(ARegistrar &obj) {
 			obsoleteTasks.push(task);
 			break;
 		}
-		case REG_GROUP: {
+		case ARegistrar::GROUP: {
 			CGroup *group = dynamic_cast<CGroup*>(&obj);
 			LOG_II("CTaskHandler::remove " << (*group))
 			groupToTask.erase(group->key);
 			break;
 		}
 		default:
-			assert(true);
+			assert(false);
 	}
 	
 	obj.unreg(*this);
@@ -109,12 +108,12 @@ void CTaskHandler::update() {
 	}
 }
 
-float3 CTaskHandler::getPos(CGroup &group) {
+float3 CTaskHandler::getPos(const CGroup& group) {
 	assert(groupToTask.find(group.key) != groupToTask.end());
 	return groupToTask[group.key]->pos;
 }
 
-ATask* CTaskHandler::getTask(CGroup &group) {
+ATask* CTaskHandler::getTask(const CGroup& group) {
 	std::map<int, ATask*>::iterator i = groupToTask.find(group.key);
 	if (i == groupToTask.end())
 		return NULL;
@@ -130,12 +129,14 @@ ATask* CTaskHandler::getTaskByTarget(int uid) {
 	return NULL;
 }
 
-bool CTaskHandler::addTask(ATask *task) {
+bool CTaskHandler::addTask(ATask* task, ATask::NPriority p) {
 	if (task == NULL)
 		return false;
 	
 	assert(task->t != TASK_UNDEFINED);
 
+	task->priority = p;
+
 	std::list<CGroup*>::iterator it;
 
 	// NOTE: this is required because otherwise memory used by task will never 
@@ -157,9 +158,9 @@ bool CTaskHandler::addTask(ATask *task) {
 		return false;
 	}
 	
-	// TODO: after MoveTask introducing remove the code below
+	// TODO: after MoveTask is implemented remove the code below
 	for(it = task->groups.begin(); it != task->groups.end(); ++it) {
-		CGroup *group = *it;
+		CGroup* group = *it;
 		if (task->isMoving && !ai->pathfinder->pathAssigned(*group)) {
 			if(!ai->pathfinder->addGroup(*group)) {
 				task->remove();
