@@ -11,7 +11,7 @@
 #include "ReusableObjectFactory.hpp"
 
 std::map<std::string, unitCategory> CUnitTable::str2cat;
-std::map<unitCategory, std::string> CUnitTable::cat2str;
+CUnitTable::UnitCategory2StrMap CUnitTable::cat2str;
 std::vector<unitCategory> CUnitTable::cats;
 
 CUnitTable::CUnitTable(AIClasses *ai): ARegistrar(100) {
@@ -29,7 +29,8 @@ CUnitTable::CUnitTable(AIClasses *ai): ARegistrar(100) {
 		cat2str[AIR]         = "AIR";
 		cat2str[SEA]         = "SEA";
 		cat2str[LAND]        = "LAND";
-				
+		cat2str[SUB]         = "SUB";
+		
 		cat2str[STATIC]      = "STATIC";
 		cat2str[MOBILE]      = "MOBILE";
 
@@ -59,15 +60,32 @@ CUnitTable::CUnitTable(AIClasses *ai): ARegistrar(100) {
 		cat2str[KBOT]        = "KBOT";
 		cat2str[VEHICLE]     = "VEHICLE";
 		cat2str[HOVER]       = "HOVER";
-		cat2str[AIRCRAFT]    = "AIRCRAFT"; // not used yet
-		cat2str[NAVAL]       = "NAVAL"; // not used yet
+		cat2str[AIRCRAFT]    = "AIRCRAFT";
+		cat2str[NAVAL]       = "NAVAL";
 
 		cat2str[DEFENSE]     = "DEFENSE";
+
+		cat2str[JAMMER]      = "JAMMER";
+		cat2str[NUKE]        = "NUKE";
+		cat2str[ANTINUKE]    = "ANTINUKE";
+		cat2str[PARALYZER]   = "PARALYZER";
+		cat2str[TORPEDO]     = "TORPEDO";
+		cat2str[TRANSPORT]   = "TRANSPORT";
+		cat2str[EBOOSTER]    = "EBOOSTER";
+		cat2str[MBOOSTER]    = "MBOOSTER";
+		cat2str[SHIELD]      = "SHIELD";
+		cat2str[NANOTOWER]   = "NANOTOWER";
+		cat2str[REPAIRPAD]   = "REPAIRPAD";
+
+		cat2str[WIND]        = "WIND";
+		cat2str[TIDAL]       = "TIDAL";
+		
+		assert(cat2str.size() == MAX_CATEGORIES);
 	}
 
 	if (str2cat.empty()) {
 		/* Create the str2cat table and cats vector */
-		std::map<unitCategory,std::string>::iterator i;
+		UnitCategory2StrMap::iterator i;
 		for (i = cat2str.begin(); i != cat2str.end(); ++i) {
 			cats.push_back(i->first);
 			str2cat[i->second] = i->first;
@@ -81,24 +99,32 @@ CUnitTable::CUnitTable(AIClasses *ai): ARegistrar(100) {
 	buildTechTree();
 
 	bool success = false;
-
-	std::string filename = ai->cfgparser->getFilename(GET_CAT|GET_TEAM);
-	if (ai->cfgparser->fileExists(filename))
-		success = ai->cfgparser->parseCategories(filename, units);
-	if (!success) {
-		filename = ai->cfgparser->getFilename(GET_CAT);
-		success = ai->cfgparser->parseCategories(filename, units);
-		if (!success) {
-			filename = util::GetAbsFileName(ai->cb, std::string(CFG_FOLDER) + filename, false);
-			generateCategorizationFile(filename);
+	unsigned int lastFlags = GET_CAT;
+	unsigned int flagsOrder[] = { GET_CAT|GET_VER|GET_TEAM, GET_CAT|GET_VER, GET_CAT|GET_TEAM, GET_CAT };
+	std::string filename;
+	
+	for (int i = 0; i < sizeof(flagsOrder) / sizeof(unsigned int); i++) {
+		lastFlags = flagsOrder[i];
+		filename = ai->cfgparser->getFilename(lastFlags);
+		if (ai->cfgparser->fileExists(filename)) {
+			success = ai->cfgparser->parseCategories(filename, units);
+			if (success)
+				break;
 		}
+		else
+			LOG_WW("CUnitTable::CUnitTable Categorization file not found: " << filename)
 	}
-	if (success) {
-		// try loading overload file...
-		filename = ai->cfgparser->getFilename(GET_CAT|GET_VER);
-		if (ai->cfgparser->fileExists(filename))
-			ai->cfgparser->parseCategories(filename, units);
-	}		
+	
+	if (!success) {
+		filename = util::GetAbsFileName(ai->cb, std::string(CFG_FOLDER) + filename, false);
+		generateCategorizationFile(filename);
+	}
+
+	filename = ai->cfgparser->getFilename(lastFlags|GET_PATCH);
+	if (ai->cfgparser->fileExists(filename))
+		ai->cfgparser->parseCategories(filename, units, true);
+	else
+		LOG_WW("CUnitTable::CUnitTable Categorization patch-file not found: " << filename)
 
 	/* Generate the buildBy and canBuild lists per UnitType */
 	/*
@@ -135,45 +161,59 @@ CUnitTable::~CUnitTable()
 {
 }
 
-void CUnitTable::generateCategorizationFile(std::string &fileName) {
+void CUnitTable::generateCategorizationFile(std::string& fileName) {
+	const std::string modShortName(ai->cb->GetModShortName());
+	const std::string modVersion(ai->cb->GetModVersion());
+	
 	std::ofstream file(fileName.c_str(), std::ios::trunc);
-	file << "# Unit Categorization for E323AI\n\n# Categories to choose from:\n";
-	std::map<unitCategory,std::string>::iterator i;
-	std::map<int, UnitType>::iterator j;
-	UnitType *utParent;
-	for (i = cat2str.begin(); i != cat2str.end(); i++) {
+
+	file << "# Unit basic categorization file for " << AI_NAME << "\n\n";
+	file << "# Based on game " << modShortName << "-" << modVersion << "\n\n";
+	file << "# Autogenerated by " << AI_VERSION << "\n# DO NOT MODIFY!\n\n";
+	file << "# Available categories:\n";
+	
+	for (UnitCategory2StrMap::iterator i = cat2str.begin(); i != cat2str.end(); ++i) {
 		file << "# " << i->second << "\n";
 	}
 	
-	file << "\n\n# " << numUnits << " units in total\n\n";
-	for (j = units.begin(); j != units.end(); j++) {
-		utParent = &(j->second);
+	file << "\n\n# Total number of unit definitions: " << numUnits << "\n\n";
+	
+	for (std::map<int, UnitType>::iterator j = units.begin(); j != units.end(); ++j) {
+		UnitType* utParent = &(j->second);
 		file << "# " << utParent->def->humanName << "\n";
 		file << utParent->def->name;
-		for (unsigned i = 0; i < cats.size(); i++)
-			if (cats[i] & utParent->cats)
+		for (unsigned int i = 0; i < cats.size(); i++)
+			if ((cats[i]&utParent->cats).any())
 				file << "," << cat2str[cats[i]];
 		file << "\n\n";
 	}
+	
 	file.close();
-	LOG_II("Generated categorizations " << fileName)
+	
+	LOG_II("CUnitTable::generateCategorizationFile " << fileName)
 }
 
-void CUnitTable::remove(ARegistrar &object) {
+void CUnitTable::remove(ARegistrar& object) {
 	CUnit *unit = dynamic_cast<CUnit*>(&object);
+	
 	LOG_II("CUnitTable::remove " << (*unit))
+	
 	builders.erase(unit->key);
 	idle.erase(unit->key);
 	metalMakers.erase(unit->key);
 	activeUnits.erase(unit->key);
 	factories.erase(unit->key);
 	defenses.erase(unit->key);
-	unitsAliveTime.erase(unit->key);
 	energyStorages.erase(unit->key);
 	unitsUnderPlayerControl.erase(unit->key);
 	unitsUnderConstruction.erase(unit->key);
 	unitsBuilding.erase(unit->key);
+	staticUnits.erase(unit->key);
+	staticWaterUnits.erase(unit->key);
+	staticEconomyUnits.erase(unit->key);
+
 	unit->unreg(*this);
+	
 	ReusableObjectFactory<CUnit>::Release(unit);
 }
 
@@ -187,41 +227,60 @@ CUnit* CUnitTable::getUnit(int uid) {
 
 CUnit* CUnitTable::requestUnit(int uid, int bid) {
 	CUnit *unit = ReusableObjectFactory<CUnit>::Instance();
+	
 	unit->ai = ai;
 	unit->reset(uid, bid);
 	unit->reg(*this);
+	
+	const unitCategory cats = unit->type->cats;
+
 	if (bid > 0)
 		builders[bid] = false;
+	
 	activeUnits[uid] = unit;
+	
 	idle[bid] = false;
 	idle[uid] = false;
-	if ((unit->type->cats&MOBILE) && bid >= 0) {
-		unit->techlvl = (activeUnits[bid]->type->cats&TECH1) ? TECH1 : unit->techlvl;
-		unit->techlvl = (activeUnits[bid]->type->cats&TECH2) ? TECH2 : unit->techlvl;
-		unit->techlvl = (activeUnits[bid]->type->cats&TECH3) ? TECH3 : unit->techlvl;
-		unit->techlvl = (activeUnits[bid]->type->cats&TECH4) ? TECH4 : unit->techlvl;
-		unit->techlvl = (activeUnits[bid]->type->cats&TECH5) ? TECH5 : unit->techlvl;
+	
+	if ((cats&MOBILE).any() && bid >= 0) {
+		const unitCategory bcats = activeUnits[bid]->type->cats;
+		unit->techlvl = (bcats&TECH1).any() ? TECH1 : unit->techlvl;
+		unit->techlvl = (bcats&TECH2).any() ? TECH2 : unit->techlvl;
+		unit->techlvl = (bcats&TECH3).any() ? TECH3 : unit->techlvl;
+		unit->techlvl = (bcats&TECH4).any() ? TECH4 : unit->techlvl;
+		unit->techlvl = (bcats&TECH5).any() ? TECH5 : unit->techlvl;
 	}
 	// NOTE: remember that NOTA has mobile defenses
-	if (((unit->type->cats&STATIC) && (unit->type->cats&ATTACKER)) || (unit->type->cats&DEFENSE))
+	if (((cats&STATIC).any() && (cats&ATTACKER).any()) || (cats&DEFENSE).any())
 		defenses[unit->key] = unit;
-	if (unit->type->cats&ESTORAGE)
+	if ((cats&ESTORAGE).any())
 		energyStorages[unit->key] = unit;
+	if ((cats&FACTORY).any())
+		factories[unit->key] = unit;
+	if ((cats&MMAKER).any())
+		metalMakers[unit->key] = unit;	
+	if ((cats&STATIC).any()) {
+		staticUnits[unit->key] = unit;
+		if (unit->isEconomy())
+			staticEconomyUnits[unit->key] = unit;
+		if ((cats&(SEA|SUB)).any())
+			staticWaterUnits[unit->key] = unit;
+	}
+	
 	return unit;
 }
 
 void CUnitTable::update() {
-	std::map<int, int>::iterator i;
-	for (i = unitsAliveTime.begin(); i != unitsAliveTime.end(); i++) {
-		/* Makes sure new units are not instantly assigned tasks */
-		if(!activeUnits[i->first]->isMicroing())
-			i->second += MULTIPLEXER;
-	}
-}
+	CUnit* unit;
+	std::map<int, CUnit*>::iterator i;
 
-bool CUnitTable::canPerformTask(CUnit &unit) {
-	/* lifetime of more then 5 seconds */
-	return unitsAliveTime.find(unit.key) != unitsAliveTime.end() && unitsAliveTime[unit.key] > IDLE_UNIT_TIMEOUT;
+	for (i = activeUnits.begin(); i != activeUnits.end(); ++i) {
+		unit = i->second;
+		if (unit->isMicroing())
+			unit->microingFrames += MULTIPLEXER;
+		else
+			unit->aliveFrames += MULTIPLEXER;
+	}
 }
 
 void CUnitTable::buildTechTree() {
@@ -244,17 +303,13 @@ void CUnitTable::buildTechTree() {
 
 		UnitType *utParent, *utChild;
 
-		if (u == units.end()) {
+		if (u == units.end())
 			utParent = insertUnit(ud);
-			MoveData* md = utParent->def->movedata;
-			if (md != NULL)
-				moveTypes[md->pathType] = md;
-		}
 		else
 			utParent = &(u->second);
 
 		buildOptions = ud->buildOptions;
-		for (j = buildOptions.begin(); j != buildOptions.end(); j++) {
+		for (j = buildOptions.begin(); j != buildOptions.end(); ++j) {
 			ud = ai->cb->GetUnitDef(j->second.c_str());
 			u = units.find(ud->id);
 
@@ -263,8 +318,8 @@ void CUnitTable::buildTechTree() {
 			else 
 				utChild = &(u->second);
 
-			utChild->buildBy[utParent->id]  = utParent;
-			utParent->canBuild[utChild->id] = utChild;
+			utChild->buildBy[utParent->def->id]  = utParent;
+			utParent->canBuild[utChild->def->id] = utChild;
 		}
 	}
 
@@ -277,8 +332,8 @@ void CUnitTable::buildTechTree() {
 
 UnitType* CUnitTable::insertUnit(const UnitDef *ud) {
 	UnitType ut;
+
 	ut.def        = ud;
-	ut.id         = ud->id;
 	ut.cost       = ud->metalCost*METAL2ENERGY + ud->energyCost;
 	ut.costMetal  = ud->metalCost;
 	ut.energyMake = ud->energyMake - ud->energyUpkeep;
@@ -286,128 +341,73 @@ UnitType* CUnitTable::insertUnit(const UnitDef *ud) {
 	ut.dps        = calcUnitDps(&ut);
 	units[ud->id] = ut;
 	
+	// also register pathtype...
+	MoveData* md = ud->movedata;
+	if (md)
+		moveTypes[md->pathType] = md;
+	
 	if (maxUnitPower < ut.dps)
 		maxUnitPower = ut.dps;
 
 	return &units[ud->id];
 }
 
-bool CUnitTable::hasAntiAir(const std::vector<UnitDef::UnitDefWeapon> &weapons) {
-	for (unsigned int i = 0; i < weapons.size(); i++) {
-		const UnitDef::UnitDefWeapon *weapon = &weapons[i];
-		/*
-		weapon->def->canAttackGround
-		weapon->def->onlyTargetCategory 
-		*/
-		//FIXME: Still selects Brawlers and Rangers
-		if (weapon->def->tracks && !weapon->def->waterweapon && !weapon->def->stockpile)
-			return true;
-	}
-	return false;
-}
-
-unsigned int CUnitTable::categorizeUnit(UnitType *ut) {
-	const UnitDef *ud = ut->def;
-	unsigned int cats = 0;
+unitCategory CUnitTable::categorizeUnit(UnitType *ut) {
+	const UnitDef* ud = ut->def;
+	unitCategory cats = 0;
 	
+	//assert(ud->name != "armsonar");
+	//assert(ud->humanName != "Shark");
+
 	if (ud->isCommander)
 		cats |= COMMANDER;
 
-	if (ud->speed > 0.0f)
+	if (ud->speed > EPS)
 		cats |= MOBILE;
 	else
 		cats |= STATIC;
 
 	if (ud->canfly)
 		cats |= AIR;
-
-	if (ud->floater || ud->canSubmerge || ud->canhover || ud->minWaterDepth > 0.0f)
+	
+	if (ud->canhover)
 		cats |= SEA;
+	else if (ud->canSubmerge)
+		cats |= SUB;
+	else if (ud->movedata) {
+		if (ud->movedata->subMarine)
+			cats |= SUB; // submarine
+		else if (ud->movedata->moveType == MoveData::Ship_Move) {
+			float heightAboveWater = ai->cb->GetUnitDefHeight(ud->id) - ud->waterline;
+			if (heightAboveWater < EPS)
+				cats |= SUB;
+			else
+				cats |= SEA;
+		}
+		else if (ud->movedata->depth > 100.0f)
+			cats |= SUB; // amphibious unit
+	}
+	else if (ud->floater || ud->waterline > 0.0f || ud->minWaterDepth > 0.0f) {
+		float heightAboveWater;
+		
+		if (ud->waterline > 0.0f || ud->floater)
+			//heightAboveWater = ai->cb->GetUnitDefHeight(ud->id) - ud->waterline;
+			heightAboveWater = ai->cb->GetUnitDefRadius(ud->id) / 2.0f - ud->waterline;
+		else
+			heightAboveWater = -1.0f; // force to SUB
+		
+		if (heightAboveWater < EPS)
+			cats |= SUB;
+		else
+			cats |= SEA;
+	}
 
 	if ((ud->canhover || ud->minWaterDepth < 0.0f) && !ud->canfly)
 		cats |= LAND;
 
-	if (!ud->buildOptions.empty())
-		cats |= BUILDER;
-
 	if (ud->canAssist)
 		cats |= ASSISTER;
 
-	if (!ud->weapons.empty())
-		cats |= ATTACKER;
-
-	if (hasAntiAir(ud->weapons))
-		cats |= ANTIAIR;
-
-/*
-	if (ud->radarRadius > 0)
-		cats |= RADAR;
-
-	if (ud->sonarRadius > 0)
-		cats |= SONAR;
-	
-	if (ud->jammerRadius > 0)
-		cats |= JAMMER;
-*/
-
-	if (ud->canResurrect)
-		cats |= RESURRECTOR;
-
-	// NOTE: we aren't checking for "canMove" becasue it is usually used to set rally point
-	// for factory
-	if (!ud->buildOptions.empty()) {
-		cats |= FACTORY;
-
-		// precise factory type...
-		std::map<int, std::string>::const_iterator j;
-		for (j = ud->buildOptions.begin(); j != ud->buildOptions.end(); ++j) {
-			const UnitDef *canbuild = ai->cb->GetUnitDef(j->second.c_str());
-			
-			if (canbuild->speed < EPS && (cats&FACTORY))
-				// this is a static builder, not a factory
-				cats -= FACTORY; 
-
-			if (canbuild == NULL)
-				continue;
-			
-			if (canbuild->canfly) {
-				cats |= AIRCRAFT;
-				break;
-			}
-			
-			if (canbuild->movedata == NULL) 
-				continue;
-			
-			if (canbuild->movedata->moveFamily == MoveData::KBot &&
-				ud->minWaterDepth < 0.0f) {
-				cats |= KBOT;
-				break;
-			}
-			
-			if (canbuild->movedata->moveFamily == MoveData::Tank &&
-				ud->minWaterDepth < 0.0f) {
-				cats |= VEHICLE;
-				break;
-			}
-			
-			if (canbuild->movedata->moveFamily == MoveData::Hover) {
-				cats |= HOVER;
-				break;
-			}
-
-			if (canbuild->movedata->moveFamily == MoveData::Ship) {
-				cats |= NAVAL;
-				break;
-			}			
-		}
-		
-		// HACK: true
-		if (ud->metalCost < 2000.0f)
-			cats |= TECH1;
-		else
-			cats |= TECH2;
-	}
-
 	if (ud->metalStorage / ut->cost > 0.1f)
 		cats |= MSTORAGE;
 
@@ -417,23 +417,155 @@ unsigned int CUnitTable::categorizeUnit(UnitType *ut) {
 	if (ud->makesMetal >= 0.5f && (ud->energyUpkeep > (ud->makesMetal * 40.0f)))
 		cats |= MMAKER;
 
-	if ((ud->energyMake - ud->energyUpkeep) / ut->cost > 0.002 ||
-		ud->tidalGenerator || ud->windGenerator)
+	if ((ud->energyMake - ud->energyUpkeep) / ut->cost > 0.002 
+	|| ud->tidalGenerator || ud->windGenerator) {
 		cats |= EMAKER;
-	
+		if (ud->tidalGenerator)
+			cats |= TIDAL;
+		if (ud->windGenerator)
+			cats |= WIND;
+	}
+		
 	if (ud->extractsMetal)
 		cats |= MEXTRACTOR;
-	
 /*
+	if (ud->radarRadius > 0)
+		cats |= RADAR;
+
+	if (ud->sonarRadius > 0)
+		cats |= SONAR;
+*/	
+	// NOTE: BA's Dragonfly (transport aircraft) jammer radius = 75
+	if (ud->jammerRadius > 100) {
+		cats |= JAMMER;
+		if ((cats&STATIC).any())
+			cats |= DEFENSE;
+	}
+
+	if (!ud->weapons.empty()) {
+		cats |= ATTACKER;
+
+		if (CUnit::hasTorpedoWeapon(ud->weapons))
+			cats |= TORPEDO;
+		
+		if (CUnit::hasParalyzerWeapon(ud->weapons))
+			cats |= PARALYZER;
+		else if ((cats&AIR).any() && ud->hoverAttack)
+			cats |= ASSAULT;
+
+		/* 0 = only low, 1 = only high, 2 both */
+		if ((cats&AIR).none() && ud->highTrajectoryType >= 1)
+			cats |= ARTILLERY;
+		
+		if (CUnit::hasAntiAirWeapon(ud->weapons))
+			cats |= ANTIAIR;
+		else if (CUnit::hasNukeWeapon(ud->weapons))
+			cats |= NUKE;
+		else if (CUnit::hasInterceptorWeapon(ud->weapons))
+			cats |= ANTINUKE; // TODO: distinguish from EMP
+		else if (CUnit::hasShield(ud->weapons))
+			cats |= SHIELD;
+		
+		if ((cats&STATIC).any() && (cats&NUKE).none())
+			cats |= DEFENSE;
+	}
+	
+	if (ud->canResurrect)
+		cats |= RESURRECTOR;
+
+	// NOTE: we aren't checking for "canMove" because it is usually used 
+	// to set rally point for factory
+	if (!ud->buildOptions.empty()) {
+		int kamikazeUnitCount = 0;
+		std::map<int, std::string>::const_iterator j;
+		
+		cats |= BUILDER;
+		if ((cats&STATIC).any())
+			cats |= FACTORY;
+
+		// preprocessing stage...
+		for (j = ud->buildOptions.begin(); j != ud->buildOptions.end(); ++j) {
+			const UnitDef* canbuild = ai->cb->GetUnitDef(j->second.c_str());
+			
+			if (canbuild == NULL)
+				continue;
+			
+			if (canbuild->canKamikaze)
+				kamikazeUnitCount++;
+			
+			if (canbuild->speed < EPS && (cats&FACTORY).any())
+				// this is a static builder, not a factory
+				cats &= ~FACTORY; 
+		}
+
+		if (kamikazeUnitCount > 4)
+			cats &= ~(FACTORY|BUILDER);
+		
+		if ((cats&FACTORY).any()) {
+			// precise factory type...
+			for (j = ud->buildOptions.begin(); j != ud->buildOptions.end(); ++j) {
+				const UnitDef* canbuild = ai->cb->GetUnitDef(j->second.c_str());
+			
+				if (canbuild == NULL)
+					continue;
+			
+				if (canbuild->canfly) {
+					cats |= AIRCRAFT;
+					break;
+				}
+			
+				if (canbuild->movedata == NULL) 
+					continue;
+			
+				if (canbuild->movedata->moveFamily == MoveData::KBot
+				&& ud->minWaterDepth < 0.0f) {
+					cats |= KBOT;
+					break;
+				}
+			
+				if (canbuild->movedata->moveFamily == MoveData::Tank
+				&& ud->minWaterDepth < 0.0f) {
+					cats |= VEHICLE;
+					break;
+				}
+			
+				if (canbuild->movedata->moveFamily == MoveData::Hover) {
+					cats |= HOVER;
+					break;
+				}
+
+				if (canbuild->movedata->moveFamily == MoveData::Ship) {
+					cats |= NAVAL;
+					break;
+				}			
+			}
+		}
+		
+		/*
+		// TODO: improve heuristic estimator then uncomment the code below
+		if ((cats&BUILDER).any()) {
+			if (ud->metalCost < 2000.0f)
+				cats |= TECH1;
+			else
+				cats |= TECH2;
+		}
+		*/
+	}
+
+	if ((cats&ASSISTER).any() && (cats&(BUILDER|FACTORY)).none()) {
+		// NOTE: default value for "buildDistance" = 128
+		if (ud->buildDistance < 130.0f)
+			cats |= REPAIRPAD;
+		else
+			cats |= NANOTOWER;
+	}
+	
 	if (ud->loadingRadius > 0.0f && ud->transportCapacity > 0)
 		cats |= TRANSPORT;
-*/
 
-	/* 0 = only low, 1 = only high, 2 both */
-	if (ud->highTrajectoryType >= 1)
-		cats |= ARTILLERY;
-
-	if ((cats&ATTACKER) && (cats&MOBILE) && !(cats&BUILDER) && ud->speed >= 50.0f) {
+	/*
+	// TODO: improve heuristic estimator then uncomment the code below
+	if ((cats&ATTACKER).any() && (cats&MOBILE).any() && (cats&BUILDER).none() && ud->speed >= 50.0f) {
 		std::map<int, UnitType*>::iterator i,j;
 		for (i = ut->buildBy.begin(); i != ut->buildBy.end(); ++i) {
 			bool isCheapest = true;
@@ -450,6 +582,7 @@ unsigned int CUnitTable::categorizeUnit(UnitType *ut) {
 			}
 		}
 	}
+	*/
 
 	return cats;
 }
@@ -459,7 +592,19 @@ float CUnitTable::calcUnitDps(UnitType *ut) {
 	return ut->def->power;
 }
 
-int CUnitTable::factoryCount(unsigned int c) {
+int CUnitTable::unitCount(unitCategory c) {
+	int result = 0;
+	std::map<int, CUnit*>::iterator i;
+
+	for (i = activeUnits.begin(); i != activeUnits.end(); ++i) {
+		if ((c&i->second->type->cats) == c)
+			result++;
+	}
+	
+	return result;
+}
+
+int CUnitTable::factoryCount(unitCategory c) {
 	int result = 0;
 	std::map<int, CUnit*>::iterator i;
 
@@ -471,41 +616,44 @@ int CUnitTable::factoryCount(unsigned int c) {
 	return result;
 }
 
-bool CUnitTable::gotFactory(unsigned int c) {
+bool CUnitTable::gotFactory(unitCategory c) {
 	return factoryCount(c) > 0;
 }
 
-void CUnitTable::getBuildables(UnitType *ut, unsigned include, unsigned exclude, std::multimap<float, UnitType*> &candidates) {
-	static const unsigned int envCats = (AIR|SEA|LAND);
-	unsigned int incEnvCats = (envCats&include);
+void CUnitTable::getBuildables(UnitType* ut, unitCategory include, unitCategory exclude, std::multimap<float, UnitType*>& candidates) {
+	if (include.none())
+		return;
+
+	unitCategory incEnvCats = (CATS_ENV&include);
 	std::vector<unitCategory> incCats, excCats;
 	
 	// split categories...
 	for (unsigned int i = 0; i < cats.size(); i++) {
-		if (include&cats[i])
-			incCats.push_back(cats[i]);
-		else if (exclude&cats[i])
+		// NOTE: excluding tags have priority over including tags
+		if ((exclude&cats[i]).any())
 			excCats.push_back(cats[i]);
+		else if ((include&cats[i]).any())
+			incCats.push_back(cats[i]);
 	}
 
 	std::map<int, UnitType*>::iterator j;
 	for (j = ut->canBuild.begin(); j != ut->canBuild.end(); ++j) {
 		bool valid = true;
-		unsigned int cat = j->second->cats;
+		unitCategory cat = j->second->cats;
 		for (unsigned int i = 0; i < incCats.size(); i++) {
 			// NOTE: evironment tags are handled differently: if requested
-			// AIR, LAND & SEA in any combination that means having at least
-			// one match automatically qualifies unit as valid
-			if (incCats[i]&envCats) {
-				if (incEnvCats) {
+			// AIR, LAND, SEA & SUB in any combination that means having 
+			// at least one match automatically qualifies unit as valid
+			if ((incCats[i]&CATS_ENV).any()) {
+				if (incEnvCats.any()) {
 					// filter by environment tags is active
-					if (!(incEnvCats&cat)) {
+					if ((incEnvCats&cat).none()) {
 						valid = false;
 						break;
 					}
 				}
 			}
-			else if (!(incCats[i]&cat)) {
+			else if ((incCats[i]&cat).none()) {
 				valid = false;
 				break;
 			}
@@ -514,7 +662,7 @@ void CUnitTable::getBuildables(UnitType *ut, unsigned include, unsigned exclude,
 		if (valid) {
 			/* Filter out excludes */
 			for (unsigned int i = 0; i < excCats.size(); i++) {
-				if (excCats[i]&cat) {
+				if ((excCats[i]&cat).any()) {
 					valid = false;
 					break;
 				}
@@ -531,7 +679,7 @@ void CUnitTable::getBuildables(UnitType *ut, unsigned include, unsigned exclude,
 		LOG_WW("CUnitTable::getBuildables no candidates found INCLUDE(" << debugCategories(include) << ") EXCLUDE("<<debugCategories(exclude)<<") for unitdef(" << ut->def->humanName << ")")
 }
 
-UnitType* CUnitTable::canBuild(UnitType *ut, unsigned int c) {
+UnitType* CUnitTable::canBuild(UnitType *ut, unitCategory c) {
 	std::map<int, UnitType*>::iterator it;
 	// TODO: make it compatible with environment tags
 	for (it = ut->canBuild.begin(); it != ut->canBuild.end(); it++) {
@@ -544,11 +692,11 @@ UnitType* CUnitTable::canBuild(UnitType *ut, unsigned int c) {
 	return NULL;
 }
 
-CUnit* CUnitTable::getUnitByDef(std::map<int, CUnit*> &dic, const UnitDef *udef) {
+CUnit* CUnitTable::getUnitByDef(std::map<int, CUnit*>& dic, const UnitDef *udef) {
 	return CUnitTable::getUnitByDef(dic, udef->id);
 }
 
-CUnit* CUnitTable::getUnitByDef(std::map<int, CUnit*> &dic, int did) {
+CUnit* CUnitTable::getUnitByDef(std::map<int, CUnit*>& dic, int did) {
 	CUnit* unit;
 	std::map<int, CUnit*>::const_iterator i;
 	for(i = dic.begin(); i != dic.end(); i++) {
@@ -560,9 +708,9 @@ CUnit* CUnitTable::getUnitByDef(std::map<int, CUnit*> &dic, int did) {
 	return NULL;
 }
 
-UnitType* CUnitTable::getUnitTypeByCats(unsigned int c) {
+UnitType* CUnitTable::getUnitTypeByCats(unitCategory c) {
 	std::map<int, UnitType>::iterator it;
-	for (it = units.begin(); it != units.end(); it++) {
+	for (it = units.begin(); it != units.end(); ++it) {
 		if ((it->second.cats&c) == c)
 			return &(it->second);
 	}
@@ -574,7 +722,7 @@ int CUnitTable::setOnOff(std::map<int, CUnit*>& list, bool value) {
 	std::map<int, CUnit*>::iterator i;
 	
 	for (i = list.begin(); i != list.end(); ++i) {
-		CUnit *unit = i->second;
+		CUnit* unit = i->second;
 		if (value != unit->isOn()) {
 			unit->setOnOff(value);
 			result++;
@@ -584,27 +732,27 @@ int CUnitTable::setOnOff(std::map<int, CUnit*>& list, bool value) {
 	return result;
 }
 
-std::string CUnitTable::debugCategories(unsigned categories) {
+std::string CUnitTable::debugCategories(const unitCategory& categories) {
 	std::string cats("");
-	std::map<unitCategory, std::string>::iterator i;
-	for (i = cat2str.begin(); i != cat2str.end(); i++) {
-		int v = categories & i->first;
+	UnitCategory2StrMap::iterator i;
+	for (i = cat2str.begin(); i != cat2str.end(); ++i) {
+		unitCategory v = categories & i->first;
 		if (v == i->first)
 			cats += i->second + " | ";
 	}
-	cats = cats.substr(0, cats.length()-3);
+	cats = cats.substr(0, cats.length() - 3);
 	return cats;
 }
 
 std::string CUnitTable::debugCategories(UnitType *ut) {
 	std::string cats("");
-	std::map<unitCategory, std::string>::iterator i;
-	for (i = cat2str.begin(); i != cat2str.end(); i++) {
-		int v = ut->cats & i->first;
+	UnitCategory2StrMap::iterator i;
+	for (i = cat2str.begin(); i != cat2str.end(); ++i) {
+		unitCategory v = ut->cats & i->first;
 		if (v == i->first)
 			cats += i->second + " | ";
 	}
-	cats = cats.substr(0, cats.length()-3);
+	cats = cats.substr(0, cats.length() - 3);
 	return cats;
 }
 
