@@ -4,6 +4,7 @@
 	#define _USE_MATH_DEFINES
 #endif
 #include <cmath>
+#include <cctype> // for isspace
 
 #include "headers/HAIInterface.h"
 #include "Util.hpp"
@@ -11,10 +12,10 @@
 namespace util {
 	std::string GetAbsFileName(IAICallback* cb, const std::string& relFileName, bool readonly) {
 		char dst[2048];
-		sprintf(dst, "%s", relFileName.c_str());
 
-		// get the absolute path to the file
-		// (and create folders along the way)
+		SNPRINTF(dst, sizeof(dst), "%s", relFileName.c_str());
+
+		// get the absolute path to the file (and create folders along the way)
 		if (readonly) {
 			cb->GetValue(AIVAL_LOCATE_FILE_R, dst);
 		} else {
@@ -45,7 +46,7 @@ namespace util {
 	std::string StringStripSpaces(const std::string& s1) {
 		std::string s2; s2.reserve(s1.size());
 
-		for (std::string::const_iterator it = s1.begin(); it != s1.end(); it++) {
+		for (std::string::const_iterator it = s1.begin(); it != s1.end(); ++it) {
 			if (!isspace(*it)) {
 				s2.push_back(*it);
 			}
@@ -54,22 +55,36 @@ namespace util {
 		return s2;
 	}
 
-	int RemoveWhiteSpaceInPlace(std::string &line) {
-		int num = 0;
-		for ( int i = 0, j ; i < line.length() ; ++ i ) {
-			if ( line [i] == ' ' || line[i] == '\t') {
-				for ( j = i + 1; j < line.length ( ) ; ++j )
-				{
-					if ( line [j] != ' ' || line[i] == '\t')
-					break ;
-				}
-				num += (j - i);
-				line = line.erase(i, (j - i));
+	void StringSplit(const std::string& src, char sep, std::vector<std::string>& dest, bool empty) {
+		bool go = true;
+		size_t pos1 = 0, pos2;
+		
+		if (empty) dest.clear();
+		if (src.empty()) return;
+		
+		while (go) {
+			pos2 = src.find(sep, pos1);
+			if (pos2 == std::string::npos) {
+				pos2 = src.size();
+				go = false;
 			}
+			dest.push_back(src.substr(pos1, pos2 - pos1));
+			pos1 = pos2 + 1;
 		}
-		return num;
+	}
+
+	void RemoveWhiteSpaceInPlace(std::string& s) {
+		s.erase(remove_if(s.begin(), s.end(), isspace), s.end());
 	}
 	
+	bool StringContainsChar(const std::string& src, const char c) {
+		for (int i = 0; i < src.length(); i++ ) {
+			if (src[i] == c)
+				return true;
+		}
+		return false;
+	}
+
 	std::string IntToString(int i, const std::string& format) {
 		char buf[64];
 		SNPRINTF(buf, sizeof(buf), format.c_str(), i);
