@@ -11,9 +11,13 @@ struct AssistTask: public ATask {
 	/* Task to assist */
 	ATask *assist;
 
+	int targetAlt;
+	
 	/* overload */
 	void onUpdate();
 	/* overload */
+	bool onValidate();
+	/* overload */
 	void remove();
 	/* overload */
 	void toStream(std::ostream& out) const;
