@@ -3,13 +3,13 @@
 
 #include "../ATask.h"
 
-class UnitType;
+struct UnitType;
 
 struct BuildTask: public ATask {
 	BuildTask(AIClasses *_ai): ATask(_ai) { t = TASK_BUILD; }
 	BuildTask(AIClasses *_ai, buildType build, UnitType *toBuild, CGroup &group, float3 &pos);
 
-	/* build type to string */
+	/* Build type to string */
 	static std::map<buildType, std::string> buildStr;
 	
 	bool building;
@@ -18,7 +18,7 @@ struct BuildTask: public ATask {
 	/* The ETA in frames */
 	unsigned int eta;
 	/* The UnitType to build */
-	UnitType *toBuild;
+	UnitType* toBuild;
 
 	/* overload */
 	void onUpdate();
@@ -26,6 +26,8 @@ struct BuildTask: public ATask {
 	bool onValidate();
 	/* overload */
 	void toStream(std::ostream& out) const;
+	/* overload */
+	void onUnitDestroyed(int uid, int attacker);
 
 	bool assistable(CGroup &group, float &travelTime) const;
 };
