@@ -11,11 +11,11 @@
 // gives it (STUCK_COUNTER_LIMIT - STUCK_COUNTER_MANEUVER_LIMIT) * 60 seconds to move (longer than UNIT_STUCK_MOVE_DISTANCE (7 sec))
 #define UNIT_STUCK_COUNTER_LIMIT 15
 // stuck maneuver distance should be the movement map res since threat is not really relevant when maneuvering from a stuck pos
-#define UNIT_STUCK_MANEUVER_DISTANCE (THREATRES * 8)
+#define UNIT_STUCK_MANEUVER_DISTANCE (THREATRES * SQUARE_SIZE)
 // the max amount of difference in height there may be at the position to maneuver to (don't retreat into a hole)
 #define UNIT_MAX_MANEUVER_HEIGHT_DIFFERENCE_UP 20
 // minimum of offset between my maxrange and the enemys position before considering moving
-#define UNIT_MIN_MANEUVER_RANGE_DELTA (THREATRES * 8)
+#define UNIT_MIN_MANEUVER_RANGE_DELTA (THREATRES * SQUARE_SIZE)
 // minimum amount to maneuver when getting to max range
 #define UNIT_MIN_MANEUVER_DISTANCE (THREATRES * 4)
 // second requirement, minimum percentage of my range to move when getting to max range (too stop slow long range from dancing around)
@@ -24,7 +24,7 @@
 #define UNIT_MIN_MANEUVER_TIME 15
 
 #define UNIT_DESTINATION_SLACK (THREATRES * 4.0f * 1.4f)
-#define GROUP_DESTINATION_SLACK THREATRES * 8
+#define GROUP_DESTINATION_SLACK THREATRES * SQUARE_SIZE
 #define GROUP_MEDIAN_UNIT_SELECTION_SLACK 10.0f
 
 
@@ -198,16 +198,6 @@ int CAttackGroup::PopStuckUnit() {
 		CUNIT* unit = ai->GetUnit(*it);
 
 		if (unit->stuckCounter > UNIT_STUCK_COUNTER_LIMIT) {
-			// mark it
-			/*
-			SNPRINTF(logMsg, logMsg_maxSize,
-					"stuck %i: %i, dropping from group: %i. isMoving = %i",
-					id, unit->stuckCounter, groupID, isMoving);
-			PRINTF("%s", logMsg);
-			SNPRINTF(logMsg, logMsg_maxSize, "humanName: %s",
-					unit->def()->humanName.c_str());
-			PRINTF("%s", logMsg);
-			*/
 			unit->stuckCounter = 0;
 			units.erase(it);
 			return unit->uid;
@@ -302,15 +292,6 @@ void CAttackGroup::AssignTarget(std::vector<float3> path, float3 position, float
 
 // attack routine (the "find new enemy" part)
 void CAttackGroup::FindDefenseTarget(float3 groupPosition, int frameNr) {
-	/*
-	static const unsigned int logMsg_maxSize = 512;
-	char logMsg[logMsg_maxSize];
-	SNPRINTF(logMsg, logMsg_maxSize,
-			"AG: FindDefenseTarget(), group %i, frame %i, numUnits %i",
-			this->groupID, frameNr, this->units.size());
-	PRINTF("%s", logMsg);
-	*/
-
 	// KLOOTNOTE: numEnemies will be zero if no enemies in LOS or radar when
 	// non-ccb callback used, rely on AttackHandler to pick "global" targets
 	// and on this function for "local" ones
@@ -357,8 +338,7 @@ void CAttackGroup::FindDefenseTarget(float3 groupPosition, int frameNr) {
 			isMoving = true;
 			this->pathIterator = 0;
 		}
-	}
-	else {
+	} else {
 		// attempt to path back to base if there are no targets
 		// KLOOTNOTE: this branch is now purposely never taken
 		// (might not be the best idea to leave units lingering
@@ -367,10 +347,11 @@ void CAttackGroup::FindDefenseTarget(float3 groupPosition, int frameNr) {
 
         pathToTarget.clear();
 		float3 closestBaseSpot = ai->ah->GetClosestBaseSpot(groupPosition);
-		float costToTarget = ai->pather->FindBestPathToRadius(pathToTarget, groupPosition, THREATRES * 8, closestBaseSpot);
+
+		const float costToTarget = ai->pather->FindBestPathToRadius(pathToTarget, groupPosition, THREATRES * SQUARE_SIZE, closestBaseSpot);
 
 		// TODO: GetKBaseMeans() for support of multiple islands/movetypes
-		// TODO: this doesn't need to be to radius
+		// TODO: this doesn't need to be radius
 
 		if (costToTarget == 0 && pathToTarget.size() <= 2) {
 			isMoving = false;
@@ -488,85 +469,84 @@ int CAttackGroup::SelectEnemy(int numEnemies, const float3& groupPos) {
 
 
 void CAttackGroup::AttackEnemy(int enemySelected, int numUnits, float range, int frameSpread) {
-	float3 enemyPos = ai->ccb->GetUnitPos(ai->unitIDs[enemySelected]);
+	const float3& enemyPos = ai->ccb->GetUnitPos(ai->unitIDs[enemySelected]);
+
 	assert(CloakedFix(ai->unitIDs[enemySelected]));
 	isShooting = true;
 
 	assert(numUnits >= 0);
+
 	for (unsigned int i = 0; i < (unsigned int)numUnits; i++) {
 		CUNIT* unit = ai->GetUnit(units[i]);
 		const UnitDef* udef = ai->cb->GetUnitDef(unit->uid);
 
-		// does our unit exist and is it not currently maneuvering?
-		if (udef != NULL && (unit->maneuverCounter-- <= 0)) {
-			// TODO: add a routine finding best (not just closest) target
-			// TODO: in some cases, force-fire on position
-			// TODO: add canAttack
-			unit->Attack(ai->unitIDs[enemySelected]);
-
-			// TODO: this should be the max-range of the lowest-ranged weapon
-			// the unit has assuming you want to rush in with the heavy stuff
-			assert(range >= ai->cb->GetUnitMaxRange(unit->uid));
-
-			// SINGLE UNIT MANEUVERING: testing the possibility of retreating to max
-			// range if target is too close, EXCEPT FOR FLAMETHROWER-EQUIPPED units
-			float3 myPos = ai->cb->GetUnitPos(unit->uid);
-
-			const float maxRange = ai->ut->GetMaxRange(udef);
-			const float losDiff = (maxRange - udef->losRadius);
-		//	const float myRange = (losDiff > 0.0f)? (maxRange + udef->losRadius) * 0.5f: maxRange;
-			const float myRange = (losDiff > 0.0f)? maxRange * 0.75f: maxRange;
-
-			const bool b5 = udef->canfly;
-			const bool b6 = myPos.y < (ai->cb->GetElevation(myPos.x, myPos.z) + 25);
-			const bool b7 = (myRange - UNIT_MIN_MANEUVER_RANGE_DELTA) > myPos.distance2D(enemyPos);
-
-			// is it air, or air that's landed
-			if (!b5 || (b6 && b7)) {
-				bool debug1 = true;
-				bool debug2 = false;
-
-				std::vector<float3> tempPath;
-
-				// note 1: we don't need a path, just a position
-				// note 2: should avoid other immediate friendly units and/or immediate enemy units + radius
-				// maybe include the height parameter in the search? probably not possible
-				// doesn't this mean pathing might happen every second? outer limit should harsher than inner
-				float3 unitPos = ai->ccb->GetUnitPos(ai->unitIDs[enemySelected]);
-				float dist = ai->pather->FindBestPathToRadius(tempPath, myPos, myRange, unitPos);
-
-				if (tempPath.size() > 0) {
-					float3 moveHere = tempPath.back();
-					dist = myPos.distance2D(moveHere);
-
-					// TODO: Penetrators are now broken
-					// is the position between the proposed destination and the
-					// enemy higher than the average of mine and his height?
-					float v1 = ((moveHere.y + enemyPos.y) / 2.0f) + UNIT_MAX_MANEUVER_HEIGHT_DIFFERENCE_UP;
-					float v2 = ai->cb->GetElevation((moveHere.x + enemyPos.x) / 2, (moveHere.z + enemyPos.z) / 2);
-					bool losHack = v1 > v2;
-					float a = (float) UNIT_MIN_MANEUVER_TIME / frameSpread;
-					float b = (dist / unit->def()->speed);
-					float c = ceilf(std::max(a, b));
-
-					// assume the pathfinder returns correct Y values
-					// REMEMBER that this will suck for planes
-					if (dist > std::max((UNIT_MIN_MANEUVER_RANGE_PERCENTAGE * myRange), float(UNIT_MIN_MANEUVER_DISTANCE)) && losHack) {
-						debug2 = true;
-						unit->maneuverCounter = int(c);
-						unit->Move(moveHere);
-					}
-				}
-				if (debug1 && !debug2) {
-					// pathfinder run but path not used?
-				}
-			}
-			else if (!udef->canfly || myPos.y < (ai->cb->GetElevation(myPos.x, myPos.z) + 25)) {
-				// this unit is an air unit
-			}
+		if (udef == NULL || unit->maneuverCounter-- > 0) {
+			// our unit does not exist (?) or is it currently maneuvering
+			continue;
 		}
-		else {
-			// OUR unit is dead?
+
+		// TODO: add a routine finding best (not just closest) target
+		// TODO: in some cases, force-fire on position
+		// TODO: add canAttack
+		unit->Attack(ai->unitIDs[enemySelected]);
+
+		// TODO: this should be the max-range of the lowest-ranged weapon
+		// the unit has assuming you want to rush in with the heavy stuff
+		assert(range >= ai->cb->GetUnitMaxRange(unit->uid));
+
+		// SINGLE UNIT MANEUVERING: testing the possibility of retreating to max
+		// range if target is too close, EXCEPT FOR FLAMETHROWER-EQUIPPED units
+		float3 myPos = ai->cb->GetUnitPos(unit->uid);
+
+		const float maxRange = ai->ut->GetMaxRange(udef);
+		const float losDiff = (maxRange - udef->losRadius);
+	//	const float myRange = (losDiff > 0.0f)? (maxRange + udef->losRadius) * 0.5f: maxRange;
+		const float myRange = (losDiff > 0.0f)? maxRange * 0.75f: maxRange;
+
+		const bool b6 = (myPos.y < (ai->cb->GetElevation(myPos.x, myPos.z) + 25.0f));
+		const bool b7 = (myRange - UNIT_MIN_MANEUVER_RANGE_DELTA) > myPos.distance2D(enemyPos);
+
+		if (!udef->canfly || (b6 && b7)) {
+			std::vector<float3> tempPath;
+
+			// note 1: we don't need a path, just a position
+			// note 2: should avoid other immediate friendly units and/or immediate enemy units + radius
+			// maybe include the height parameter in the search? probably not possible
+			// doesn't this mean pathing might happen every second? outer limit should harsher than inner
+			const float3 unitPos = ai->ccb->GetUnitPos(ai->unitIDs[enemySelected]);
+
+
+			//!! TODO:
+			//!!     for deterministic reproduction, should be able to
+			//!!     set the random seed for the RNG's to a fixed value
+			ai->pather->FindBestPathToRadius(tempPath, myPos, myRange, unitPos);
+
+			if (!tempPath.empty()) {
+				const float3& moveHere = tempPath.back();
+				const float dist = myPos.distance2D(moveHere);
+
+				// TODO: Penetrators are now broken
+				// is the position between the proposed destination and the
+				// enemy higher than the average of mine and his height?
+				const float v1 = ((moveHere.y + enemyPos.y) * 0.5f) + UNIT_MAX_MANEUVER_HEIGHT_DIFFERENCE_UP;
+				const float v2 = ai->cb->GetElevation((moveHere.x + enemyPos.x) * 0.5f, (moveHere.z + enemyPos.z) * 0.5f);
+
+				if (v1 <= v2) {
+					// nope
+					continue;
+				}
+
+				const float a = float(UNIT_MIN_MANEUVER_TIME) / frameSpread;
+				const float b = (dist / unit->def()->speed);
+				const float c = ceilf(std::max(a, b));
+
+				// assume the pathfinder returns correct Y values
+				// REMEMBER that this will suck for planes
+				if (dist > std::max((UNIT_MIN_MANEUVER_RANGE_PERCENTAGE * myRange), float(UNIT_MIN_MANEUVER_DISTANCE))) {
+					unit->maneuverCounter = int(c);
+					unit->Move(moveHere);
+				}
+			}
 		}
 	}
 }
@@ -582,13 +562,15 @@ void CAttackGroup::MoveAlongPath(float3& groupPosition, int numUnits) {
 	int pathMaxIndex = (int) pathToTarget.size() - 1;
 	int step1 = std::min(pathIterator + maxStepsAhead / 2, pathMaxIndex);
 	int step2 = std::min(pathIterator + maxStepsAhead, pathMaxIndex);
-	float3 moveToHereFirst = pathToTarget[step1];
-	float3 moveToHere = pathToTarget[step2];
+
+	const float3& moveToHereFirst = pathToTarget[step1];
+	const float3& moveToHere = pathToTarget[step2];
 
 	// if we aren't there yet
 	if (groupPosition.distance2D(pathToTarget[pathMaxIndex]) > GROUP_DESTINATION_SLACK) {
 		// TODO: give a group the order instead of each unit
 		assert(numUnits >= 0);
+
 		for (unsigned int i = 0; i < (unsigned int)numUnits; i++) {
 			CUNIT* unit = ai->GetUnit(units[i]);
 
