@@ -95,13 +95,17 @@ void CUnitHandler::IdleUnitUpdate(int frame) {
 						float3 pos = ai->cb->GetUnitPos(builderID);
 
 						std::stringstream msg;
-							msg << "[CUnitHandler::IdleUnitUpdate()] frame " << frame << "\n";
+							msg << "[CUnitHandler::IdleUnitUpdate()][frame=" << frame << "]\n";
 							msg << "\tfailed to verify order for builder " << builderID;
 							msg << " with " << (myCommands->size()) << " remaining commands\n";
-						L(ai, msg.str());
+						ai->GetLogger()->Log(msg.str());
 
 						ClearOrder(*i, false);
 
+						// due to the Legacy AI Wrapper inner workings,
+						// the command-queue we fetched above is now invalid,
+						// because it got changed in ClearOrder()
+						myCommands = ai->cb->GetCurrentUnitCommands(builderID);
 						if (!myCommands->empty()) {
 							DecodeOrder(*i, true);
 						} else {
@@ -361,10 +365,10 @@ void CUnitHandler::ClearOrder(BuilderTracker* builderTracker, bool reportError)
 		BuildTask* buildTask = GetBuildTask(buildTaskID);
 
 		std::stringstream msg;
-			msg << "[CUnitHandler::ClearOrder()] frame " << frame << "\n";
+			msg << "[CUnitHandler::ClearOrder()][frame=" << frame << "]\n";
 			msg << "\tbuilder " << builderID << " is reported idle but";
 			msg << " still has a build-task with ID " << (buildTaskID) << "\n";
-		L(ai, msg.str());
+		ai->GetLogger()->Log(msg.str());
 
 		if (buildTask->builderTrackers.size() > 1) {
 			BuildTaskRemove(builderTracker);
@@ -382,10 +386,10 @@ void CUnitHandler::ClearOrder(BuilderTracker* builderTracker, bool reportError)
 		const std::string& taskName = taskPlan->def->humanName;
 
 		std::stringstream msg;
-			msg << "[CUnitHandler::ClearOrder()] frame " << frame << "\n";
+			msg << "[CUnitHandler::ClearOrder()][frame=" << frame << "]\n";
 			msg << "\tbuilder " << builderID << " is reported idle but";
 			msg << " still has a task-plan named \"" << (taskName) << "\"\n";
-		L(ai, msg.str());
+		ai->GetLogger()->Log(msg.str());
 
 		// mask this build-spot as bad
 		ai->dm->MaskBadBuildSpot(taskPlan->pos);
@@ -409,10 +413,10 @@ void CUnitHandler::ClearOrder(BuilderTracker* builderTracker, bool reportError)
 		hit = true;
 
 		std::stringstream msg;
-			msg << "[CUnitHandler::ClearOrder()] frame " << frame << "\n";
+			msg << "[CUnitHandler::ClearOrder()][frame=" << frame << "]\n";
 			msg << "\tbuilder " << builderID << " is reported idle but";
 			msg << " still has a factory ID of " << factoryID << "\n";
-		L(ai, msg.str());
+		ai->GetLogger()->Log(msg.str());
 
 		// remove the builder from its job
 		FactoryBuilderRemove(builderTracker);
@@ -442,11 +446,11 @@ void CUnitHandler::DecodeOrder(BuilderTracker* builderTracker, bool reportError)
 
 		if (reportError) {
 			std::stringstream msg;
-				msg << "[CUnitHandler::DecodeOrder()] frame " << frame << "\n";
+				msg << "[CUnitHandler::DecodeOrder()][frame=" << frame << "]\n";
 				msg << "\tbuilder " << builderID << " claimed idle, but has";
 				msg << " command " << cID << " with " << n << " parameters";
 				msg << " (params[0]: " << ((n > 0)? c->params[0]: -1) << ")\n";
-			L(ai, msg.str());
+			ai->GetLogger()->Log(msg.str());
 		}
 
 		if (cID < 0) {
@@ -528,9 +532,9 @@ void CUnitHandler::DecodeOrder(BuilderTracker* builderTracker, bool reportError)
 		// empty?
 		// assert(false);
 		std::stringstream msg;
-			msg << "[CUnitHandler::DecodeOrder()] frame " << frame << "\n";
+			msg << "[CUnitHandler::DecodeOrder()][frame=" << frame << "]\n";
 			msg << "\tbuilder " << builderID << " should not have an empty queue!\n";
-		L(ai, msg.str());
+		ai->GetLogger()->Log(msg.str());
 	}
 }
 
@@ -649,9 +653,9 @@ void CUnitHandler::BuildTaskCreate(int id) {
 			// to restart a dead building, or a human has taken control),
 			// make one anyway
 			std::stringstream msg;
-				msg << "[CUnitHandler::BuildTaskCreate()] frame " << (ai->cb->GetCurrentFrame()) << "\n";
+				msg << "[CUnitHandler::BuildTaskCreate()][frame=" << ai->cb->GetCurrentFrame() << "]\n";
 				msg << "\tBuildTask Creation Error for task with ID " << id << "\n";
-			L(ai, msg.str());
+			ai->GetLogger()->Log(msg.str());
 
 			if (category == CAT_DEFENCE) {
 				ai->dm->AddDefense(pos, newUnitDef);
@@ -708,7 +712,7 @@ void CUnitHandler::BuildTaskCreate(int id) {
 						msg.str("");
 							msg << "\tadded builder " << builderTracker->builderID << " to";
 							msg << " build-task with ID " << builderTracker->buildTaskId << "\n";
-						L(ai, msg.str());
+						ai->GetLogger()->Log(msg.str());
 					}
 				}
 			}
@@ -883,11 +887,11 @@ bool CUnitHandler::BuildTaskAddBuilder(int builderID, UnitCategory category) {
 			// (and so should b5 in most of the *A mods)
 
 			std::stringstream msg;
-				msg << "[CUnitHandler::BuildTaskAddBuilder()] frame " << frame << "\n";
+				msg << "[CUnitHandler::BuildTaskAddBuilder()][frame=" << frame << "]\n";
 				msg << "\tbuilder " << builderID << " not able to be added to CAT_FACTORY build-task\n";
 				msg << "\tb1: " << b1 << ", b2: " << b2 << ", b3: " << b3;
 				msg << ", b4: " << b4 << ", b5: " << b5;
-			L(ai, msg.str());
+			ai->GetLogger()->Log(msg.str());
 		}
 
 		return false;
@@ -952,9 +956,9 @@ bool CUnitHandler::BuildTaskAddBuilder(int builderID, UnitCategory category) {
 
 	if (b5) {
 		std::stringstream msg;
-			msg << "[CUnitHandler::BuildTaskAddBuilder()] frame " << frame << "\n";
+			msg << "[CUnitHandler::BuildTaskAddBuilder()][frame=" << frame << "]\n";
 			msg << "\tno joinable CAT_FACTORY build-tasks or task-plans for builder " << builderID;
-		L(ai, msg.str());
+		ai->GetLogger()->Log(msg.str());
 	}
 
 	return false;
@@ -990,10 +994,10 @@ void CUnitHandler::TaskPlanCreate(int builder, float3 pos, const UnitDef* builtd
 				TaskPlanAdd(&*i, builderTracker);
 			} else {
 				std::stringstream msg;
-					msg << "[CUnitHandler::TaskPlanCreate()] frame " << ai->cb->GetCurrentFrame() << "\n";
+					msg << "[CUnitHandler::TaskPlanCreate()][frame=" << ai->cb->GetCurrentFrame() << "]\n";
 					msg << "\ttask-plan for \"" << builtdef->humanName << "\" already present";
 					msg << " at position <" << pos.x << ", " << pos.y << ", " << pos.z << ">\n";
-				L(ai, msg.str());
+				ai->GetLogger()->Log(msg.str());
 			}
 		}
 	}
@@ -1347,10 +1351,10 @@ void CUnitHandler::UpdateUpgradeTasks(int frame) {
 					// give a build order for the replacement structure
 					if (cq->size() == 0 || ((cq->front()).id != -task->newBuildingDef->id)) {
 						std::stringstream msg;
-							msg << "[CUnitHandler::UpdateUpgradeTasks()] frame " << frame << "\n";
+							msg << "[CUnitHandler::UpdateUpgradeTasks()][frame=" << frame << "]\n";
 							msg << "\tgiving build order for \"" << task->newBuildingDef->humanName;
 							msg << "\" to builder " << builderID << "\n";
-						L(ai, msg.str());
+						ai->GetLogger()->Log(msg.str());
 
 						ai->GetUnit(builderID)->Build_ClosestSite(task->newBuildingDef, task->oldBuildingPos);
 					}
@@ -1358,10 +1362,10 @@ void CUnitHandler::UpdateUpgradeTasks(int frame) {
 					// give a reclaim order for the original structure
 					if (cq->size() == 0 || ((cq->front()).id != CMD_RECLAIM)) {
 						std::stringstream msg;
-							msg << "[CUnitHandler::UpdateUpgradeTasks()] frame " << frame << "\n";
+							msg << "[CUnitHandler::UpdateUpgradeTasks()][frame=" << frame << "]\n";
 							msg << "\tgiving reclaim order for \"" << ai->cb->GetUnitDef(oldBuildingID)->humanName;
 							msg << "\" to builder " << builderID << "\n";
-						L(ai, msg.str());
+						ai->GetLogger()->Log(msg.str());
 
 						ai->GetUnit(builderID)->Reclaim(oldBuildingID);
 					}
