@@ -1,8 +1,9 @@
 #include "BuilderPlacement.h"
-#include "Sim/Units/UnitDef.h"
+#include "LegacyCpp/UnitDef.h"
 //#include "ExternalAI/IAICallback.h"
 #include <set>
 #include "Sim/MoveTypes/MoveInfo.h"
+#include "Sim/Units/CommandAI/CommandQueue.h"
 #include <time.h>
 
 ResourceSiteExtBO::ResourceSiteExtBO(sRAIUnitDef* UDR )
@@ -103,7 +104,7 @@ cBuilderPlacement::cBuilderPlacement(IAICallback* callback, cRAI* global)
 	*l<<"\nLoading Build-Placement ...";
 	const double SecondAlgorithmTimeLimit = 2.5; // seconds per RAI
 
-	double clockStart = clock();
+	//double clockStart = clock();
 
 	if( int(G->Units.size()) == 0 )
 		*l<<"\nERROR: G->Units";
