@@ -84,7 +84,7 @@ GlobalResourceMap::GlobalResourceMap(IAICallback* _cb, cLogFile* l, GlobalTerrai
 		return;
 	}
 
-	int *fList;
+	int *fList = NULL;
 	int fSize = 0;
 	if( UDGeoResource )
 	{
@@ -104,7 +104,6 @@ GlobalResourceMap::GlobalResourceMap(IAICallback* _cb, cLogFile* l, GlobalTerrai
 	relResourceFileName = "cache/" + cRAI::MakeFileSystemCompatible(cb->GetModHumanName());
 	relResourceFileName += "-" + IntToString(cb->GetModHash(), "%x");
 	relResourceFileName += "-" + cRAI::MakeFileSystemCompatible(cb->GetMapName());
-	relResourceFileName.resize(relResourceFileName.size() - 4); // cut off extension
 	relResourceFileName += "-" + IntToString(cb->GetMapHash(), "%x");
 	relResourceFileName += ".res";
 
@@ -115,6 +114,11 @@ GlobalResourceMap::GlobalResourceMap(IAICallback* _cb, cLogFile* l, GlobalTerrai
 		resourceFile_r = fopen(resourceFileName_r.c_str(), "rb");
 	}
 
+	// get absolute file name
+	if (!cRAI::LocateFile(cb, relResourceFileName, resourceFileName_w, true)) {
+		throw 12;
+	}
+
 	bool useResourceFile = false;
 	if( resourceFile_r )
 	{
@@ -713,14 +717,7 @@ GlobalResourceMap::~GlobalResourceMap()
 {
 	if( saveResourceFile )
 	{
-		string resourceFileName_w;
-		FILE* resourceFile_w = NULL;
-		// get absolute file name
-		if (cRAI::LocateFile(cb, relResourceFileName, resourceFileName_w, true)) {
-			resourceFile_w = fopen(resourceFileName_w.c_str(), "wb");
-		} else {
-			throw 12;
-		}
+		FILE* resourceFile_w = fopen(resourceFileName_w.c_str(), "wb");
 
 		int size;
 		fwrite(&(size=saveUD.size()), sizeof(int), 1, resourceFile_w);
