@@ -68,6 +68,42 @@ static inline int getResourceId_Energy(const SSkirmishAICallback* sAICallback, i
 	return resIndEnergy;
 }
 
+static inline void copyShortToUCharArray(const short* src, unsigned char* const dst, const size_t size) {
+
+	int i;
+	for (i = 0; i < size; ++i) {
+		dst[i] = (unsigned char) src[i];
+	}
+}
+static inline void copyIntToUShortArray(const int* src, unsigned short* const dst, const size_t size) {
+
+	int i;
+	for (i = 0; i < size; ++i) {
+		dst[i] = (unsigned short) src[i];
+	}
+}
+
+// FIXME: group ID's have no runtime bound
+const int maxGroups = MAX_UNITS;
+
+// FIXME: these are far too generous, but we
+// don't have easy access to the right values
+// on the AI side (so better waste memory than
+// risk SEGVs)
+static const int numUnitDefs = MAX_UNITS; // unitDefHandler->numUnitDefs;
+static const int numFeatDefs = MAX_UNITS; // featureHandler->numFeatureDefs;
+static const int numWeapDefs = MAX_UNITS; // weaponDefHandler->numWeaponDefs;
+
+
+size_t CAIAICallback::numClbInstances = 0;
+float* CAIAICallback::heightMap = NULL;
+float* CAIAICallback::cornersHeightMap = NULL;
+float* CAIAICallback::slopeMap = NULL;
+unsigned short* CAIAICallback::losMap = NULL;
+unsigned short* CAIAICallback::radarMap = NULL;
+unsigned short* CAIAICallback::jammerMap = NULL;
+unsigned char* CAIAICallback::metalMap = NULL;
+
 
 CAIAICallback::CAIAICallback()
 	: IAICallback(), teamId(-1), sAICallback(NULL) {
@@ -79,47 +115,95 @@ CAIAICallback::CAIAICallback(int teamId, const SSkirmishAICallback* sAICallback)
 	init();
 }
 
+CAIAICallback::~CAIAICallback() {
+
+	numClbInstances--;
+
+	for (int i=0; i < numWeapDefs; ++i) {
+		delete weaponDefs[i];
+		weaponDefs[i] = NULL;
+	}
+	delete weaponDefs;
+	weaponDefs = NULL;
+	delete weaponDefFrames;
+	weaponDefFrames = NULL;
+
+	for (int i=0; i < numUnitDefs; ++i) {
+		delete unitDefs[i];
+		unitDefs[i] = NULL;
+	}
+	delete unitDefs;
+	unitDefs = NULL;
+	delete unitDefFrames;
+	unitDefFrames = NULL;
+
+	for (int i=0; i < numFeatDefs; ++i) {
+		delete featureDefs[i];
+		featureDefs[i] = NULL;
+	}
+	delete featureDefs;
+	featureDefs = NULL;
+	delete featureDefFrames;
+	featureDefFrames = NULL;
+
+	for (int i=0; i < maxGroups; ++i) {
+		delete groupPossibleCommands[i];
+		groupPossibleCommands[i] = NULL;
+	}
+	delete groupPossibleCommands;
+	groupPossibleCommands = NULL;
+
+	for (int i=0; i < MAX_UNITS; ++i) {
+		delete unitPossibleCommands[i];
+		unitPossibleCommands[i] = NULL;
+	}
+	delete unitPossibleCommands;
+	unitPossibleCommands = NULL;
+
+	for (int i=0; i < MAX_UNITS; ++i) {
+		delete unitCurrentCommandQueues[i];
+		unitCurrentCommandQueues[i] = NULL;
+	}
+	delete unitCurrentCommandQueues;
+	unitCurrentCommandQueues = NULL;
+
+	if (numClbInstances == 0) {
+		delete[] heightMap; heightMap = NULL;
+		delete[] cornersHeightMap; cornersHeightMap = NULL;
+		delete[] slopeMap; slopeMap = NULL;
+		delete[] losMap; losMap = NULL;
+		delete[] radarMap; radarMap = NULL;
+		delete[] jammerMap; jammerMap = NULL;
+		delete[] metalMap; metalMap = NULL;
+	}
+}
+
 
 void CAIAICallback::init() {
-	// FIXME: group ID's have no runtime bound
-	const int maxGroups = MAX_UNITS;
 
-	// FIXME: these are far too generous, but we
-	// don't have easy access to the right values
-	// on the AI side (so better waste memory than
-	// risk SEGVs)
-	const int numUnitDefs = MAX_UNITS; // unitDefHandler->numUnitDefs;
-	const int numFeatDefs = MAX_UNITS; // featureHandler->numFeatureDefs;
-	const int numWeapDefs = MAX_UNITS; // weaponDefHandler->numWeaponDefs;
+	numClbInstances++;
 
 	weaponDefs      = new WeaponDef*[numWeapDefs];
 	weaponDefFrames = new int[numWeapDefs];
-
 	fillWithNULL((void**) weaponDefs, numWeapDefs);
 	fillWithMinusOne(weaponDefFrames, numWeapDefs);
 
-
 	unitDefs      = new UnitDef*[numUnitDefs];
 	unitDefFrames = new int[numUnitDefs];
-
 	fillWithNULL((void**) unitDefs, numUnitDefs);
 	fillWithMinusOne(unitDefFrames, numUnitDefs);
 
+	featureDefs      = new FeatureDef*[numFeatDefs];
+	featureDefFrames = new int[numFeatDefs];
+	fillWithNULL((void**) featureDefs, numFeatDefs);
+	fillWithMinusOne(featureDefFrames, numFeatDefs);
 
 	groupPossibleCommands    = new std::vector<CommandDescription>*[maxGroups];
 	unitPossibleCommands     = new std::vector<CommandDescription>*[MAX_UNITS];
 	unitCurrentCommandQueues = new CCommandQueue*[MAX_UNITS];
-
 	fillWithNULL((void**) groupPossibleCommands,    maxGroups);
 	fillWithNULL((void**) unitPossibleCommands,     MAX_UNITS);
 	fillWithNULL((void**) unitCurrentCommandQueues, MAX_UNITS);
-
-
-	featureDefs      = new FeatureDef*[numFeatDefs];
-	featureDefFrames = new int[numFeatDefs];
-
-	fillWithNULL((void**) featureDefs, numFeatDefs);
-	fillWithMinusOne(featureDefFrames, numFeatDefs);
 }
 
 
@@ -143,12 +227,52 @@ int CAIAICallback::GetPlayerTeam(int player) {
 	return sAICallback->Clb_Game_getPlayerTeam(teamId, player);
 }
 
-const char* CAIAICallback::GetTeamSide(int team) {
-	return sAICallback->Clb_Game_getTeamSide(teamId, team);
+const char* CAIAICallback::GetTeamSide(int otherTeamId) {
+	return sAICallback->Clb_Game_getTeamSide(teamId, otherTeamId);
 }
 
-int CAIAICallback::GetTeamAllyTeam(int team) {
-	return sAICallback->Clb_Game_getTeamAllyTeam(teamId, team);
+int CAIAICallback::GetTeamAllyTeam(int otherTeamId) {
+	return sAICallback->Clb_Game_getTeamAllyTeam(teamId, otherTeamId);
+}
+
+float CAIAICallback::GetTeamMetalCurrent(int otherTeamId) {
+	static int m = getResourceId_Metal(sAICallback, teamId);
+	return sAICallback->Clb_Game_getTeamResourceCurrent(teamId, otherTeamId, m);
+}
+
+float CAIAICallback::GetTeamMetalIncome(int otherTeamId) {
+	static int m = getResourceId_Metal(sAICallback, teamId);
+	return sAICallback->Clb_Game_getTeamResourceIncome(teamId, otherTeamId, m);
+}
+
+float CAIAICallback::GetTeamMetalUsage(int otherTeamId) {
+	static int m = getResourceId_Metal(sAICallback, teamId);
+	return sAICallback->Clb_Game_getTeamResourceUsage(teamId, otherTeamId, m);
+}
+
+float CAIAICallback::GetTeamMetalStorage(int otherTeamId) {
+	static int m = getResourceId_Metal(sAICallback, teamId);
+	return sAICallback->Clb_Game_getTeamResourceStorage(teamId, otherTeamId, m);
+}
+
+float CAIAICallback::GetTeamEnergyCurrent(int otherTeamId) {
+	static int e = getResourceId_Energy(sAICallback, teamId);
+	return sAICallback->Clb_Game_getTeamResourceCurrent(teamId, otherTeamId, e);
+}
+
+float CAIAICallback::GetTeamEnergyIncome(int otherTeamId) {
+	static int e = getResourceId_Energy(sAICallback, teamId);
+	return sAICallback->Clb_Game_getTeamResourceIncome(teamId, otherTeamId, e);
+}
+
+float CAIAICallback::GetTeamEnergyUsage(int otherTeamId) {
+	static int e = getResourceId_Energy(sAICallback, teamId);
+	return sAICallback->Clb_Game_getTeamResourceUsage(teamId, otherTeamId, e);
+}
+
+float CAIAICallback::GetTeamEnergyStorage(int otherTeamId) {
+	static int e = getResourceId_Energy(sAICallback, teamId);
+	return sAICallback->Clb_Game_getTeamResourceStorage(teamId, otherTeamId, e);
 }
 
 bool CAIAICallback::IsAllied(int firstAllyTeamId, int secondAllyTeamId) {
@@ -404,7 +528,6 @@ const UnitDef* CAIAICallback::GetUnitDefById(int unitDefId) {
 		unitDef->turnInPlace = sAICallback->Clb_UnitDef_isTurnInPlace(teamId, unitDefId);
 		unitDef->upright = sAICallback->Clb_UnitDef_isUpright(teamId, unitDefId);
 		unitDef->collide = sAICallback->Clb_UnitDef_isCollide(teamId, unitDefId);
-		unitDef->controlRadius = sAICallback->Clb_UnitDef_getControlRadius(teamId, unitDefId);
 		unitDef->losRadius = sAICallback->Clb_UnitDef_getLosRadius(teamId, unitDefId);
 		unitDef->airLosRadius = sAICallback->Clb_UnitDef_getAirLosRadius(teamId, unitDefId);
 		unitDef->losHeight = sAICallback->Clb_UnitDef_getLosHeight(teamId, unitDefId);
@@ -682,8 +805,6 @@ int CAIAICallback::GetMapHeight() {
 
 const float* CAIAICallback::GetHeightMap() {
 
-	static float* heightMap = NULL;
-
 	if (heightMap == NULL) {
 		int size = sAICallback->Clb_Map_0ARRAY1SIZE0getHeightMap(teamId);
 		heightMap = new float[size]; // NOTE: memory leak, but will be used till end of the game anyway
@@ -695,8 +816,6 @@ const float* CAIAICallback::GetHeightMap() {
 
 const float* CAIAICallback::GetCornersHeightMap() {
 
-	static float* cornersHeightMap = NULL;
-
 	if (cornersHeightMap == NULL) {
 		int size = sAICallback->Clb_Map_0ARRAY1SIZE0getCornersHeightMap(teamId);
 		cornersHeightMap = new float[size]; // NOTE: memory leak, but will be used till end of the game anyway
@@ -716,8 +835,6 @@ float CAIAICallback::GetMaxHeight() {
 
 const float* CAIAICallback::GetSlopeMap() {
 
-	static float* slopeMap = NULL;
-
 	if (slopeMap == NULL) {
 		int size = sAICallback->Clb_Map_0ARRAY1SIZE0getSlopeMap(teamId);
 		slopeMap = new float[size]; // NOTE: memory leak, but will be used till end of the game anyway
@@ -729,8 +846,6 @@ const float* CAIAICallback::GetSlopeMap() {
 
 const unsigned short* CAIAICallback::GetLosMap() {
 
-	static unsigned short* losMap = NULL;
-
 	if (losMap == NULL) {
 		int size = sAICallback->Clb_Map_0ARRAY1SIZE0getLosMap(teamId);
 		losMap = new unsigned short[size]; // NOTE: memory leak, but will be used till end of the game anyway
@@ -750,8 +865,6 @@ int CAIAICallback::GetLosMapResolution() {
 
 const unsigned short* CAIAICallback::GetRadarMap() {
 
-	static unsigned short* radarMap = NULL;
-
 	if (radarMap == NULL) {
 		int size = sAICallback->Clb_Map_0ARRAY1SIZE0getRadarMap(teamId);
 		radarMap = new unsigned short[size]; // NOTE: memory leak, but will be used till end of the game anyway
@@ -763,8 +876,6 @@ const unsigned short* CAIAICallback::GetRadarMap() {
 
 const unsigned short* CAIAICallback::GetJammerMap() {
 
-	static unsigned short* jammerMap = NULL;
-
 	if (jammerMap == NULL) {
 		int size = sAICallback->Clb_Map_0ARRAY1SIZE0getJammerMap(teamId);
 		jammerMap = new unsigned short[size]; // NOTE: memory leak, but will be used till end of the game anyway
@@ -776,7 +887,6 @@ const unsigned short* CAIAICallback::GetJammerMap() {
 
 const unsigned char* CAIAICallback::GetMetalMap() {
 
-	static unsigned char* metalMap = NULL;
 	static int m = getResourceId_Metal(sAICallback, teamId);
 
 	if (metalMap == NULL) {
@@ -1154,6 +1264,8 @@ void CAIAICallback::GetUnitDefList(const UnitDef** list) {
 	for (int i = 0; i < size; ++i) {
 		list[i] = this->GetUnitDefById(unitDefIds[i]);
 	}
+
+	delete[] unitDefIds;
 }
 
 float CAIAICallback::GetUnitDefHeight(int def) {
@@ -1686,6 +1798,24 @@ int CAIAICallback::HandleCommand(int commandId, void* data) {
 			break;
 		}
 
+		case AIHCFeatureTraceRayId: {
+			AIHCFeatureTraceRay* cppCmdData = (AIHCFeatureTraceRay*) data;
+			SFeatureTraceRayCommand cCmdData = {
+				cppCmdData->rayPos.toSAIFloat3(),
+				cppCmdData->rayDir.toSAIFloat3(),
+				cppCmdData->rayLen,
+				cppCmdData->srcUID,
+				cppCmdData->hitFID,
+				cppCmdData->flags
+			};
+
+			ret = sAICallback->Clb_Engine_handleCommand(teamId, COMMAND_TO_ID_ENGINE, -1, COMMAND_TRACE_RAY_FEATURE, &cCmdData);
+
+			cppCmdData->rayLen = cCmdData.rayLen;
+			cppCmdData->hitFID = cCmdData.hitFID;
+			break;
+		}
+
 		case AIHCPauseId: {
 			AIHCPause* cppCmdData = (AIHCPause*) data;
 			SPauseCommand cCmdData = {
