@@ -20,7 +20,7 @@
 
 #include "IAICheats.h"
 
-class SSkirmishAICallback;
+struct SSkirmishAICallback;
 class CAIAICallback;
 
 /**
@@ -72,7 +72,6 @@ public:
 	bool IsUnitCloaked(int unitid);
 	bool IsUnitParalyzed(int unitid);
 
-	bool OnlyPassiveCheats();
 	void EnableCheatEvents(bool enable);
 
 	bool GetProperty(int id, int property, void* dst);
@@ -83,7 +82,8 @@ private:
 	int teamId;
 	const SSkirmishAICallback* sAICallback;
 	CAIAICallback* aiCallback;
-	void setCheatsEnabled(bool enable);
+
+	void SetCheatsEnabled(bool enable);
 };
 
 #endif // _AIAICHEATS_H
