@@ -5,6 +5,7 @@
 
 use strict;
 use File::Basename;
+use File::Copy;
 use Cwd 'abs_path';
 use File::Spec::Functions;
 
@@ -86,12 +87,11 @@ $nsisDefines="$nsisDefines -DVERSION_TAG=\"$tag\"";
 # Download some files to be included in the installer
 system("sh", "installer/springlobby_download.sh");
 chdir("$installerDir/downloads");
+system("wget", "-N", "http://springrts.com/dl/rapid-spring-latest-win32.7z");
 system("wget", "-N", "http://springrts.com/dl/TASServer.jar");
-system("wget", "-N", "http://springrts.com/dl/SelectionEditor.zip");
-system("mkdir", "-p", "./SelectionEditor/");
-system("unzip", "-o", "SelectionEditor.zip", "-d", "./SelectionEditor/");
 system("wget", "-N", "http://www.springlobby.info/installer/springsettings.exe");
-system("wget", "-N", "http://files.caspring.org/caupdater/SpringDownloader.exe");
+system("wget", "-N", "http://zero-k.info/lobby/setup.exe");
+system("wget", "-N", "http://zero-k.info/lobby/setup_icon.ico");
 chdir("$installerDir/..");
 
 
