@@ -14,6 +14,7 @@
 #include "Map/MapDamage.h"
 #include "Map/ReadMap.h"
 #include "Rendering/Env/BaseWater.h"
+#include "Rendering/Models/3DModel.h"
 #include "Rendering/GroundDecalHandler.h"
 #include "Sim/Features/Feature.h"
 #include "Sim/Misc/CollisionHandler.h"
@@ -83,9 +84,29 @@ void CGameHelper::DoExplosionDamage(CUnit* unit,
 	// position of its collision volume and "unit radius" by
 	// the volume's minimally-bounding sphere
 	//
-	float3 dif = (unit->midPos + unit->collisionVolume->GetOffsets()) - expPos;
-	const float volRad = unit->collisionVolume->GetBoundingRadius();
-	const float expDist = std::max(dif.Length(), volRad + 0.1f);
+	const int damageFrame = unit->lastAttackedPieceFrame;
+	const LocalModelPiece* piece = unit->lastAttackedPiece;
+	const CollisionVolume* volume = NULL;
+
+	float3 basePos;
+	float3 diffPos;
+
+	if (piece != NULL && unit->unitDef->usePieceCollisionVolumes && damageFrame == gs->frameNum) {
+		volume = piece->colvol;
+		basePos = piece->GetPos() + volume->GetOffsets();
+		basePos = unit->pos + 
+			unit->rightdir * basePos.x +
+			unit->updir    * basePos.y +
+			unit->frontdir * basePos.z;
+	} else {
+		volume = unit->collisionVolume;
+		basePos = unit->midPos + volume->GetOffsets();
+	}
+
+	diffPos = basePos - expPos;
+
+	const float volRad = volume->GetBoundingRadius();
+	const float expDist = std::max(diffPos.Length(), volRad + 0.1f);
 
 	// expDist2 is the distance from the boundary of the
 	// _volume's_ minimally-bounding sphere (!) to the
@@ -105,7 +126,7 @@ void CGameHelper::DoExplosionDamage(CUnit* unit,
 	// that should not be touched)
 	// Clamp expDist to radius to prevent division by zero
 	// (expDist2 can never be > radius). We still need the
-	// original expDist later to normalize dif.
+	// original expDist later to normalize diffPos.
 	// expDist2 _can_ exceed radius when explosion is eg.
 	// on shield surface: in that case don't do any damage
 	float expDist2 = expDist - volRad;
@@ -123,134 +144,138 @@ void CGameHelper::DoExplosionDamage(CUnit* unit,
 
 	float mod  = (expRad - expDist1) / (expRad - expDist1 * edgeEffectiveness);
 	float mod2 = (expRad - expDist2) / (expRad - expDist2 * edgeEffectiveness);
-	dif /= expDist;
-	dif.y += 0.12f;
+
+	diffPos /= expDist;
+	diffPos.y += 0.12f;
 
 	if (mod < 0.01f)
 		mod = 0.01f;
 
-	DamageArray damageDone = damages * mod2;
-	float3 addedImpulse = dif * (damages.impulseFactor * mod * (damages[0] + damages.impulseBoost) * 3.2f);
+	const DamageArray damageDone = damages * mod2;
+	const float3 addedImpulse = diffPos * (damages.impulseFactor * mod * (damages[0] + damages.impulseBoost) * 3.2f);
 
-	if (expDist2 < (expSpeed * 4.0f)) { //damage directly
+	if (expDist2 < (expSpeed * 4.0f)) { // damage directly
 		unit->DoDamage(damageDone, owner, addedImpulse, weaponId);
-	} else { //damage later
+	} else { // damage later
 		WaitingDamage* wd = new WaitingDamage((owner? owner->id: -1), unit->id, damageDone, addedImpulse, weaponId);
 		waitingDamages[(gs->frameNum + int(expDist2 / expSpeed) - 3) & 127].push_front(wd);
 	}
 }
 
 void CGameHelper::DoExplosionDamage(CFeature* feature,
-	const float3& expPos, float expRad, CUnit* owner, const DamageArray& damages)
+	const float3& expPos, float expRad, const DamageArray& damages)
 {
-	CollisionVolume* cv = feature->collisionVolume;
+	const CollisionVolume* cv = feature->collisionVolume;
 
 	if (cv) {
-		float3 dif = (feature->midPos + cv->GetOffsets()) - expPos;
+		const float3 dif = (feature->midPos + cv->GetOffsets()) - expPos;
+
 		float expDist = std::max(dif.Length(), 0.1f);
 		float expMod = (expRad - expDist) / expRad;
+		float dmgScale = (damages[0] + damages.impulseBoost);
 
 		// always do some damage with explosive stuff
 		// (DDM wreckage etc. is too big to normally
 		// be damaged otherwise, even by BB shells)
 		// NOTE: this will also be only approximate
 		// for non-spherical volumes
-		if ((expRad > 8.0f) && (expDist < (cv->GetBoundingRadius() * 1.1f)) && (expMod < 0.1f)) {
+		if ((expRad > SQUARE_SIZE) && (expDist < (cv->GetBoundingRadius() * 1.1f)) && (expMod < 0.1f)) {
 			expMod = 0.1f;
 		}
+
 		if (expMod > 0.0f) {
-			feature->DoDamage(damages * expMod, owner,
-				dif * (damages.impulseFactor * expMod / expDist *
-				(damages[0] + damages.impulseBoost)));
+			feature->DoDamage(damages * expMod, dif * (damages.impulseFactor * expMod / expDist * dmgScale));
 		}
 	}
 }
 
 
+
 void CGameHelper::Explosion(
 	float3 expPos, const DamageArray& damages,
 	float expRad, float edgeEffectiveness,
 	float expSpeed, CUnit* owner,
 	bool damageGround, float gfxMod,
 	bool ignoreOwner, bool impactOnly,
-	CExplosionGenerator* explosionGraphics, CUnit* hit,
+	CExplosionGenerator* explosionGraphics,
+	CUnit* hitUnit,
 	const float3& impactDir, int weaponId,
-	CFeature* hitfeature
+	CFeature* hitFeature
 ) {
 	if (luaUI) {
-		if ((weaponId >= 0) && (weaponId < weaponDefHandler->numWeaponDefs)) {
-			WeaponDef& wd = weaponDefHandler->weaponDefs[weaponId];
-			const float cameraShake = wd.cameraShake;
-			if (cameraShake > 0.0f) {
-				luaUI->ShockFront(cameraShake, expPos, expRad);
-			}
+		const WeaponDef* wd = weaponDefHandler->GetWeaponById(weaponId);
+
+		if (wd != NULL && wd->cameraShake > 0.0f) {
+			luaUI->ShockFront(wd->cameraShake, expPos, expRad);
 		}
 	}
 
-	bool noGfx = eventHandler.Explosion(weaponId, expPos, owner);
+	const bool noGfx = eventHandler.Explosion(weaponId, expPos, owner);
 
 #ifdef TRACE_SYNC
 	tracefile << "Explosion: ";
 	tracefile << expPos.x << " " << damages[0] <<  " " << expRad << "\n";
 #endif
 
-	float h2 = ground->GetHeight2(expPos.x, expPos.z);
+	const float h2 = ground->GetHeight2(expPos.x, expPos.z);
+
 	expPos.y = std::max(expPos.y, h2);
 	expRad = std::max(expRad, 1.0f);
 
 	if (impactOnly) {
-		if (hit) {
-			DoExplosionDamage(hit, expPos, expRad, expSpeed, ignoreOwner, owner, edgeEffectiveness, damages, weaponId);
-		} else if (hitfeature) {
-			DoExplosionDamage(hitfeature, expPos, expRad, owner, damages);
+		if (hitUnit) {
+			DoExplosionDamage(hitUnit, expPos, expRad, expSpeed, ignoreOwner, owner, edgeEffectiveness, damages, weaponId);
+		} else if (hitFeature) {
+			DoExplosionDamage(hitFeature, expPos, expRad, damages);
 		}
 	} else {
 		float height = std::max(expPos.y - h2, 0.0f);
 
-		// damage all units within the explosion radius
-		vector<CUnit*> units = qf->GetUnitsExact(expPos, expRad);
-		vector<CUnit*>::iterator ui;
-		bool hitUnitDamaged = false;
+		{
+			// damage all units within the explosion radius
+			const vector<CUnit*>& units = qf->GetUnitsExact(expPos, expRad);
+			bool hitUnitDamaged = false;
 
-		for (ui = units.begin(); ui != units.end(); ++ui) {
-			CUnit* unit = *ui;
+			for (vector<CUnit*>::const_iterator ui = units.begin(); ui != units.end(); ++ui) {
+				CUnit* unit = *ui;
 
-			if (unit == hit) {
-				hitUnitDamaged = true;
+				if (unit == hitUnit) {
+					hitUnitDamaged = true;
+				}
+
+				DoExplosionDamage(unit, expPos, expRad, expSpeed, ignoreOwner, owner, edgeEffectiveness, damages, weaponId);
 			}
 
-			DoExplosionDamage(unit, expPos, expRad, expSpeed, ignoreOwner, owner, edgeEffectiveness, damages, weaponId);
-		}
-
-		// HACK: for a unit with an offset coldet volume, the explosion
-		// (from an impacting projectile) position might not correspond
-		// to its quadfield position so we need to damage it separately
-		if (hit && !hitUnitDamaged) {
-			DoExplosionDamage(hit, expPos, expRad, expSpeed, ignoreOwner, owner, edgeEffectiveness, damages, weaponId);
-		}
-
-
-		// damage all features within the explosion radius
-		vector<CFeature*> features = qf->GetFeaturesExact(expPos, expRad);
-		vector<CFeature*>::iterator fi;
-		bool hitFeatureDamaged = false;
-
-		for (fi = features.begin(); fi != features.end(); ++fi) {
-			CFeature* feature = *fi;
-
-			if (hitfeature == feature) {
-				hitFeatureDamaged = true;
+			// HACK: for a unit with an offset coldet volume, the explosion
+			// (from an impacting projectile) position might not correspond
+			// to its quadfield position so we need to damage it separately
+			if (hitUnit != NULL && !hitUnitDamaged) {
+				DoExplosionDamage(hitUnit, expPos, expRad, expSpeed, ignoreOwner, owner, edgeEffectiveness, damages, weaponId);
 			}
-			DoExplosionDamage(feature, expPos, expRad, owner, damages);
 		}
 
-		if (hitfeature && !hitFeatureDamaged) {
-			DoExplosionDamage(hitfeature, expPos, expRad, owner, damages);
+		{
+			// damage all features within the explosion radius
+			const vector<CFeature*>& features = qf->GetFeaturesExact(expPos, expRad);
+			bool hitFeatureDamaged = false;
+
+			for (vector<CFeature*>::const_iterator fi = features.begin(); fi != features.end(); ++fi) {
+				CFeature* feature = *fi;
+
+				if (feature == hitFeature) {
+					hitFeatureDamaged = true;
+				}
+
+				DoExplosionDamage(feature, expPos, expRad, damages);
+			}
+
+			if (hitFeature != NULL && !hitFeatureDamaged) {
+				DoExplosionDamage(hitFeature, expPos, expRad, damages);
+			}
 		}
 
 		// deform the map
-		if (damageGround && !mapDamage->disabled &&
-		    (expRad > height) && (damages.craterMult > 0.0f)) {
+		if (damageGround && !mapDamage->disabled && (expRad > height) && (damages.craterMult > 0.0f)) {
 			float damage = damages[0] * (1.0f - (height / expRad));
 			if (damage > (expRad * 10.0f)) {
 				damage = expRad * 10.0f; // limit the depth somewhat
@@ -264,19 +289,20 @@ void CGameHelper::Explosion(
 		if (!explosionGraphics) {
 			explosionGraphics = stdExplosionGenerator;
 		}
-		explosionGraphics->Explosion(expPos, damages[0], expRad, owner, gfxMod, hit, impactDir);
+		explosionGraphics->Explosion(expPos, damages[0], expRad, owner, gfxMod, hitUnit, impactDir);
 	}
 
 	groundDecals->AddExplosion(expPos, damages[0], expRad);
-
 	water->AddExplosion(expPos, damages[0], expRad);
 }
 
+
+
 //////////////////////////////////////////////////////////////////////
 // Raytracing
 //////////////////////////////////////////////////////////////////////
 
-// called by {CRifle, CBeamLaser, CLightningCannon}::Fire()
+// called by {CRifle, CBeamLaser, CLightningCannon}::Fire() and Skirmish AIs
 float CGameHelper::TraceRay(const float3& start, const float3& dir, float length, float /*power*/,
 			    const CUnit* owner, const CUnit*& hit, int collisionFlags,
 			    const CFeature** hitfeature)
@@ -1063,22 +1089,22 @@ float3 CGameHelper::GetUnitErrorPos(const CUnit* unit, int allyteam)
 }
 
 
-void CGameHelper::BuggerOff(float3 pos, float radius, bool spherical, CUnit* exclude)
+void CGameHelper::BuggerOff(float3 pos, float radius, bool spherical, bool forced, int teamId, CUnit* excludeUnit)
 {
-	std::vector<CUnit*> units = qf->GetUnitsExact(pos, radius + 8, spherical);
+	const std::vector<CUnit*> &units = qf->GetUnitsExact(pos, radius + SQUARE_SIZE, spherical);
+	const int allyTeamId = teamHandler->AllyTeam(teamId);
 
-	for (std::vector<CUnit*>::iterator ui = units.begin(); ui != units.end(); ++ui) {
+	for (std::vector<CUnit*>::const_iterator ui = units.begin(); ui != units.end(); ++ui) {
 		CUnit* u = *ui;
-		bool allied = true;
 
-		if (exclude) {
-			const int eAllyTeam = exclude->allyteam;
-			const int uAllyTeam = u->allyteam;
-			allied = (teamHandler->Ally(uAllyTeam, eAllyTeam) || teamHandler->Ally(eAllyTeam, uAllyTeam));
-		}
+		// don't send BuggerOff commands to enemy units
+		const int uAllyTeamId = u->allyteam;
+		const bool allied = (
+				teamHandler->Ally(uAllyTeamId,  allyTeamId) ||
+				teamHandler->Ally(allyTeamId, uAllyTeamId));
 
-		if (u != exclude && allied && !u->unitDef->pushResistant && !u->usingScriptMoveType) {
-			u->commandAI->BuggerOff(pos, radius + 8);
+		if ((u != excludeUnit) && allied && ((!u->unitDef->pushResistant && !u->usingScriptMoveType) || forced)) {
+			u->commandAI->BuggerOff(pos, radius + SQUARE_SIZE);
 		}
 	}
 }
