@@ -72,9 +72,6 @@ const unsigned SYNCCHECK_TIMEOUT = 300;
 /// used to prevent msg spam
 const unsigned SYNCCHECK_MSG_TIMEOUT = 400;
 
-///msecs to wait until the game starts after all players are ready
-const spring_duration gameStartDelay = spring_secs(4);
-
 /// The time intervall in msec for sending player statistics to each client
 const spring_duration playerInfoTime = spring_secs(2);
 
@@ -106,9 +103,9 @@ void SetBoolArg(bool& value, const std::string& str)
 }
 
 
-CGameServer* gameServer=0;
+CGameServer* gameServer = 0;
 
-CGameServer::CGameServer(int hostport, bool onlyLocal, const GameData* const newGameData, const CGameSetup* const mysetup)
+CGameServer::CGameServer(const std::string& hostIP, int hostPort, const GameData* const newGameData, const CGameSetup* const mysetup)
 : setup(mysetup)
 {
 	assert(setup);
@@ -152,17 +149,25 @@ CGameServer::CGameServer(int hostport, bool onlyLocal, const GameData* const new
 	allowAdditionalPlayers = configHandler->Get("AllowAdditionalPlayers", false);
 	whiteListAdditionalPlayers = configHandler->Get("WhiteListAdditionalPlayers", true);
 
-	if (!onlyLocal)
-		UDPNet.reset(new netcode::UDPListener(hostport));
+	if (!setup->onlyLocal) {
+		UDPNet.reset(new netcode::UDPListener(hostPort, hostIP));
+	}
+
+	std::string autohostip = configHandler->Get("AutohostIP", std::string("127.0.0.1"));
+	if (StringToLower(autohostip) == "localhost") {
+		// FIXME temporary hack: we do not support (host-)names.
+		// "localhost" was the only name supported in the past.
+		// added 7. January 2011, to be removed in ~ 1 year
+		autohostip = "127.0.0.1";
+	}
+	const int autohostport = configHandler->Get("AutohostPort", 0);
 
-	std::string autohostip = configHandler->Get("AutohostIP", std::string("localhost"));
-	int autohostport = configHandler->Get("AutohostPort", 0);
-	
 	if (autohostport > 0) {
 		AddAutohostInterface(autohostip, autohostport);
 	}
+
 	rng.Seed(newGameData->GetSetup().length());
-	Message(str( format(ServerStart) %hostport), false);
+	Message(str(format(ServerStart) %hostPort), false);
 
 	lastTick = spring_gettime();
 
@@ -243,7 +248,7 @@ CGameServer::~CGameServer()
 	// TODO: move this to a method in CTeamHandler
 	// Figure out who won the game.
 	int numTeams = (int)setup->teamStartingData.size();
-	if (setup->useLuaGaia) {
+	if (setup->useLuaGaia && (numTeams > 0)) {
 		--numTeams;
 	}
 	int winner = -1;
@@ -276,13 +281,22 @@ void CGameServer::AddLocalClient(const std::string& myName, const std::string& m
 	localClientNumber = BindConnection(myName, "", myVersion, true, boost::shared_ptr<netcode::CConnection>(new netcode::CLocalConnection()));
 }
 
-void CGameServer::AddAutohostInterface(const std::string& autohostip, const int remotePort)
+void CGameServer::AddAutohostInterface(const std::string& autohostIP, const int autohostPort)
 {
-	if (!hostif)
-	{
-		hostif.reset(new AutohostInterface(autohostip, remotePort));
-		hostif->SendStart();
-		Message(str(format(ConnectAutohost) %remotePort), false);
+	if (!hostif) {
+		hostif.reset(new AutohostInterface(autohostIP, autohostPort));
+		if (hostif->IsInitialized()) {
+			hostif->SendStart();
+			Message(str(format(ConnectAutohost) %autohostPort), false);
+		} else {
+			// Quit if we are instructed to communicate with an auto-host,
+			// but are unable to do so. As we do not want an auto-host running
+			// a spring game that he has no control over. If we get here,
+			// it suggests a configuration problem in the auto-host.
+			hostif.reset();
+			Message(str(format(ConnectAutohostFailed) %autohostIP %autohostPort), false);
+			quitServer = true;
+		}
 	}
 }
 
@@ -1279,7 +1293,7 @@ void CGameServer::ProcessPacket(const unsigned playernum, boost::shared_ptr<cons
 						}
 						break;
 					}
-					case TEAMMSG_TEAM_DIED: { // don't send to clients, they don't need it
+					case TEAMMSG_TEAM_DIED: {
 						const unsigned team = inbuf[3];
 #ifndef DEDICATED
 						if (players[player].isLocal) // currently only host is allowed
@@ -1296,6 +1310,7 @@ void CGameServer::ProcessPacket(const unsigned playernum, boost::shared_ptr<cons
 									//players[p].team = 0;
 									players[p].spectator = true;
 									if (hostif) hostif->SendPlayerDefeated(p);
+									Broadcast(CBaseNetProtocol::Get().SendTeamDied(player, team));
 								}
 							}
 							// The teams Skirmish AIs destruction process
@@ -1466,42 +1481,6 @@ void CGameServer::ProcessPacket(const unsigned playernum, boost::shared_ptr<cons
 			break;
 		}
 
-		case NETMSG_REGISTER_NETMSG: {
-			const unsigned char player = inbuf[1];
-			const unsigned char msg = inbuf[2];
-			MsgToForwardMap::iterator itor = relayingMessagesMap.find( msg );
-
-			if ( itor != relayingMessagesMap.end() ) { // one entry already exists in the map
-				PlayersToForwardMsgvec &toForward = itor->second;
-				if ( toForward.find( player ) == toForward.end() ) {
-					toForward.insert( player );
-				}
-			}
-			else {
-				PlayersToForwardMsgvec toForward;
-				toForward.insert( player );
-				relayingMessagesMap[msg] = toForward;
-			}
-			break;
-		}
-
-		case NETMSG_UNREGISTER_NETMSG: {
-			const unsigned char player = inbuf[1];
-			const unsigned char msg = inbuf[2];
-			MsgToForwardMap::iterator itor = relayingMessagesMap.find( msg );
-			if ( itor == relayingMessagesMap.end() ) { // no entry already exists in the map
-				break;
-			}
-			PlayersToForwardMsgvec& toForward = itor->second;
-			if ( toForward.find( player ) != toForward.end() ) {
-				toForward.erase( player );
-				if ( toForward.size() == 0 ) {
-					relayingMessagesMap.erase( itor );
-				}
-			}
-			break;
-		}
-
 #ifdef SYNCDEBUG
 		case NETMSG_SD_CHKRESPONSE:
 		case NETMSG_SD_BLKRESPONSE:
@@ -1526,17 +1505,6 @@ void CGameServer::ProcessPacket(const unsigned playernum, boost::shared_ptr<cons
 		break;
 	}
 
-	// forward special messages to the players that request them
-	size_t playersSize = players.size();
-	MsgToForwardMap::iterator toRelay = relayingMessagesMap.find( msgCode );
-	if ( toRelay != relayingMessagesMap.end() ) {
-		PlayersToForwardMsgvec& toRelaySet = toRelay->second;
-		for ( PlayersToForwardMsgvec::iterator playerToRelay = toRelaySet.begin(); playerToRelay != toRelaySet.end(); playerToRelay++ ) {
-			if ( *playerToRelay < playersSize ) {
-				players[*playerToRelay].SendData(packet);
-			}
-		}
-	}
 }
 
 void CGameServer::ServerReadNet()
@@ -1553,6 +1521,10 @@ void CGameServer::ServerReadNet()
 				netcode::UnpackPacket msg(packet, 3);
 				std::string name, passwd, version;
 				unsigned char reconnect;
+				unsigned short netversion;
+				msg >> netversion;
+				if(netversion != NETWORK_VERSION)
+					throw netcode::UnpackPacketException("Wrong network version");
 				msg >> name;
 				msg >> passwd;
 				msg >> version;
@@ -1654,22 +1626,26 @@ void CGameServer::CheckForGameStart(bool forced)
 		}
 	}
 
-	if (allReady || forced)
-	{
+	// msecs to wait until the game starts after all players are ready
+	const spring_duration gameStartDelay = spring_secs(setup->gameStartDelay);
+
+	if (allReady || forced) {
 		if (!spring_istime(readyTime)) {
 			readyTime = spring_gettime();
 			rng.Seed(spring_tomsecs(readyTime-serverStartTime));
-			Broadcast(CBaseNetProtocol::Get().SendStartPlaying(spring_tomsecs(gameStartDelay)));
+			// we have to wait at least 1 msec, because 0 is a special case
+			const unsigned countdown = std::max(1, spring_tomsecs(gameStartDelay));
+			Broadcast(CBaseNetProtocol::Get().SendStartPlaying(countdown));
 		}
 	}
-	if (spring_istime(readyTime) && (spring_gettime() - readyTime) > gameStartDelay)
-	{
+	if (spring_istime(readyTime) && ((spring_gettime() - readyTime) > gameStartDelay)) {
 		StartGame();
 	}
 }
 
 void CGameServer::StartGame()
 {
+	assert(!gameHasStarted);
 	gameHasStarted = true;
 	startTime = gameTime;
 	if (!canReconnect && !allowAdditionalPlayers)
@@ -1880,18 +1856,24 @@ void CGameServer::PushAction(const Action& action)
 			}
 		}
 	}
-#ifdef DEDICATED // we already have a quit command in the client
-	else if (action.command == "kill")
-	{
+	else if (action.command == "kill") {
 		quitServer = true;
 	}
-	else if (action.command == "pause")
-	{
-		isPaused = !isPaused;
+	else if (action.command == "pause") {
+		bool newPausedState = isPaused;
+		if (action.extra.empty()) {
+			// if no param is given, toggle paused state
+			newPausedState = !isPaused;
+		} else {
+			// if a param is given, interpret it as "bool setPaused"
+			SetBoolArg(newPausedState, action.extra);
+		}
+		if (newPausedState != isPaused) {
+			// the state changed
+			isPaused = newPausedState;
+		}
 	}
-#endif
-	else
-	{
+	else {
 		// only forward to players (send over network)
 		CommandMessage msg(action, SERVER_PLAYER);
 		Broadcast(boost::shared_ptr<const RawPacket>(msg.Pack()));
@@ -2213,15 +2195,14 @@ unsigned CGameServer::BindConnection(std::string name, const std::string& passwd
 			newPlayer.SendData(*vit); // throw at him all stuff he missed until now
 
 	if (!demoReader || setup->demoName.empty()) { // gamesetup from demo?
-		const unsigned newPlayerTeam = setup->playerStartingData[newPlayerNumber].team;
-		if (!players[newPlayerNumber].spectator && !teams[newPlayerTeam].active) { // create new team
-			players[newPlayerNumber].readyToStart = (setup->startPosType != CGameSetup::StartPos_ChooseInGame);
-			teams[newPlayerTeam].active = true;
-		}
-		players[newPlayerNumber].team = newPlayerTeam;
-
-		if (!players[newPlayerNumber].spectator && !setup->playerStartingData[newPlayerNumber].spectator)
+		if (!newPlayer.spectator) {
+			unsigned newPlayerTeam = newPlayer.team;
+			if (!teams[newPlayerTeam].active) { // create new team
+				newPlayer.readyToStart = (setup->startPosType != CGameSetup::StartPos_ChooseInGame);
+				teams[newPlayerTeam].active = true;
+			}
 			Broadcast(CBaseNetProtocol::Get().SendJoinTeam(newPlayerNumber, newPlayerTeam));
+		}
 	}
 
 	Message(str(format(" -> Connection established (given id %i)") %newPlayerNumber));
@@ -2303,3 +2284,4 @@ void CGameServer::AddToPacketCache(boost::shared_ptr<const netcode::RawPacket> &
 	}
 	packetCache.back().push_back(pckt);
 }
+
