@@ -37,6 +37,7 @@ CGameSetup::CGameSetup()
 	, disableMapDamage(false)
 	, maxSpeed(0.0f)
 	, minSpeed(0.0f)
+	, onlyLocal(false)
 	, hostDemo(false)
 	, numDemoPlayers(0)
 	, gameMode(0)
@@ -381,7 +382,7 @@ bool CGameSetup::Init(const std::string& buf)
 	// Parse
 	TdfParser file(buf.c_str(),buf.size());
 
-	if(!file.SectionExist("GAME"))
+	if (!file.SectionExist("GAME"))
 		return false;
 
 	// Game parameters
@@ -396,6 +397,9 @@ bool CGameSetup::Init(const std::string& buf)
 	demoName    = file.SGetValueDef("",  "GAME\\Demofile");
 	hostDemo    = !demoName.empty();
 
+	file.GetTDef(gameStartDelay, (unsigned int) 4, "GAME\\GameStartDelay");
+
+	file.GetDef(onlyLocal,        "0", "GAME\\OnlyLocal");
 	file.GetDef(useLuaGaia,       "1", "GAME\\ModOptions\\LuaGaia");
 	file.GetDef(gameMode,         "0", "GAME\\ModOptions\\GameMode");
 	file.GetDef(noHelperAIs,      "0", "GAME\\ModOptions\\NoHelperAIs");
