@@ -85,12 +85,22 @@ public:
 	float maxSpeed;
 	float minSpeed;
 
+	/** if true, this is a non-network game (one local client, eg. when watching a demo) */
+	bool onlyLocal;
+
 	bool hostDemo;
 	std::string demoName;
 	int numDemoPlayers;
 
 	std::string saveName;
 
+	/**
+	 * The number of seconds till the game starts,
+	 * counting from the moment when all players are connected and ready.
+	 * Default: 4 (seconds)
+	 */
+	unsigned int gameStartDelay;
+
 	int gameMode;
 	int noHelperAIs;
 
