@@ -180,15 +180,8 @@ void COSCStatsSender::UpdateDestination() {
 	if (network->destination == NULL) {
 		network->destination = new boost::asio::ip::udp::endpoint();
 	}
-	network->destination->address(boost::asio::ip::address::from_string(dstAddress));
+	network->destination->address(netcode::WrapIP(dstAddress));
 	network->destination->port(dstPort);
-
-#ifdef STREFLOP_H
-	//! (date of note: 08/05/10)
-	//! something in from_string() is invalidating the FPU flags
-	//! tested on win2k and linux (not happening there)
-	streflop_init<streflop::Simple>();
-#endif
 }
 
 bool COSCStatsSender::IsTimeToSend(int frameNum) {
