@@ -59,13 +59,10 @@ CPreGame::CPreGame(const ClientSetup* setup) :
 	net = new CNetProtocol();
 	activeController=this;
 
-	if(!settings->isHost)
-	{
-		net->InitClient(settings->hostip.c_str(), settings->hostport, settings->myPlayerName, settings->myPasswd, SpringVersion::GetFull());
+	if (!settings->isHost) {
+		net->InitClient(settings->hostIP.c_str(), settings->hostPort, settings->myPlayerName, settings->myPasswd, SpringVersion::GetFull());
 		timer = SDL_GetTicks();
-	}
-	else
-	{
+	} else {
 		net->InitLocalClient();
 	}
 	ISound::Initialize(); // should have finished until server response arrives
@@ -183,14 +180,14 @@ void CPreGame::StartServer(const std::string& setupscript)
 	// (Which is OK, since unitsync does not have map options available either.)
 	setup->LoadStartPositions();
 
-	const std::string modArchive = archiveScanner->ArchiveFromName(setup->modName);
+	const std::string& modArchive = archiveScanner->ArchiveFromName(setup->modName);
+	const std::string& mapArchive = archiveScanner->ArchiveFromName(setup->mapName);
 	startupData->SetModChecksum(archiveScanner->GetArchiveCompleteChecksum(modArchive));
-	const std::string mapArchive = archiveScanner->ArchiveFromName(setup->mapName);
 	startupData->SetMapChecksum(archiveScanner->GetArchiveCompleteChecksum(mapArchive));
 
 	good_fpu_control_registers("before CGameServer creation");
 	startupData->SetSetup(setup->gameSetupText);
-	gameServer = new CGameServer(settings->hostport, (setup->playerStartingData.size() == 1), startupData, setup);
+	gameServer = new CGameServer(settings->hostIP, settings->hostPort, startupData, setup);
 	delete startupData;
 	gameServer->AddLocalClient(settings->myPlayerName, SpringVersion::GetFull());
 	good_fpu_control_registers("after CGameServer creation");
@@ -234,6 +231,7 @@ void CPreGame::UpdateClientNet()
 					pckt >> spectator;
 					pckt >> team;
 					pckt >> name;
+
 					CPlayer player;
 					player.name = name;
 					player.spectator = spectator;
@@ -334,6 +332,7 @@ void CPreGame::ReadDataFromDemo(const std::string& demoName)
 			tgame->AddPair("MapName", demoScript->mapName);
 			tgame->AddPair("Gametype", demoScript->modName);
 			tgame->AddPair("Demofile", demoName);
+			tgame->AddPair("OnlyLocal", 1);
 
 			for (std::map<std::string, TdfParser::TdfSection*>::iterator it = tgame->sections.begin(); it != tgame->sections.end(); ++it)
 			{
@@ -382,9 +381,11 @@ void CPreGame::ReadDataFromDemo(const std::string& demoName)
 			}
 			logOutput.Print("Starting GameServer");
 			good_fpu_control_registers("before CGameServer creation");
-			gameServer = new CGameServer(settings->hostport, true, data, tempSetup);
+
+			gameServer = new CGameServer(settings->hostIP, settings->hostPort, data, tempSetup);
 			gameServer->AddLocalClient(settings->myPlayerName, SpringVersion::GetFull());
 			delete data;
+
 			good_fpu_control_registers("after CGameServer creation");
 			logOutput.Print("GameServer started");
 			break;
@@ -430,6 +431,18 @@ void CPreGame::GameDataReceived(boost::shared_ptr<const netcode::RawPacket> pack
 		throw content_error("Server sent us incorrect script");
 	}
 
+	// some sanity checks
+	for(int p = 0; p < playerHandler->ActivePlayers(); ++p) {
+		CPlayer *player = playerHandler->Player(p);
+		if(player->playerNum >= playerHandler->ActivePlayers() || player->playerNum < 0)
+			throw content_error("Invalid player in game data");
+		if(player->team >= teamHandler->ActiveTeams() || player->team < 0)
+			throw content_error("Invalid team in game data");
+		int allyteam = teamHandler->AllyTeam(player->team);
+		if(allyteam >= teamHandler->ActiveAllyTeams() || allyteam < 0)
+			throw content_error("Invalid ally team in game data");
+	}
+
 	gs->SetRandSeed(gameData->GetRandomSeed(), true);
 	LogObject() << "Using map " << gameSetup->mapName << "\n";
 
