@@ -872,10 +872,6 @@ int LuaUnsyncedCtrl::SetTeamColor(lua_State* L)
 
 int LuaUnsyncedCtrl::AssignMouseCursor(lua_State* L)
 {
-	if (!CLuaHandle::GetActiveHandle()->GetUserMode()) {
-		return 0;
-	}
-
 	const int args = lua_gettop(L); // number of arguments
 	if ((args < 2) || !lua_isstring(L, 1) || !lua_isstring(L, 2)) {
 		luaL_error(L, "Incorrect arguments to AssignMouseCursor()");
@@ -896,22 +892,19 @@ int LuaUnsyncedCtrl::AssignMouseCursor(lua_State* L)
 		}
 	}
 
-	if (mouse->AssignMouseCursor(cmdName, fileName, hotSpot, overwrite)) {
-		lua_pushboolean(L, true);
-	} else {
-		lua_pushboolean(L, false);
+	const bool worked = mouse->AssignMouseCursor(cmdName, fileName, hotSpot, overwrite);
+
+	if (CLuaHandle::GetActiveHandle()->GetUserMode()) {
+		lua_pushboolean(L, worked);
+		return 1;
 	}
 
-	return 1;
+	return 0;
 }
 
 
 int LuaUnsyncedCtrl::ReplaceMouseCursor(lua_State* L)
 {
-	if (!CLuaHandle::GetActiveHandle()->GetUserMode()) {
-		return 0;
-	}
-
 	const int args = lua_gettop(L); // number of arguments
 	if ((args < 1) || !lua_isstring(L, 1) || !lua_isstring(L, 2)) {
 		luaL_error(L, "Incorrect arguments to ReplaceMouseCursor()");
@@ -927,9 +920,14 @@ int LuaUnsyncedCtrl::ReplaceMouseCursor(lua_State* L)
 		}
 	}
 
-	lua_pushboolean(L, mouse->ReplaceMouseCursor(oldName, newName, hotSpot));
+	const bool worked = mouse->ReplaceMouseCursor(oldName, newName, hotSpot);
 
-	return 1;
+	if (CLuaHandle::GetActiveHandle()->GetUserMode()) {
+		lua_pushboolean(L, worked);
+		return 1;
+	}
+
+	return 0;
 }
 
 /******************************************************************************/
@@ -1540,7 +1538,7 @@ int LuaUnsyncedCtrl::GetConfigInt(lua_State* L)
 	}
 	const string name = luaL_checkstring(L, 1);
 	const int def     = luaL_optint(L, 2, 0);
-	const bool setInOverlay = (bool) luaL_optnumber(L, 3, 0); // default: false
+	const bool setInOverlay = lua_isboolean(L, 3) ? lua_toboolean(L, 3) : false;
 	const int value = configHandler->Get(name, def, setInOverlay);
 	lua_pushnumber(L, value);
 	return 1;
@@ -1554,7 +1552,7 @@ int LuaUnsyncedCtrl::SetConfigInt(lua_State* L)
 	}
 	const string name = luaL_checkstring(L, 1);
 	const int value   = luaL_checkint(L, 2);
-	const bool useOverlay = (bool) luaL_optnumber(L, 3, 0); // default: false
+	const bool useOverlay = lua_isboolean(L, 3) ? lua_toboolean(L, 3) : false;
 	configHandler->Set(name, value, useOverlay);
 	return 0;
 }
@@ -1567,7 +1565,7 @@ int LuaUnsyncedCtrl::GetConfigString(lua_State* L)
 	}
 	const string name = luaL_checkstring(L, 1);
 	const string def  = luaL_optstring(L, 2, "");
-	const bool setInOverlay = (bool) luaL_optnumber(L, 3, 0); // default: false
+	const bool setInOverlay = lua_isboolean(L, 3) ? lua_toboolean(L, 3) : false;
 	const string value = configHandler->GetString(name, def, setInOverlay);
 	lua_pushstring(L, value.c_str());
 	return 1;
@@ -1581,7 +1579,7 @@ int LuaUnsyncedCtrl::SetConfigString(lua_State* L)
 	}
 	const string name  = luaL_checkstring(L, 1);
 	const string value = luaL_checkstring(L, 2);
-	const bool useOverlay = (bool) luaL_optnumber(L, 3, 0); // default: false
+	const bool useOverlay = lua_isboolean(L, 3) ? lua_toboolean(L, 3) : false;
 	configHandler->SetString(name, value, useOverlay);
 	return 0;
 }
