@@ -8,40 +8,41 @@
 #include <string>
 #include <ctype.h>
 
+#include "Lua/LuaParser.h"
 #include "Lua/LuaSyncedRead.h"
-#include "FileSystem/FileSystem.h"
+#include "System/Util.h"
+#include "System/FileSystem/FileSystem.h"
 
-using namespace std;
 
-string MapParser::GetMapConfigName(const string& mapName)
+std::string MapParser::GetMapConfigName(const std::string& mapFileName)
 {
-	if (mapName.length() < 3)
-		return "";
-
-	const string extension = filesystem.GetExtension(mapName);
+	const std::string directory = filesystem.GetDirectory(mapFileName);
+	const std::string filename  = filesystem.GetBasename(mapFileName);
+	const std::string extension = filesystem.GetExtension(mapFileName);
 
 	if (extension == "sm3") {
-		return mapName;
+		return mapFileName;
 	}
 	else if (extension == "smf") {
-		return mapName.substr(0, mapName.find_last_of('.')) + ".smd";
+		return directory + filename + ".smd";
 	}
 	else {
-		return "";
+		return mapFileName;
 	}
 }
 
 
-MapParser::MapParser(const string& mapName) : parser(NULL)
+MapParser::MapParser(const std::string& mapFileName) : parser(NULL)
 {
-	const string mapConfig = GetMapConfigName(mapName);
+	const std::string mapConfig = GetMapConfigName(mapFileName);
 
 	parser = new LuaParser("maphelper/mapinfo.lua", SPRING_VFS_MAP_BASE, SPRING_VFS_MAP_BASE);
 	parser->GetTable("Map");
-	parser->AddString("fileName", filesystem.GetFilename(mapName));
-	parser->AddString("fullName", mapName);
+	parser->AddString("fileName", filesystem.GetFilename(mapFileName));
+	parser->AddString("fullName", mapFileName);
 	parser->AddString("configFile", mapConfig);
 	parser->EndTable();
+
 #if !defined UNITSYNC && !defined DEDICATED && !defined BUILDING_AI
 	// this should not be included with unitsync:
 	// 1. avoids linkage with LuaSyncedRead
@@ -50,6 +51,7 @@ MapParser::MapParser(const string& mapName) : parser(NULL)
 	parser->AddFunc("GetMapOptions", LuaSyncedRead::GetMapOptions);
 	parser->EndTable();
 #endif // !defined UNITSYNC && !defined DEDICATED && !defined BUILDING_AI
+
 	if (!parser->Execute())
 	{
 		// do nothing
@@ -65,12 +67,16 @@ MapParser::~MapParser()
 
 bool MapParser::GetStartPos(int team, float3& pos) const
 {
+	errorLog.clear();
+
 	if (!parser->IsValid()) {
+		errorLog = "Map-Parser: Failed to get start position for team " + IntToString(team) + ", reason: parser not ready || file not found";
 		return false;
 	}
 	const LuaTable teamsTable = parser->GetRoot().SubTable("teams");
 	const LuaTable posTable = teamsTable.SubTable(team).SubTable("startPos");
 	if (!posTable.IsValid()) {
+		errorLog = "Map-Parser: Failed to get start position for team " + IntToString(team) + ", reason: " + parser->GetErrorLog();
 		return false;
 	}
 
@@ -84,8 +90,10 @@ bool MapParser::GetStartPos(int team, float3& pos) const
 LuaTable MapParser::GetRoot()
 {
 	if (parser) {
+		errorLog.clear();
 		return parser->GetRoot();
 	} else {
+		errorLog = "Map-Parser: Failed to get parser root node, reason: parser not ready || file not found.";
 		return LuaTable();
 	}
 }
@@ -104,8 +112,8 @@ bool MapParser::IsValid() const
 std::string MapParser::GetErrorLog() const
 {
 	if (parser) {
-		return parser->GetErrorLog();
+		return errorLog;
 	} else {
-		return "could not find file";
+		return "Map-Parser: parser not ready || file not found";
 	}
 }
