@@ -1,31 +1,8 @@
-/*
----------------------------------------------------------------------
-   Terrain Renderer using texture splatting and geomipmapping
-   Copyright (c) 2006 Jelmer Cnossen
+/* This file is part of the Spring engine (GPL v2 or later), see LICENSE.html */
 
-   This software is provided 'as-is', without any express or implied
-   warranty. In no event will the authors be held liable for any
-   damages arising from the use of this software.
+#ifndef _TERRAIN_TEXTURE_GLSL_H_
+#define _TERRAIN_TEXTURE_GLSL_H_
 
-   Permission is granted to anyone to use this software for any
-   purpose, including commercial applications, and to alter it and
-   redistribute it freely, subject to the following restrictions:
-
-   1. The origin of this software must not be misrepresented; you
-      must not claim that you wrote the original software. If you use
-      this software in a product, an acknowledgment in the product
-      documentation would be appreciated but is not required.
-
-   2. Altered source versions must be plainly marked as such, and
-      must not be misrepresented as being the original software.
-
-   3. This notice may not be removed or altered from any source
-      distribution.
-
-   Jelmer Cnossen
-   j.cnossen at gmail dot com
----------------------------------------------------------------------
-*/
 namespace terrain 
 {
 	struct Blendmap;
@@ -128,3 +105,5 @@ namespace terrain
 	};
 };
 
+#endif // _TERRAIN_TEXTURE_GLSL_H_
+
