@@ -1,3 +1,5 @@
+/* This file is part of the Spring engine (GPL v2 or later), see LICENSE.html */
+
 #include "StdAfx.h"
 
 #include "Map/SMF/BFGroundTextures.h"
@@ -6,18 +8,20 @@
 #include <cstdlib>
 #include <cstdio>
 
+#include "Rendering/GL/PBO.h"
+#include "Rendering/GlobalRendering.h"
 #include "Map/SMF/mapfile.h"
 #include "Map/SMF/SmfReadMap.h"
 #include "Map/MapInfo.h"
 #include "Game/Camera.h"
 #include "Game/Game.h"
-#include "System/FileSystem/FileHandler.h"
-#include "System/Platform/errorhandler.h"
-#include "System/TimeProfiler.h"
+#include "System/Exceptions.h"
+#include "System/FastMath.h"
 #include "System/GlobalUnsynced.h"
 #include "System/LogOutput.h"
 #include "System/mmgr.h"
-#include "System/FastMath.h"
+#include "System/TimeProfiler.h"
+#include "System/FileSystem/FileHandler.h"
 
 using std::sprintf;
 using std::string;
@@ -29,13 +33,6 @@ CBFGroundTextures::CBFGroundTextures(CSmfReadMap* rm) :
 	numBigTexX(gs->mapx / bigSquareSize),
 	numBigTexY(gs->mapy / bigSquareSize)
 {
-	usePBO = false;
-	if (GLEW_EXT_pixel_buffer_object && rm->usePBO) {
-		glGenBuffers(10, pboIDs);
-		currentPBO = 0;
-		usePBO = true;
-	}
-
 	// todo: refactor: put reading code in CSmfFile and keep errorhandling/progress reporting here..
 	map = rm;
 	CFileHandler* ifs = rm->GetFile().GetFileHandler();
@@ -60,7 +57,7 @@ CBFGroundTextures::CBFGroundTextures(CSmfReadMap* rm) :
 		if (smf.smtFileNames.size() != tileHeader.numTileFiles) {
 			logOutput.Print(
 				"[CBFGroundTextures] mismatched number of .smt file "
-				"references between map's .smd (%d) and header (%d);"
+				"references between map's .smd ("_STPF_") and header (%d);"
 				" ignoring .smd overrides",
 				smf.smtFileNames.size(), tileHeader.numTileFiles
 			);
@@ -124,8 +121,7 @@ CBFGroundTextures::CBFGroundTextures(CSmfReadMap* rm) :
 		if (strcmp(tfh.magic, "spring tilefile") != 0 || tfh.version != 1 || tfh.tileSize != 32 || tfh.compressionType != 1) {
 			char t[500];
 			sprintf(t, "[CBFGroundTextures] file \"%s\" does not match .smt format", tileFileName.c_str());
-			handleerror(0, t, "Error reading tile-file", 0);
-			exit(0);
+			throw content_error(t);
 		}
 
 		for (int b = 0; b < numSmallTiles; ++b) {
@@ -216,10 +212,6 @@ CBFGroundTextures::~CBFGroundTextures(void)
 	delete[] tileMap;
 	delete[] tiles;
 
-	if (usePBO) {
-		glDeleteBuffers(10,pboIDs);
-	}
-
 	delete[] heightMaxes;
 	delete[] heightMins;
 	delete[] stretchFactors;
@@ -230,8 +222,9 @@ void CBFGroundTextures::SetTexture(int x, int y)
 {
 	GroundSquare* square = &squares[y * numBigTexX + x];
 	glBindTexture(GL_TEXTURE_2D, square->texture);
-	if (game->GetDrawMode() == CGame::normalDraw) {
-		square->lastUsed = gu->drawFrame;
+
+	if (game->GetDrawMode() == CGame::gameNormalDraw) {
+		square->lastUsed = globalRendering->drawFrame;
 	}
 }
 
@@ -252,16 +245,21 @@ inline bool CBFGroundTextures::TexSquareInView(int btx, int bty) {
 
 void CBFGroundTextures::DrawUpdate(void)
 {
-	float diag = fastmath::apxsqrt(gu->viewSizeX*gu->viewSizeX + gu->viewSizeY*gu->viewSizeY); //screen diagonal number of pixels
+	// screen-diagonal number of pixels
+	const float diag = fastmath::apxsqrt(globalRendering->viewSizeX * globalRendering->viewSizeX + globalRendering->viewSizeY * globalRendering->viewSizeY);
+
 	for (int y = 0; y < numBigTexY; ++y) {
-		float dy = cam2->pos.z - y * bigSquareSize * SQUARE_SIZE - (SQUARE_SIZE << 6);
+		float dy =
+			cam2->pos.z -
+			y * bigSquareSize * SQUARE_SIZE -
+			(SQUARE_SIZE << 6);
 		dy = max(0.0f, float(fabs(dy) - (SQUARE_SIZE << 6)));
 
 		for (int x = 0; x < numBigTexX; ++x) {
 			GroundSquare* square = &squares[y * numBigTexX + x];
 
 			if (!TexSquareInView(x, y)) {
-				if ((square->texLevel < 3) && (gu->drawFrame - square->lastUsed > 120)) {
+				if ((square->texLevel < 3) && (globalRendering->drawFrame - square->lastUsed > 120)) {
 					// `unload` texture (= load lowest mipmap)
 					// if the square wasn't visible for 120 vframes
 					glDeleteTextures(1, &square->texture);
@@ -270,39 +268,53 @@ void CBFGroundTextures::DrawUpdate(void)
 				continue;
 			}
 
-			float dx = cam2->pos.x - x * bigSquareSize * SQUARE_SIZE - (SQUARE_SIZE << 6);
+			float dx =
+				cam2->pos.x -
+				x * bigSquareSize * SQUARE_SIZE -
+				(SQUARE_SIZE << 6);
 			dx = max(0.0f, float(fabs(dx) - (SQUARE_SIZE << 6)));
-			float dz = max( cam2->pos.y - (heightMaxes[y * numBigTexX + x] + heightMins[y * numBigTexX + x])/2 ,0.0f);
-			float dist = fastmath::apxsqrt(dx * dx + dy * dy + dz * dz);
 
-			// so, we shall work under the following assumptions:
-			// the minimum mip level is the closest ceiling mip level that we can use based on distance, FOV and tile size.
-			// we can increase this mip level IF the stretch factor requires us to do so.
-			// for simplicitys sake we will approximate tile size with a sphere of 512 elmos radius- which is =~ a sqrt2*1024 =~ 1400 diag pixels diameter sphere.
-			// half fov is 45 degs, for default ta and most other camera modes
-			int wantedLevel =0;
-			float dh=heightMaxes[y * numBigTexX + x] - heightMins[y * numBigTexX + x];
-			float sp=0; //screenpixels
-			if (dh > 1024) // this means that is the heightmap chunk is taller than it is wide, then we use the tallness metric instead of the width for calculating the size of it on screen.
-				sp = (dh)*(diag/2)/dist; //dist and viewsize based number (screenpixels).
+			const float dz = max(cam2->pos.y - (heightMaxes[y * numBigTexX + x] + heightMins[y * numBigTexX + x]) / 2, 0.0f);
+			const float dist = fastmath::apxsqrt(dx * dx + dy * dy + dz * dz);
+
+			// we work under the following assumptions:
+			//    the minimum mip level is the closest ceiling mip level that we can use
+			//    based on distance, FOV and tile size; we can increase this mip level IF
+			//    the stretch factor requires us to do so.
+			//
+			//    we will approximate tile size with a sphere 512 elmos in radius, which
+			//    translates to a diameter of =~ sqrt2 * 1024 =~ 1400 pixels
+			//
+			//    half (vertical) FOV is 45 degs, for default and most other camera modes
+			int wantedLevel = 0;
+			float heightDiff = heightMaxes[y * numBigTexX + x] - heightMins[y * numBigTexX + x];
+			int screenPixels = 1024;
+
+			if (dist > 0.0f) {
+				if (heightDiff > 1024.0f) {
+					// this means the heightmap chunk is taller than it is wide,
+					// so we use the tallness metric instead for calculating its
+					// on-screen size in pixels
+					screenPixels = int((heightDiff) * (diag * 0.5f) / dist);
+				} else {
+					screenPixels = int(1024 * (diag * 0.5f) / dist);
+				}
+			}
+
+			if (screenPixels > 513)
+				wantedLevel = 0;
+			else if (screenPixels > 257)
+				wantedLevel = 1;
+			else if (screenPixels > 129)
+				wantedLevel = 2;
 			else
-				sp = 1024*(diag/2)/dist;
-
-			if (sp>513)
-				wantedLevel=0;
-			else if (sp > 257)
-				wantedLevel=1;
-			else if (sp > 129)
-				wantedLevel=2;
-			else
-				wantedLevel=3;
-
-			if (stretchFactors[y*numBigTexX+x]>16000 && wantedLevel>0) //16k is an approximation of the sobel sum required to have a heightmap that has double the texture area than a flat square.
-				wantedLevel--;
-
-			if (wantedLevel > 3)
 				wantedLevel = 3;
 
+			// 16K is an approximation of the Sobel sum required to have a
+			// heightmap that has double the texture area of a flat square
+			if (stretchFactors[y * numBigTexX + x] > 16000 && wantedLevel > 0)
+				wantedLevel--;
+
 			if (square->texLevel != wantedLevel) {
 				glDeleteTextures(1, &square->texture);
 				LoadSquare(x, y, wantedLevel);
@@ -317,23 +329,9 @@ void CBFGroundTextures::LoadSquare(int x, int y, int level)
 {
 	int size = 1024 >> level;
 
-	GLint* buf = NULL;
-	bool usedPBO = false;
-
-	if (usePBO) {
-		if (currentPBO > 9) currentPBO = 0;
-		glBindBuffer(GL_PIXEL_UNPACK_BUFFER, pboIDs[currentPBO++]);
-		glBufferData(GL_PIXEL_UNPACK_BUFFER, size * size / 2, 0, GL_STREAM_DRAW);
-
-		//! map the buffer object into client's memory
-		buf = (GLint*)glMapBuffer(GL_PIXEL_UNPACK_BUFFER, GL_WRITE_ONLY);
-		usedPBO = true;
-	}
-
-	if (buf == NULL) {
-		buf = (GLint*)(new GLubyte[size * size / 2]);
-		usedPBO = false;
-	}
+	pbo.Bind();
+	pbo.Resize(size * size / 2);
+	GLint* buf = (GLint*)pbo.MapBuffer();
 
 	GroundSquare* square = &squares[y * numBigTexX + x];
 	square->texLevel = level;
@@ -352,6 +350,8 @@ void CBFGroundTextures::LoadSquare(int x, int y, int level)
 		}
 	}
 
+	pbo.UnmapBuffer();
+
 	glGenTextures(1, &square->texture);
 	glBindTexture(GL_TEXTURE_2D, square->texture);
 	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
@@ -368,14 +368,6 @@ void CBFGroundTextures::LoadSquare(int x, int y, int level)
 		glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_PRIORITY, 0.5f);
 	}
 
-	if (usedPBO) {
-		glUnmapBuffer(GL_PIXEL_UNPACK_BUFFER);
-		glCompressedTexImage2D(GL_TEXTURE_2D, 0, GL_COMPRESSED_RGBA_S3TC_DXT1_EXT, size, size, 0, size * size / 2, 0);
-		if (!gu->atiHacks)
-			glBufferData(GL_PIXEL_UNPACK_BUFFER, 0, 0, GL_STREAM_DRAW); //discard old content
-		glBindBuffer(GL_PIXEL_UNPACK_BUFFER, 0);
-	} else {
-		glCompressedTexImage2D(GL_TEXTURE_2D, 0, GL_COMPRESSED_RGBA_S3TC_DXT1_EXT, size, size, 0, size * size / 2, buf);
-		delete[] buf;
-	}
+	glCompressedTexImage2D(GL_TEXTURE_2D, 0, GL_COMPRESSED_RGBA_S3TC_DXT1_EXT, size, size, 0, size * size / 2, pbo.GetPtr());
+	pbo.Unbind();
 }
