@@ -1,10 +1,9 @@
-// BFGroundTextures.h
-///////////////////////////////////////////////////////////////////////////
+/* This file is part of the Spring engine (GPL v2 or later), see LICENSE.html */
 
-#ifndef __BF_GROUND_TEXTURES_H__
-#define __BF_GROUND_TEXTURES_H__
+#ifndef _BF_GROUND_TEXTURES_H_
+#define _BF_GROUND_TEXTURES_H_
 
-#include "Rendering/GL/myGL.h"
+#include "Rendering/GL/PBO.h"
 
 class CFileHandler;
 class CSmfReadMap;
@@ -46,13 +45,11 @@ protected:
 	float* stretchFactors;
 
 	//! use Pixel Buffer Objects for async. uploading (DMA)
-	bool usePBO;
-	GLuint pboIDs[10];
-	int currentPBO;
+	PBO pbo;
 
 	float anisotropy;
 
 	inline bool TexSquareInView(int, int);
 };
 
-#endif // __BF_GROUND_TEXTURES_H__
+#endif // _BF_GROUND_TEXTURES_H_
