@@ -1,3 +1,5 @@
+/* This file is part of the Spring engine (GPL v2 or later), see LICENSE.html */
+
 #include "StdAfx.h"
 #include "mmgr.h"
 
@@ -15,6 +17,9 @@ CBaseSky* sky=0;
 CBaseSky::CBaseSky(void)
 {
 	wireframe = false;
+	dynamicSky = false;
+	cloudDensity = 0;
+	fogStart = 0;
 }
 
 CBaseSky::~CBaseSky(void)
@@ -25,7 +30,7 @@ CBaseSky* CBaseSky::GetSky()
 {
 	if(!mapInfo->atmosphere.skyBox.empty())
 		return new CSkyBox("maps/" + mapInfo->atmosphere.skyBox);
-	else if(GLEW_ARB_fragment_program && configHandler->Get("AdvSky",1) && ProgramStringIsNative(GL_FRAGMENT_PROGRAM_ARB,"clouds.fp"))
+	else if(GLEW_ARB_fragment_program && configHandler->Get("AdvSky",1) && ProgramStringIsNative(GL_FRAGMENT_PROGRAM_ARB,"ARB/clouds.fp"))
 		return new CAdvSky();
 	else
 		return new CBasicSky();
