@@ -1,6 +1,4 @@
-// TreeDrawer.cpp: implementation of the CBasicTreeDrawer class.
-//
-//////////////////////////////////////////////////////////////////////
+/* This file is part of the Spring engine (GPL v2 or later), see LICENSE.html */
 
 #include "StdAfx.h"
 #include "mmgr.h"
@@ -9,13 +7,14 @@
 #include "Game/Camera.h"
 #include "Lua/LuaParser.h"
 #include "Map/Ground.h"
+#include "Map/MapInfo.h"
 #include "Map/ReadMap.h"
+#include "Rendering/GlobalRendering.h"
 #include "Rendering/GL/VertexArray.h"
 #include "Rendering/Textures/Bitmap.h"
-#include "LogOutput.h"
-#include "Exceptions.h"
-#include "GlobalUnsynced.h"
-#include "Map/MapInfo.h"
+#include "System/LogOutput.h"
+#include "System/Exceptions.h"
+#include "System/GlobalUnsynced.h"
 
 //////////////////////////////////////////////////////////////////////
 // Construction/Destruction
@@ -26,7 +25,7 @@ static const float MAX_TREE_HEIGHT_3=MAX_TREE_HEIGHT*0.3f;
 static const float MAX_TREE_HEIGHT_36=MAX_TREE_HEIGHT*0.36f;
 static const float MAX_TREE_HEIGHT_6=MAX_TREE_HEIGHT*0.6f;
 
-CBasicTreeDrawer::CBasicTreeDrawer()
+CBasicTreeDrawer::CBasicTreeDrawer(): CBaseTreeDrawer()
 {
 	lastListClean=0;
 
@@ -159,9 +158,9 @@ CBasicTreeDrawer::CBasicTreeDrawer()
 	nTrees=treesX*treesY;
 	trees=new TreeSquareStruct[nTrees];
 
-	for(TreeSquareStruct* pTSS=trees; pTSS<trees+nTrees; ++pTSS) {
-		pTSS->displist=0;
-		pTSS->farDisplist=0;
+	for (TreeSquareStruct* pTSS = trees; pTSS < trees + nTrees; ++pTSS) {
+		pTSS->displist = 0;
+		pTSS->farDisplist = 0;
 	}
 }
 
@@ -188,10 +187,16 @@ static void inline SetArrayQ(float t1,float t2,float3 v)
 
 struct CBasicTreeSquareDrawer : CReadMap::IQuadDrawer
 {
-	CBasicTreeSquareDrawer() {td=0;}
-	void DrawQuad (int x,int y);
+	CBasicTreeSquareDrawer(CBasicTreeDrawer* td_, int cx_, int cy_, float treeDistance_)
+		: td(td_),
+		cx(cx_),
+		cy(cy_),
+		treeDistance(treeDistance_)
+	{}
 
-	CBasicTreeDrawer *td;
+	void DrawQuad(int x, int y);
+
+	CBasicTreeDrawer* td;
 	int cx,cy;
 	float treeDistance;
 };
@@ -384,20 +389,16 @@ void CBasicTreeDrawer::Draw(float treeDistance,bool drawReflection)
 	glEnable(GL_TEXTURE_2D);
 	glEnable(GL_ALPHA_TEST);
 
-	if(gu->drawFog) {
+	if(globalRendering->drawFog) {
 		glFogfv(GL_FOG_COLOR, mapInfo->atmosphere.fogColor);
 		glEnable(GL_FOG);
 	}
 	glColor4f(1,1,1,1);
 
-	int cx=(int)(camera->pos.x/(SQUARE_SIZE*TREE_SQUARE_SIZE));
-	int cy=(int)(camera->pos.z/(SQUARE_SIZE*TREE_SQUARE_SIZE));
+	const int cx=(int)(camera->pos.x/(SQUARE_SIZE*TREE_SQUARE_SIZE));
+	const int cy=(int)(camera->pos.z/(SQUARE_SIZE*TREE_SQUARE_SIZE));
 
-	CBasicTreeSquareDrawer drawer;
-	drawer.td = this;
-	drawer.cx = cx;
-	drawer.cy = cy;
-	drawer.treeDistance = treeDistance * SQUARE_SIZE * TREE_SQUARE_SIZE;
+	CBasicTreeSquareDrawer drawer(this, cx, cy, treeDistance * SQUARE_SIZE * TREE_SQUARE_SIZE);
 
 	GML_STDMUTEX_LOCK(tree); // Draw
 
