@@ -462,7 +462,7 @@ static bool CheckParseErrors(GLenum target, const char* filename, const char* pr
 		"%d (near \"%.30s\") when loading %s-program file %s:\n%s",
 		errorPos, program + errorPos,
 		(target == GL_VERTEX_PROGRAM_ARB? "vertex": "fragment"),
-		filename, (const char*) errString
+		filename, errString ? (const char*) errString : "(null)"
 	);
 
 	return true;
@@ -487,12 +487,14 @@ static unsigned int LoadProgram(GLenum target, const char* filename, const char*
 		throw content_error(c);
 	}
 
-	char* fbuf = new char[file.FileSize()];
-	file.Read(fbuf, file.FileSize());
+	int fSize = file.FileSize();
+	char* fbuf = new char[fSize + 1];
+	file.Read(fbuf, fSize);
+	fbuf[fSize] = '\0';
 
 	glGenProgramsARB(1, &ret);
 	glBindProgramARB(target, ret);
-	glProgramStringARB(target, GL_PROGRAM_FORMAT_ASCII_ARB, file.FileSize(), fbuf);
+	glProgramStringARB(target, GL_PROGRAM_FORMAT_ASCII_ARB, fSize, fbuf);
 
 	if (CheckParseErrors(target, filename, fbuf)) {
 		ret = 0;
