@@ -1,9 +1,10 @@
+/* This file is part of the Spring engine (GPL v2 or later), see LICENSE.html */
+
 #include "StdAfx.h"
 #include <algorithm>
 #include <cctype>
 #include "mmgr.h"
 
-#include "FileSystem/FileSystem.h"
 #include "GroundDecalHandler.h"
 #include "Game/Camera.h"
 #include "Lua/LuaParser.h"
@@ -11,17 +12,18 @@
 #include "Map/Ground.h"
 #include "Map/MapInfo.h"
 #include "Map/ReadMap.h"
-#include "ConfigHandler.h"
-#include "ShadowHandler.h"
-#include "GL/VertexArray.h"
-#include "Textures/Bitmap.h"
+#include "Rendering/GlobalRendering.h"
+#include "Rendering/ShadowHandler.h"
+#include "Rendering/GL/myGL.h"
+#include "Rendering/Textures/Bitmap.h"
 #include "Sim/Units/Unit.h"
 #include "Sim/Units/UnitDef.h"
 #include "Sim/Units/UnitTypes/Building.h"
-#include "GlobalUnsynced.h"
-#include "LogOutput.h"
-#include "Util.h"
-#include "Exceptions.h"
+#include "System/ConfigHandler.h"
+#include "System/Exceptions.h"
+#include "System/GlobalUnsynced.h"
+#include "System/LogOutput.h"
+#include "System/FileSystem/FileSystem.h"
 
 using std::list;
 using std::min;
@@ -75,9 +77,9 @@ CGroundDecalHandler::CGroundDecalHandler(void)
 	delete[] buf;
 
 	if (shadowHandler->canUseShadows) {
-		decalVP    = LoadVertexProgram("GroundDecals.vp");
-		decalFPsmf = LoadFragmentProgram("GroundDecalsSMF.fp");
-		decalFPsm3 = LoadFragmentProgram("GroundDecalsSM3.fp");
+		decalVP    = LoadVertexProgram("ARB/GroundDecals.vp");
+		decalFPsmf = LoadFragmentProgram("ARB/GroundDecalsSMF.fp");
+		decalFPsm3 = LoadFragmentProgram("ARB/GroundDecalsSM3.fp");
 	}
 }
 
@@ -422,6 +424,20 @@ void CGroundDecalHandler::Draw(void)
 		glTexEnvi(GL_TEXTURE_ENV, GL_COMBINE_ALPHA_ARB, GL_REPLACE);
 		glTexEnvi(GL_TEXTURE_ENV, GL_SOURCE0_ALPHA_ARB, GL_PREVIOUS_ARB);
 		glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_COMBINE_ARB);
+	CBaseGroundDrawer *gd = readmap->GetGroundDrawer();
+	if (gd->DrawExtraTex()) {
+			glActiveTextureARB(GL_TEXTURE3_ARB);
+			glEnable(GL_TEXTURE_2D);
+			glTexEnvi(GL_TEXTURE_ENV,GL_COMBINE_RGB_ARB,GL_ADD_SIGNED_ARB);
+			glTexEnvi(GL_TEXTURE_ENV,GL_COMBINE_ALPHA_ARB,GL_MODULATE);
+			glTexEnvi(GL_TEXTURE_ENV,GL_SOURCE0_ALPHA_ARB,GL_PREVIOUS_ARB);
+			glTexEnvi(GL_TEXTURE_ENV,GL_SOURCE1_ALPHA_ARB,GL_TEXTURE);
+			glTexEnvi(GL_TEXTURE_ENV,GL_TEXTURE_ENV_MODE,GL_COMBINE_ARB);
+
+			SetTexGen(1.0f/(gs->pwr2mapx*SQUARE_SIZE),1.0f/(gs->pwr2mapy*SQUARE_SIZE),0,0);
+
+			glBindTexture(GL_TEXTURE_2D, gd->infoTex);
+	}
 	glActiveTextureARB(GL_TEXTURE0_ARB);
 
 
@@ -475,7 +491,7 @@ void CGroundDecalHandler::Draw(void)
 					if (decal->owner && decal->owner->buildProgress >= 0) {
 						decal->alpha = decal->owner->buildProgress;
 					} else if (!decal->gbOwner) {
-						decal->alpha -= decal->alphaFalloff * gu->lastFrameTime * gs->speedFactor;
+						decal->alpha -= decal->alphaFalloff * globalRendering->lastFrameTime * gs->speedFactor;
 					}
 
 					if (decal->alpha < 0.0f) {
@@ -508,7 +524,7 @@ void CGroundDecalHandler::Draw(void)
 				}
 			}
 
-			glBindTexture(GL_TEXTURE_2D, 0);
+			//glBindTexture(GL_TEXTURE_2D, 0);
 		}
 	}
 
@@ -703,6 +719,14 @@ void CGroundDecalHandler::Draw(void)
 		glTexEnvi(GL_TEXTURE_ENV, GL_SOURCE0_ALPHA_ARB, GL_PREVIOUS_ARB);
 		glTexEnvi(GL_TEXTURE_ENV, GL_SOURCE1_ALPHA_ARB, GL_TEXTURE);
 		glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);
+	glActiveTextureARB(GL_TEXTURE3_ARB); //! infotex
+		glDisable(GL_TEXTURE_2D);
+		glDisable(GL_TEXTURE_GEN_S);
+		glDisable(GL_TEXTURE_GEN_T);
+		glTexEnvi(GL_TEXTURE_ENV, GL_COMBINE_ALPHA_ARB, GL_MODULATE);
+		glTexEnvi(GL_TEXTURE_ENV, GL_SOURCE0_ALPHA_ARB, GL_PREVIOUS_ARB);
+		glTexEnvi(GL_TEXTURE_ENV, GL_SOURCE1_ALPHA_ARB, GL_TEXTURE);
+		glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);
 	glActiveTextureARB(GL_TEXTURE0_ARB);
 }
 
@@ -723,7 +747,9 @@ void CGroundDecalHandler::UnitMoved(CUnit* unit)
 
 void CGroundDecalHandler::UnitMovedNow(CUnit* unit)
 {
-	if(decalLevel == 0)
+	if (decalLevel == 0)
+		return;
+	if (unit->unitDef->trackType < 0)
 		return;
 
 	int zp = (int(unit->pos.z)/SQUARE_SIZE*2);
@@ -738,7 +764,7 @@ void CGroundDecalHandler::UnitMovedNow(CUnit* unit)
 
 	float3 pos = unit->pos+unit->frontdir*unit->unitDef->trackOffset;
 
-	TrackPart *tp = new TrackPart;
+	TrackPart* tp = new TrackPart;
 	tp->pos1 = pos+unit->rightdir*unit->unitDef->trackWidth*0.5f;
 	tp->pos1.y = ground->GetHeight2(tp->pos1.x,tp->pos1.z);
 	tp->pos2 = pos-unit->rightdir*unit->unitDef->trackWidth*0.5f;
@@ -789,7 +815,6 @@ void CGroundDecalHandler::RemoveUnit(CUnit* unit)
 
 int CGroundDecalHandler::GetTrackType(const std::string& name)
 {
-
 	if (decalLevel == 0) {
 		return 0;
 	}
@@ -1027,6 +1052,15 @@ void CGroundDecalHandler::AddBuilding(CBuilding* building)
 {
 	if (decalLevel == 0)
 		return;
+	if (!building->unitDef->useBuildingGroundDecal)
+		return;
+	if (building->unitDef->buildingDecalType < 0)
+		return;
+
+	GML_STDMUTEX_LOCK(decal); // AddBuilding
+
+	if (building->buildingDecal)
+		return;
 
 	int posx = int(building->pos.x / 8);
 	int posy = int(building->pos.z / 8);
@@ -1054,8 +1088,6 @@ void CGroundDecalHandler::AddBuilding(CBuilding* building)
 	decal->posx = posx - (decal->xsize / 2);
 	decal->posy = posy - (decal->ysize / 2);
 
-	GML_STDMUTEX_LOCK(decal); // AddBuilding
-
 	building->buildingDecal = decal;
 	buildingDecalTypes[building->unitDef->buildingDecalType]->buildingDecals.insert(decal);
 }
@@ -1063,12 +1095,18 @@ void CGroundDecalHandler::AddBuilding(CBuilding* building)
 
 void CGroundDecalHandler::RemoveBuilding(CBuilding* building, CUnitDrawer::GhostBuilding* gb)
 {
+	if (!building->unitDef->useBuildingGroundDecal)
+		return;
+
 	GML_STDMUTEX_LOCK(decal); // RemoveBuilding
 
 	BuildingGroundDecal* decal = building->buildingDecal;
 	if (!decal)
 		return;
 
+	if(gb)
+		gb->decal = decal;
+
 	decal->owner = 0;
 	decal->gbOwner = gb;
 	building->buildingDecal = 0;
@@ -1080,24 +1118,20 @@ void CGroundDecalHandler::RemoveBuilding(CBuilding* building, CUnitDrawer::Ghost
  */
 void CGroundDecalHandler::ForceRemoveBuilding(CBuilding* building)
 {
-	GML_STDMUTEX_LOCK(decal); // ForcedRemoveBuilding
-
-	if (!building || !building->buildingDecal)
+	if (!building)
+		return;
+	if (!building->unitDef->useBuildingGroundDecal)
 		return;
 
-	std::vector<BuildingDecalType*>& types = buildingDecalTypes;
-	std::vector<BuildingDecalType*>::iterator bdt;
+	GML_STDMUTEX_LOCK(decal); // ForcedRemoveBuilding
 
-	for (bdt = types.begin(); bdt != types.end(); ++bdt) {
-		std::set<BuildingGroundDecal*>& decals = (*bdt)->buildingDecals;
-		std::set<BuildingGroundDecal*>::iterator bgd = decals.find(building->buildingDecal);
-		if (bgd != decals.end()) {
-			BuildingGroundDecal* decal = *bgd;
-			decals.erase(bgd);
-			building->buildingDecal = NULL;
-			delete decal;
-		}
-	}
+	BuildingGroundDecal* decal = building->buildingDecal;
+	if (!decal)
+		return;
+
+	decal->owner = NULL;
+	decal->alpha = 0.0f;
+	building->buildingDecal = NULL;
 }
 
 
@@ -1118,14 +1152,14 @@ int CGroundDecalHandler::GetBuildingDecalType(const std::string& name)
 		++a;
 	}
 
-	BuildingDecalType* tt = new BuildingDecalType;
-	tt->name = lowerName;
 	const std::string fullName = "unittextures/" + lowerName;
 	CBitmap bm;
 	if (!bm.Load(fullName)) {
 		throw content_error("Could not load building decal from file " + fullName);
 	}
 
+	BuildingDecalType* tt = new BuildingDecalType;
+	tt->name = lowerName;
 	tt->texture = bm.CreateTexture(true);
 
 //	GML_STDMUTEX_LOCK(decaltype); // GetBuildingDecalType
@@ -1134,16 +1168,3 @@ int CGroundDecalHandler::GetBuildingDecalType(const std::string& name)
 
 	return (buildingDecalTypes.size() - 1);
 }
-
-
-void CGroundDecalHandler::SetTexGen(float scalex,float scaley, float offsetx, float offsety)
-{
-	GLfloat plan[]={scalex,0,0,offsetx};
-	glTexGeni(GL_S,GL_TEXTURE_GEN_MODE,GL_EYE_LINEAR);
-	glTexGenfv(GL_S,GL_EYE_PLANE,plan);
-	glEnable(GL_TEXTURE_GEN_S);
-	GLfloat plan2[]={0,0,scaley,offsety};
-	glTexGeni(GL_T,GL_TEXTURE_GEN_MODE,GL_EYE_LINEAR);
-	glTexGenfv(GL_T,GL_EYE_PLANE,plan2);
-	glEnable(GL_TEXTURE_GEN_T);
-}
