@@ -1,30 +1,34 @@
+/* This file is part of the Spring engine (GPL v2 or later), see LICENSE.html */
+
 #include "StdAfx.h"
-// glFont.cpp: implementation of the CglFont class.
-//
-//////////////////////////////////////////////////////////////////////
 
 #include "glFont.h"
+#include <string>
+#include <cstring> // for memset, memcpy
 #include <stdio.h>
 #include <stdarg.h>
 #include <stdexcept>
+#ifndef   HEADLESS
 #include <ft2build.h>
 #include FT_FREETYPE_H
+#endif // HEADLESS
 
-#include "LogOutput.h"
 #include "Game/Camera.h"
-#include "myMath.h"
+#include "Rendering/GlobalRendering.h"
+#include "Rendering/GL/VertexArray.h"
+#include "Rendering/Textures/Bitmap.h"
+#include "System/LogOutput.h"
+#include "System/myMath.h"
+#include "System/FileSystem/FileHandler.h"
+#include "System/FileSystem/FileSystem.h"
+#include "System/GlobalUnsynced.h"
+#include "System/Util.h"
+#include "System/Exceptions.h"
+#include "System/mmgr.h"
+#include "System/float4.h"
 #include "bitops.h"
-#include "FileSystem/FileHandler.h"
-#include "FileSystem/FileSystem.h"
-#include "GL/VertexArray.h"
-#include "Textures/Bitmap.h"
-#include "GlobalUnsynced.h"
-#include "Util.h"
-#include "Exceptions.h"
-#include "mmgr.h"
-#include "float4.h"
 
-#undef GetCharWidth //winapi.h
+#undef GetCharWidth // winapi.h
 
 using std::string;
 
@@ -108,6 +112,7 @@ class texture_size_exception : public std::exception
 };
 
 
+#ifndef   HEADLESS
 #undef __FTERRORS_H__
 #define FT_ERRORDEF( e, v, s )  { e, s },
 #define FT_ERROR_START_LIST     {
@@ -128,6 +133,7 @@ static const char* GetFTError(FT_Error e)
 	}
 	return "Unknown error";
 }
+#endif // HEADLESS
 
 /*******************************************************************************/
 /*******************************************************************************/
@@ -139,23 +145,23 @@ static const char* GetFTError(FT_Error e)
 class CFontTextureRenderer
 {
 public:
-	CFontTextureRenderer(int _outlinewidth, int _outlineweight) :
-		texWidth(0), texHeight(0),
+	CFontTextureRenderer(int _outlinewidth, int _outlineweight)
+		: texWidth(0), texHeight(0),
 		outlinewidth(_outlinewidth), outlineweight(_outlineweight),
-		curX(0), curY(0), curHeight(0),
+		atlas(NULL),
+		cur(NULL),
+		curX(0), curY(0),
+		curHeight(0),
 		numGlyphs(0),
-		maxGlyphWidth(0), maxGlyphHeight(0), numPixels(0)
+		maxGlyphWidth(0), maxGlyphHeight(0),
+		numPixels(0)
 	{
-		numGlyphs = 0;
-		texWidth = texHeight = 0;
-		curX = curY = curHeight = 0;
-		numPixels = maxGlyphWidth = maxGlyphHeight = 0;
 	};
 
 	GLuint CreateTexture();
 
 	void AddGlyph(unsigned int& index, int& xsize, int& ysize, unsigned char* pixels, int& pitch);
-	void GetGlyph(unsigned int& index, CglFont::GlyphInfo* g);
+	void GetGlyph(unsigned int& index, CglFont::GlyphInfo* g) const;
 
 	int texWidth, texHeight;
 private:
@@ -258,9 +264,9 @@ void CFontTextureRenderer::CopyGlyphsIntoBitmapAtlas(bool outline)
 
 
 
-void CFontTextureRenderer::GetGlyph(unsigned int& index, CglFont::GlyphInfo* g)
+void CFontTextureRenderer::GetGlyph(unsigned int& index, CglFont::GlyphInfo* g) const
 {
-	GlyphInfo& gi = glyphs[index];
+	const GlyphInfo& gi = glyphs[index];
 	g->u0 = gi.u / (float)texWidth;
 	g->v0 = gi.v / (float)texHeight;
 	g->u1 = (gi.u+gi.xsize) / (float)texWidth;
@@ -367,7 +373,7 @@ GLuint CFontTextureRenderer::CreateTexture()
 
 	//! generate the ogl texture
 	texHeight = curY + curHeight;
-	if (!gu->supportNPOTs)
+	if (!globalRendering->supportNPOTs)
 		texHeight = next_power_of_2(texHeight);
 	GLuint tex;
 	glGenTextures(1, &tex);
@@ -400,7 +406,9 @@ CglFont::CglFont(const std::string& fontfile, int size, int _outlinewidth, float
 	fontPath(fontfile),
 	outlineWidth(_outlinewidth),
 	outlineWeight(_outlineweight),
-	inBeginEnd(false)
+	inBeginEnd(false),
+	autoOutlineColor(true),
+	setColor(false)
 {
 	if (size<=0)
 		size = 14;
@@ -408,6 +416,12 @@ CglFont::CglFont(const std::string& fontfile, int size, int _outlinewidth, float
 	const float invSize = 1.0f / size;
 	const float normScale = invSize / 64.0f;
 
+	//! setup character range
+	charstart = 32;
+	charend   = 254; //! char 255 = colorcode
+	chars     = (charend - charstart) + 1;
+
+#ifndef   HEADLESS
 	FT_Library library;
 	FT_Face face;
 
@@ -461,11 +475,6 @@ CglFont::CglFont(const std::string& fontfile, int size, int _outlinewidth, float
 		throw content_error(msg);
 	}
 
-	//! setup character range
-	charstart = 32;
-	charend   = 254; //! char 255 = colorcode
-	chars     = (charend - charstart) + 1;
-
 	//! get font information
 	fontFamily = face->family_name;
 	fontStyle  = face->style_name;
@@ -548,6 +557,15 @@ CglFont::CglFont(const std::string& fontfile, int size, int _outlinewidth, float
 	FT_Done_Face(face);
 	FT_Done_FreeType(library);
 	delete[] buf;
+#else  // HEADLESS
+	fontFamily = "NONE";
+	fontStyle  = "NONE";
+	fontDescender = 0.0f;
+	lineHeight = 0.0f;
+	fontTexture = 0;
+	texWidth  = 0;
+	texHeight = 0;
+#endif // HEADLESS
 
 	textColor    = white;
 	outlineColor = darkOutline;
@@ -582,75 +600,28 @@ CglFont::~CglFont()
 /*******************************************************************************/
 
 template <typename T>
-static inline int SkipColorCodesOld(const std::string& text, T c)
+static inline int SkipColorCodesOld(const std::string& text, T pos)
 {
-	while (text[c] == CglFont::ColorCodeIndicator) { //FIXME use a non-printable char? (<32)
-		c += 4;
-		if (c >= text.size()) { return -1; }
+	while (text[pos] == CglFont::ColorCodeIndicator) {
+		pos += 4;
+		if (pos >= text.size()) { return -1; }
 	}
-	return c;
+	return pos;
 }
 
 
 template <typename T>
-static inline bool SkipNewLineOld(const std::string& text, T* c)
-{
-	if (text[*c] == '\x0d') {
-		(*c)++;
-		if (*c < text.length() && text[*c] == '\x0a') {
-			(*c)++;
-		}
-		return true;
-	} else if (text[*c] == '\x0a') {
-		(*c)++;
-		return true;
-	}
-	return false;
-}
-
-
-/**
- * @brief SkipNewLine
- * @param text
- * @param c index in the string
- * @return <0 := end of string; returned value is: -(skippedLines + 1)
- *         else: number of skipped lines (can be zero)
- */
-template <typename T>
-static inline int SkipNewLine(const std::string& text, T* c)
-{
-	const size_t length = text.length();
-	int skippedLines = 0;
-	while (*c < length) {
-		if (text[*c] == '\x0d') {
-			skippedLines++;
-			(*c)++;
-			if (*c < length && text[*c] == '\x0a') {
-				(*c)++;
-			}
-		} else if (text[*c] == '\x0a') {
-			skippedLines++;
-			(*c)++;
-		} else {
-			return skippedLines;
-		}
-	}
-	return -(1 + skippedLines);
-}
-
-
-template <typename T>
-static inline int SkipColorCodes(const std::string& text, T* c, float4* color)
+static inline int SkipColorCodes(const std::string& text, T* pos, float4* color)
 {
 	int colorFound = 0;
-	while (text[(*c)] == CglFont::ColorCodeIndicator) {
-		(*c) += 4;
-		if ((*c) >= text.size()) {
+	while (text[(*pos)] == CglFont::ColorCodeIndicator) {
+		(*pos) += 4;
+		if ((*pos) >= text.size()) {
 			return -(1 + colorFound);
 		} else {
-			(*color)[0] = ((unsigned char) text[(*c)-3]) / 255.0f;
-			(*color)[1] = ((unsigned char) text[(*c)-2]) / 255.0f;
-			(*color)[2] = ((unsigned char) text[(*c)-1]) / 255.0f;
+			(*color)[0] = ((unsigned char) text[(*pos)-3]) / 255.0f;
+			(*color)[1] = ((unsigned char) text[(*pos)-2]) / 255.0f;
+			(*color)[2] = ((unsigned char) text[(*pos)-1]) / 255.0f;
 			colorFound = 1;
 		}
 	}
@@ -658,33 +629,82 @@ static inline int SkipColorCodes(const std::string& text, T* c, float4* color)
 }
 
 
+/**
+ * @brief SkipNewLine
+ * @param text
+ * @param pos index in the string
+ * @return <0 := end of string; returned value is: -(skippedLines + 1)
+ *         else: number of skipped lines (can be zero)
+ */
+template <typename T>
+static inline int SkipNewLine(const std::string& text, T* pos)
+{
+	const size_t length = text.length();
+	int skippedLines = 0;
+	while (*pos < length) {
+		const char& chr = text[*pos];
+		switch(chr) {
+			case '\x0d': //! CR
+				skippedLines++;
+				(*pos)++;
+				if (*pos < length && text[*pos] == '\x0a') { //! CR+LF
+					(*pos)++;
+				}
+				break;
+
+			case '\x0a': //! LF
+				skippedLines++;
+				(*pos)++;
+				break;
+
+			default:
+				return skippedLines;
+		}
+	}
+	return -(1 + skippedLines);
+}
+
 
 template <typename T>
-static inline bool SkipColorCodesAndNewLines(const std::string& text, T* c, float4* color, bool* colorChanged, int* skippedLines)
+static inline bool SkipColorCodesAndNewLines(const std::string& text, T* pos, float4* color, bool* colorChanged, int* skippedLines, float4* colorReset)
 {
 	const size_t length = text.length();
 	(*colorChanged) = false;
 	(*skippedLines) = 0;
-	while (*c < length) {
-		if (text[*c] == '\x0d') {
-			(*skippedLines)++;
-			(*c)++;
-			if (*c < length && text[*c] == '\x0a') {
-				(*c)++;
-			}
-		} else if (text[*c] == '\x0a') {
-			(*skippedLines)++;
-			(*c)++;
-		} else if (text[*c] == CglFont::ColorCodeIndicator) {
-			*c += 4;
-			if ((*c) < length) {
-				(*color)[0] = ((unsigned char) text[(*c) - 3]) / 255.0f;
-				(*color)[1] = ((unsigned char) text[(*c) - 2]) / 255.0f;
-				(*color)[2] = ((unsigned char) text[(*c) - 1]) / 255.0f;
+	while (*pos < length) {
+		const char& chr = text[*pos];
+		switch(chr) {
+			case CglFont::ColorCodeIndicator:
+				*pos += 4;
+				if ((*pos) < length) {
+					(*color)[0] = ((unsigned char) text[(*pos) - 3]) / 255.0f;
+					(*color)[1] = ((unsigned char) text[(*pos) - 2]) / 255.0f;
+					(*color)[2] = ((unsigned char) text[(*pos) - 1]) / 255.0f;
+					*colorChanged = true;
+				}
+				break;
+
+			case CglFont::ColorResetIndicator:
+				(*pos)++;
+				(*color) = *colorReset;
 				*colorChanged = true;
-			}
-		} else {
-			return false;
+				break;
+
+			case '\x0d': //! CR
+				(*skippedLines)++;
+				(*pos)++;
+				if (*pos < length && text[*pos] == '\x0a') { //! CR+LF
+					(*pos)++;
+				}
+				break;
+
+			case '\x0a': //! LF
+				(*skippedLines)++;
+				(*pos)++;
+				break;
+
+			default:
+				return false;
 		}
 	}
 	return true;
@@ -708,7 +728,7 @@ std::string CglFont::StripColorCodes(const std::string& text)
 	std::string nocolor;
 	nocolor.reserve(len);
 	for (unsigned int i = 0; i < len; i++) {
-		if (text[i] == '\xff') {
+		if (text[i] == ColorCodeIndicator) {
 			i += 3;
 		} else {
 			nocolor += text[i];
@@ -743,25 +763,33 @@ float CglFont::GetTextWidth(const std::string& text) const
 	for (int pos = 0; pos < text.length(); pos++) {
 		const char& c = text[pos];
 		switch(c) {
+			//! inlined colorcode
 			case ColorCodeIndicator:
-				{
-					pos = SkipColorCodesOld(text, pos);
-					if (pos<0) {
-						pos = text.length();
-					} else {
-						pos--;
-					}
-				} break;
-			case '\x0d':
+				pos = SkipColorCodesOld(text, pos);
+				if (pos<0) {
+					pos = text.length();
+				} else {
+					pos--;
+				}
+				break;
+
+			//! reset color
+			case ColorResetIndicator:
+				break;
+
+			//! newline
+			case '\x0d': //! CR+LF
 				if (pos+1 < text.length() && text[pos+1] == '\x0a')
 					pos++;
-			case '\x0a':
+			case '\x0a': //! LF
 				w += glyphs[*prv_char].kerning[0];
 				if (w > maxw)
 					maxw = w;
 				w = 0.0f;
 				prv_char = &nullChar;
 				break;
+
+			//! printable char
 			default:
 				cur_char = reinterpret_cast<const unsigned char*>(&c);
 				w += glyphs[*prv_char].kerning[*cur_char];
@@ -779,33 +807,42 @@ float CglFont::GetTextWidth(const std::string& text) const
 
 float CglFont::GetTextHeight(const std::string& text, float* descender, int* numLines) const
 {
+	if (text.empty()) {
+		if (descender) *descender = 0.0f;
+		if (numLines) *numLines = 0;
+		return 0.0f;
+	}
+
 	float h = 0.0f, d = lineHeight + fontDescender;
 	unsigned int multiLine = 1;
 
 	for (int pos = 0 ; pos < text.length(); pos++) {
 		const char& c = text[pos];
 		switch(c) {
-			//! inline colorcodes
+			//! inlined colorcode
 			case ColorCodeIndicator:
-				{
-					pos = SkipColorCodesOld(text, pos);
-					if (pos<0) {
-						pos = text.length();
-					} else {
-						pos--;
-					}
-				} break;
+				pos = SkipColorCodesOld(text, pos);
+				if (pos<0) {
+					pos = text.length();
+				} else {
+					pos--;
+				}
+				break;
+
+			//! reset color
+			case ColorResetIndicator:
+				break;
 
 			//! newline
-			case '\x0d':
+			case '\x0d': //! CR+LF
 				if (pos+1 < text.length() && text[pos+1] == '\x0a')
 					pos++;
-			case '\x0a':
+			case '\x0a': //! LF
 				multiLine++;
 				d = lineHeight + fontDescender;
 				break;
 
-			//! normal char
+			//! printable char
 			default:
 				const unsigned char* uc = reinterpret_cast<const unsigned char*>(&c);
 				const GlyphInfo* g = &glyphs[ *uc ];
@@ -824,21 +861,27 @@ float CglFont::GetTextHeight(const std::string& text, float* descender, int* num
 
 int CglFont::GetTextNumLines(const std::string& text) const
 {
+	if (text.empty())
+		return 0;
+
 	int lines = 1;
 
 	for (int pos = 0 ; pos < text.length(); pos++) {
 		const char& c = text[pos];
 		switch(c) {
-			//! inline colorcodes
+			//! inlined colorcode
 			case ColorCodeIndicator:
-				{
-					pos = SkipColorCodesOld(text, pos);
-					if (pos<0) {
-						pos = text.length();
-					} else {
-						pos--;
-					}
-				} break;
+				pos = SkipColorCodesOld(text, pos);
+				if (pos<0) {
+					pos = text.length();
+				} else {
+					pos--;
+				}
+				break;
+
+			//! reset color
+			case ColorResetIndicator:
+				break;
 
 			//! newline
 			case '\x0d':
@@ -847,6 +890,7 @@ int CglFont::GetTextNumLines(const std::string& text) const
 			case '\x0a':
 				lines++;
 				break;
+
 			//default:
 		}
 	}
@@ -858,8 +902,8 @@ int CglFont::GetTextNumLines(const std::string& text) const
 /*******************************************************************************/
 
 /**
- * IsUpperCase
- * return true if the given uchar is an uppercase character (WinLatin charmap)
+ * @brief IsUpperCase
+ * @return true if the given uchar is an uppercase character (WinLatin charmap)
  */
 static inline bool IsUpperCase(const unsigned char& c)
 {
@@ -873,6 +917,13 @@ static inline bool IsUpperCase(const unsigned char& c)
 }
 
 
+/**
+ * @brief GetPenalty
+ * @param c character at %strpos% in the word
+ * @param strpos position of c in the word
+ * @param strlen total length of the word
+ * @return penalty (smaller is better) to split a word at that position
+ */
 static inline float GetPenalty(const unsigned char& c, unsigned int strpos, unsigned int strlen)
 {
 	const float dist = strlen - strpos;
@@ -897,6 +948,8 @@ static inline float GetPenalty(const unsigned char& c, unsigned int strpos, unsi
 
 CglFont::word CglFont::SplitWord(CglFont::word& w, float wantedWidth, bool smart) const
 {
+	//! returns two pieces 'L'eft and 'R'ight of the split word (returns L, *wi becomes R)
+
 	word w2;
 	w2.pos = w.pos;
 
@@ -996,6 +1049,25 @@ void CglFont::AddEllipsis(std::list<line>& lines, std::list<word>& words, float
 
 	line* l = &(lines.back());
 
+	//! If the last line ends with a linebreak, remove it
+	std::list<word>::iterator wi_end = l->end;
+	if (wi_end->isLineBreak) {
+		if (l->start == l->end || l->end == words.begin()) {
+			//! there is just the linebreak in that line, so replace linebreak with just a null space
+			word w;
+			w.pos       = wi_end->pos;
+			w.isSpace   = true;
+			w.numSpaces = 0;
+			l->start = words.insert(wi_end,w);
+			l->end = l->start;
+
+			words.erase(wi_end);
+		} else {
+			wi_end = words.erase(wi_end);
+			l->end = --wi_end;
+		}
+	}
+
 	//! remove as many words until we have enough free space for the ellipsis
 	while (l->end != l->start) {
 		word& w = *l->end;
@@ -1082,89 +1154,112 @@ void CglFont::AddEllipsis(std::list<line>& lines, std::list<word>& words, float
 
 void CglFont::WrapTextConsole(std::list<word>& words, float maxWidth, float maxHeight) const
 {
-	if(words.empty())
+	if (words.empty())
 		return;
 
-	const bool splitWords = false;
-
+	const bool splitAllWords = false;
 	const unsigned int maxLines = (unsigned int)math::floor(std::max(0.0f, maxHeight / lineHeight ));
 
+	line* currLine;
 	word linebreak;
 	linebreak.isLineBreak = true;
 
 	bool addEllipsis = false;
+	bool currLineValid = false; //! true if there was added any data to the current line
 
+	std::list<word>::iterator wi = words.begin();
+
+	std::list<word> splitWords;
 	std::list<line> lines;
-	lines.push_back(line());
-	line* l = &(lines.back());
-	l->start = words.begin();
+		lines.push_back(line());
+		currLine = &(lines.back());
+		currLine->start = words.begin();
 
-	std::list<word>::iterator wi;
-	for (wi = words.begin(); wi != words.end(); wi++) {
-		//! force a LineBreak
+	for (; ;) {
+		currLineValid = true;
 		if (wi->isLineBreak) {
-			l->forceLineBreak = true;
-
-			if (lines.size() >= maxLines) {
-				addEllipsis = true;
-				break;
-			}
-
-			l->end = wi;
+			currLine->forceLineBreak = true;
+			currLine->end = wi;
 
+			//! start a new line after the '\n'
 			lines.push_back(line());
-			l = &(lines.back());
-			l->start = wi; l->start++;
-			continue;
+				currLineValid = false;
+				currLine = &(lines.back());
+				currLine->start = wi;
+				currLine->start++;
+		} else {
+			currLine->width += wi->width;
+			currLine->end = wi;
+
+			if (currLine->width > maxWidth) {
+				currLine->width -= wi->width;
+
+				//! line grew too long by adding the last word, insert a LineBreak
+				const bool splitLastWord = (wi->width > (0.5 * maxWidth));
+				const float freeWordSpace = (maxWidth - currLine->width);
+
+				if (splitAllWords || splitLastWord) {
+					//! last word W is larger than 0.5 * maxLineWidth, split it into
+					//! get 'L'eft and 'R'ight parts of the split (wL becomes Left, *wi becomes R)
+
+					//! turns *wi into R
+					word wL = SplitWord(*wi, freeWordSpace);
+
+					if (splitLastWord && wL.width == 0.0f) {
+						//! With smart splitting it can happen that the word isn't splitted at all,
+						//! this can cause a race condition when the word is longer than maxWidth.
+						//! In this case we have to force an unaesthetic split.
+						wL = SplitWord(*wi, freeWordSpace, false);
+					}
+
+					//! increase by the width of the L-part of *wi
+					currLine->width += wL.width;
+
+					//! insert the L-part right before R
+					wi = words.insert(wi, wL);
+					wi++;
+				}
+
+				//! insert the forced linebreak (either after W or before R)
+				linebreak.pos = wi->pos;
+				currLine->end = words.insert(wi, linebreak);
+
+				while (wi != words.end() && wi->isSpace)
+					wi = words.erase(wi);
+
+				lines.push_back(line());
+					currLineValid = false;
+					currLine = &(lines.back());
+					currLine->start = wi;
+					wi--; //! compensate the wi++ downwards
+			}
 		}
 
-		l->width += wi->width;
+		wi++;
 
-		//! line got too long add a LineBreak
-		if (l->width > maxWidth) {
-			l->width -= wi->width;
-			if (
-				(splitWords) ||
-				(wi->width > (0.5 * maxWidth) && (l->width + wi->width) > maxWidth) //! word is larger than 0.5*maxLineWidth, so it is better we split if needed
-			) {
-				float freeSpace = maxWidth - l->width;
-				l->end = words.insert( wi, SplitWord(*wi, freeSpace) );
-				l->width += (l->end)->width;
-			} else {
-				l->end = wi;
-				l->end--;
-			}
+		if (wi == words.end()) {
+			break;
+		}
 
-			if (lines.size() >= maxLines) {
-				addEllipsis = true;
-				break;
-			}
-
-			linebreak.pos = wi->pos;
-			l->end = words.insert( wi, linebreak );
-
-			while(wi != words.end() && wi->isSpace)
-				wi = words.erase(wi);
-
-			lines.push_back(line());
-			l = &(lines.back());
-			l->start = wi;
-			wi--;
+		if (lines.size() > maxLines) {
+			addEllipsis = true;
+			break;
 		}
-		l->end = wi;
 	}
 
+	
+
 	//! empty row
-	if (l->start == words.end() && !l->forceLineBreak) {
+	if (!currLineValid || (currLine->start == words.end() && !currLine->forceLineBreak)) {
 		lines.pop_back();
-		l = &(lines.back());
+		currLine = &(lines.back());
 	}
 
-	//! if we cut the text cos of missing space add an ellipsis
+	//! if we had to cut the text because of missing space, add an ellipsis
 	if (addEllipsis)
 		AddEllipsis(lines, words, maxWidth);
 
-	wi = l->end; wi++;
+	wi = currLine->end; wi++;
 	wi = words.erase(wi, words.end());
 }
 
@@ -1187,6 +1282,7 @@ void CglFont::SplitTextInWords(const std::string& text, std::list<word>* words,
 	for (int pos = 0; pos < length; pos++) {
 		const char& c = text[pos];
 		switch(c) {
+			//! space
 			case '\x20':
 				if (!w->isSpace) {
 					if (w->isSpace) {
@@ -1201,6 +1297,8 @@ void CglFont::SplitTextInWords(const std::string& text, std::list<word>* words,
 				}
 				w->numSpaces++;
 				break;
+
+			//! inlined colorcodes
 			case ColorCodeIndicator:
 				{
 					colorcodes->push_back(colorcode());
@@ -1215,10 +1313,22 @@ void CglFont::SplitTextInWords(const std::string& text, std::list<word>* words,
 						pos--;
 					}
 				} break;
-			case '\x0d':
+			case ColorResetIndicator:
+				{
+					colorcode* cc = &colorcodes->back();
+					if (cc->pos != numChar) {
+						colorcodes->push_back(colorcode());
+						cc = &colorcodes->back();
+						cc->pos = numChar;
+					}
+					cc->resetColor = true;
+				} break;
+
+			//! newlines
+			case '\x0d': //! CR+LF
 				if (pos+1 < length && text[pos+1] == '\x0a')
 					pos++;
-			case '\x0a':
+			case '\x0a': //! LF
 				if (w->isSpace) {
 					w->width = spaceAdvance * w->numSpaces;
 				} else if (!w->isLineBreak) {
@@ -1228,8 +1338,9 @@ void CglFont::SplitTextInWords(const std::string& text, std::list<word>* words,
 				w = &(words->back());
 				w->isLineBreak = true;
 				w->pos = numChar;
-
 				break;
+
+			//! printable chars
 			default:
 				if (w->isSpace || w->isLineBreak) {
 					if (w->isSpace) {
@@ -1265,12 +1376,18 @@ void CglFont::RemergeColorCodes(std::list<word>* words, std::list<colorcode>& co
 		}
 
 		word wc;
-		wc.text = '\xff';
-		wc.text += (unsigned char)(255 * ci->color[0]);
-		wc.text += (unsigned char)(255 * ci->color[1]);
-		wc.text += (unsigned char)(255 * ci->color[2]);
-		wc.isColorCode = true;
 		wc.pos = ci->pos;
+		wc.isColorCode = true;
+
+		if (ci->resetColor) {
+			wc.text = ColorResetIndicator;
+		} else {
+			wc.text = ColorCodeIndicator;
+			wc.text += (unsigned char)(255 * ci->color[0]);
+			wc.text += (unsigned char)(255 * ci->color[1]);
+			wc.text += (unsigned char)(255 * ci->color[2]);
+		}
+
 
 		if (wi2->isSpace || wi2->isLineBreak) {
 			while(wi2 != words->end() && (wi2->isSpace || wi2->isLineBreak))
@@ -1406,8 +1523,13 @@ void CglFont::SetTextColor(const float4* color)
 	if (color == NULL) color = &white;
 
 	if (inBeginEnd && !(*color==textColor)) {
-		stripTextColors.push_back(*color);
-		va.EndStrip();
+		if ((va.stripArrayPos - va.stripArray) != (va.drawArrayPos - va.drawArray)) {
+			stripTextColors.push_back(*color);
+			va.EndStrip();
+		} else {
+			float4& back = stripTextColors.back();
+			back = *color;
+		}
 	}
 
 	textColor = *color;
@@ -1419,8 +1541,13 @@ void CglFont::SetOutlineColor(const float4* color)
 	if (color == NULL) color = ChooseOutlineColor(textColor);
 
 	if (inBeginEnd && !(*color==outlineColor)) {
-		stripOutlineColors.push_back(*color);
-		va2.EndStrip();
+		if ((va2.stripArrayPos - va2.stripArray) != (va2.drawArrayPos - va2.drawArray)) {
+			stripOutlineColors.push_back(*color);
+			va2.EndStrip();
+		} else {
+			float4& back = stripOutlineColors.back();
+			back = *color;
+		}
 	}
 
 	outlineColor = *color;
@@ -1434,12 +1561,22 @@ void CglFont::SetColors(const float4* _textColor, const float4* _outlineColor)
 
 	if (inBeginEnd) {
 		if (!(*_textColor==textColor)) {
-			stripTextColors.push_back(*_textColor);
-			va.EndStrip();
+			if ((va.stripArrayPos - va.stripArray) != (va.drawArrayPos - va.drawArray)) {
+				stripTextColors.push_back(*_textColor);
+				va.EndStrip();
+			} else {
+				float4& back = stripTextColors.back();
+				back = *_textColor;
+			}
 		}
 		if (!(*_outlineColor==outlineColor)) {
-			stripOutlineColors.push_back(*_outlineColor);
-			va2.EndStrip();
+			if ((va2.stripArrayPos - va2.stripArray) != (va2.drawArrayPos - va2.drawArray)) {
+				stripOutlineColors.push_back(*_outlineColor);
+				va2.EndStrip();
+			} else {
+				float4& back = stripOutlineColors.back();
+				back = *_outlineColor;
+			}
 		}
 	}
 
@@ -1559,7 +1696,7 @@ void CglFont::RenderString(float x, float y, const float& scaleX, const float& s
 	unsigned int i = 0;
 
 	do {
-		endOfString = SkipColorCodesAndNewLines(str, &i, &newColor, &colorChanged, &skippedLines);
+		endOfString = SkipColorCodesAndNewLines(str, &i, &newColor, &colorChanged, &skippedLines, &baseTextColor);
 
 		if (endOfString)
 			return;
@@ -1612,7 +1749,7 @@ void CglFont::RenderStringShadow(float x, float y, const float& scaleX, const fl
 	unsigned int i = 0;
 
 	do {
-		endOfString = SkipColorCodesAndNewLines(str, &i, &newColor, &colorChanged, &skippedLines);
+		endOfString = SkipColorCodesAndNewLines(str, &i, &newColor, &colorChanged, &skippedLines, &baseTextColor);
 
 		if (endOfString)
 			return;
@@ -1674,7 +1811,7 @@ void CglFont::RenderStringOutlined(float x, float y, const float& scaleX, const
 	unsigned int i = 0;
 
 	do {
-		endOfString = SkipColorCodesAndNewLines(str, &i, &newColor, &colorChanged, &skippedLines);
+		endOfString = SkipColorCodesAndNewLines(str, &i, &newColor, &colorChanged, &skippedLines, &baseTextColor);
 
 		if (endOfString)
 			return;
@@ -1741,8 +1878,8 @@ void CglFont::glPrint(GLfloat x, GLfloat y, float s, const int& options, const s
 
 	//! render in normalized coords (0..1) instead of screencoords (0..~1024)
 	if (options & FONT_NORM) {
-		sizeX *= gu->pixelX;
-		sizeY *= gu->pixelY;
+		sizeX *= globalRendering->pixelX;
+		sizeY *= globalRendering->pixelY;
 	}
 
 	//! horizontal alignment (FONT_LEFT is default)
@@ -1779,14 +1916,9 @@ void CglFont::glPrint(GLfloat x, GLfloat y, float s, const int& options, const s
 		y = (int)y;
 	}
 
-	float4 oldTextColor, oldOultineColor;
-	size_t sts,sos;
-
-	//! backup text & outline colors
-	oldTextColor = textColor;
-	oldOultineColor = outlineColor;
-	sts = stripTextColors.size();
-	sos = stripOutlineColors.size();
+	//! backup text & outline colors (also ::ColorResetIndicator will reset to those)
+	baseTextColor = textColor;
+	baseOutlineColor = outlineColor;
 
 	//! immediate mode?
 	const bool immediate = !inBeginEnd;
@@ -1811,10 +1943,134 @@ void CglFont::glPrint(GLfloat x, GLfloat y, float s, const int& options, const s
 	}
 
 	//! reset text & outline colors (if changed via in text colorcodes)
-	if (stripTextColors.size() > sts)
-		SetTextColor(&oldTextColor);
-	if (stripOutlineColors.size() > sos)
-		SetOutlineColor(&oldOultineColor);
+	SetColors(&baseTextColor,&baseOutlineColor);
+}
+
+void CglFont::glPrintTable(GLfloat x, GLfloat y, float s, const int& options, const std::string& text) {
+	int col = 0;
+	int row = 0;
+	std::vector<std::string> coltext;
+	std::vector<int> coldata;
+	coltext.reserve(text.length());
+	coltext.push_back("");
+	unsigned char curcolor[4];
+	unsigned char defaultcolor[4];
+	defaultcolor[0] = ColorCodeIndicator;
+	for(int i = 0; i < 3; ++i)
+		defaultcolor[i+1] = (unsigned char)(textColor[i]*255.0f);
+	coldata.push_back(*(int *)&defaultcolor);
+	for(int i = 0; i < 4; ++i)
+		curcolor[i] = defaultcolor[i];
+
+	for (int pos = 0; pos < text.length(); pos++) {
+		const char& c = text[pos];
+		switch(c) {
+			//! inline colorcodes
+			case ColorCodeIndicator:
+				for(int i = 0; i < 4 && pos < text.length(); ++i, ++pos) {
+					coltext[col] += text[pos];
+					((unsigned char *)curcolor)[i] = text[pos];
+				}
+				coldata[col] = *(int *)curcolor;
+				--pos;
+				break;
+
+			//! column separator is `\t`==`horizontal tab`
+			case '\x09':
+				++col;
+				if(col >= coltext.size()) {
+					coltext.push_back("");
+					for(int i = 0; i < row; ++i)
+						coltext[col] += '\x0a';
+					coldata.push_back(*(int *)&defaultcolor);
+				}
+				if(coldata[col] != *(int *)curcolor) {
+					for(int i = 0; i < 4; ++i)
+						coltext[col] += curcolor[i];
+					coldata[col] = *(int *)curcolor;
+				}
+				break;
+
+			//! newline
+			case '\x0d': //! CR+LF
+				if (pos+1 < text.length() && text[pos + 1] == '\x0a')
+					pos++;
+			case '\x0a': //! LF
+				for(int i = 0; i < coltext.size(); ++i)
+					coltext[i] += '\x0a';
+				if(coldata[0] != *(int *)curcolor) {
+					for(int i = 0; i < 4; ++i)
+						coltext[0] += curcolor[i];
+					coldata[0] = *(int *)curcolor;
+				}
+				col = 0;
+				++row;
+				break;
+
+			//! printable char
+			default:
+				coltext[col] += c;
+		}
+	}
+
+	float totalWidth = 0.0f;
+	float maxHeight = 0.0f;
+	float minDescender = 0.0f;
+	for(int i = 0; i < coltext.size(); ++i) {
+		float colwidth = GetTextWidth(coltext[i]);
+		coldata[i] = *(int *)&colwidth;
+		totalWidth += colwidth;
+		float textDescender;
+		float textHeight = GetTextHeight(coltext[i], &textDescender);
+		if(textHeight > maxHeight)
+			maxHeight = textHeight;
+		if(textDescender < minDescender)
+			minDescender = textDescender;
+	}
+
+	//! s := scale or absolute size?
+	float ss = s;
+	if (options & FONT_SCALE) {
+		ss *= fontSize;
+	}
+
+	float sizeX = ss, sizeY = ss;
+
+	//! render in normalized coords (0..1) instead of screencoords (0..~1024)
+	if (options & FONT_NORM) {
+		sizeX *= globalRendering->pixelX;
+		sizeY *= globalRendering->pixelY;
+	}
+
+	//! horizontal alignment (FONT_LEFT is default)
+	if (options & FONT_CENTER) {
+		x -= sizeX * 0.5f * totalWidth;
+	} else if (options & FONT_RIGHT) {
+		x -= sizeX * totalWidth;
+	}
+
+	//! vertical alignment
+	if (options & FONT_BASELINE) {
+		//! nothing
+	} else if (options & FONT_DESCENDER) {
+		y -= sizeY * fontDescender;
+	} else if (options & FONT_VCENTER) {
+		y -= sizeY * 0.5f * maxHeight;
+		y -= sizeY * 0.5f * minDescender;
+	} else if (options & FONT_TOP) {
+		y -= sizeY * maxHeight;
+	} else if (options & FONT_ASCENDER) {
+		y -= sizeY * fontDescender;
+		y -= sizeY;
+	} else if (options & FONT_BOTTOM) {
+		y -= sizeY * minDescender;
+	}
+
+	for(int i = 0; i < coltext.size(); ++i) {
+		glPrint(x, y, s, (options | FONT_BASELINE) & ~(FONT_RIGHT | FONT_CENTER), coltext[i]);
+		int colwidth = coldata[i];
+		x += sizeX * *(float *)&colwidth;
+	}
 }
 
 
