@@ -108,7 +108,7 @@ void CFeature::PostLoad()
 {
 	def = featureHandler->GetFeatureDef(defName);
 	if (def->drawType == DRAWTYPE_MODEL) {
-		model = LoadModel(def);
+		model = def->LoadModel();
 		height = model->height;
 		SetRadius(model->radius);
 		midPos = pos + model->relMidPos;
@@ -158,7 +158,7 @@ void CFeature::Initialize(const float3& _pos, const FeatureDef* _def, short int
 	noSelect = def->noSelect;
 
 	if (def->drawType == DRAWTYPE_MODEL) {
-		model = LoadModel(def);
+		model = def->LoadModel();
 		if (!model) {
 			logOutput.Print("Features: Couldn't load model for " + defName);
 			SetRadius(0.0f);
@@ -354,7 +354,7 @@ bool CFeature::AddBuildPower(float amount, CUnit* builder)
 }
 
 
-void CFeature::DoDamage(const DamageArray& damages, CUnit* attacker,const float3& impulse)
+void CFeature::DoDamage(const DamageArray& damages, const float3& impulse)
 {
 	if (damages.paralyzeDamageTime) {
 		return; // paralyzers do not damage features
@@ -390,7 +390,7 @@ void CFeature::DoDamage(const DamageArray& damages, CUnit* attacker,const float3
 
 void CFeature::Kill(float3& impulse) {
 	DamageArray damage;
-	DoDamage(damage * (health + 1), 0, impulse);
+	DoDamage(damage * (health + 1), impulse);
 }
 
 
