@@ -1,3 +1,5 @@
+/* This file is part of the Spring engine (GPL v2 or later), see LICENSE.html */
+
 #include "TeamBase.h"
 
 #include <cstdlib>
@@ -25,47 +27,47 @@ TeamBase::TeamBase() :
 leader(-1),
 handicap(1),
 startPos(-100,-100,-100),
-teamAllyteam(-1),
-startMetal(-1.0),
-startEnergy(-1.0)
+teamStartNum(-1),
+teamAllyteam(-1)
 {
+	color[0] = 255;
+	color[1] = 255;
+	color[2] = 255;
+	color[3] = 255;
 }
 
 void TeamBase::SetValue(const std::string& key, const std::string& value)
 {
-	if (key == "handicap")
+	if (key == "handicap") {
 		handicap = std::atof(value.c_str()) / 100 + 1;
-	else if (key == "teamleader")
+	}
+	else if (key == "teamleader") {
 		leader = std::atoi(value.c_str());
-	else if (key == "side")
+	}
+	else if (key == "side") {
 		side = StringToLower(value);
-	else if (key == "allyteam")
+	}
+	else if (key == "allyteam") {
 		teamAllyteam = std::atoi(value.c_str());
-	else if (key == "startmetal")
-		startMetal = std::atof(value.c_str());
-	else if (key == "startenergy")
-		startEnergy = std::atof(value.c_str());
-	else if (key == "rgbcolor")
-	{
+	}
+	else if (key == "rgbcolor") {
 		std::istringstream buf(value);
-		for (size_t b = 0; b < 3; ++b)
-		{
+		for (size_t b = 0; b < 3; ++b) {
 			float tmp;
 			buf >> tmp;
 			color[b] = tmp * 255;
 		}
 		color[3] = 255;
 	}
-	else if (key == "startposx")
-	{
+	else if (key == "startposx") {
 		if (!value.empty())
 			startPos.x = atoi(value.c_str());
 	}
-	else if (key == "startposz")
-	{
+	else if (key == "startposz") {
 		if (!value.empty())
 			startPos.z = atoi(value.c_str());
 	}
-	else
+	else {
 		customValues[key] = value;
+	}
 }
