@@ -36,7 +36,6 @@ public:
 
 	bool OnSlope(void);
 
-	float baseTurnRate;
 	float turnRate;
 	float accRate;
 	float decRate;
@@ -48,6 +47,7 @@ public:
 	short int deltaHeading;
 
 	float3 flatFrontDir;
+	float3 waypointDir;
 
 	unsigned int pathId;
 	float goalRadius;
@@ -87,14 +87,11 @@ protected:
 	float Distance2D(CSolidObject *object1, CSolidObject *object2, float marginal = 0.0f);
 
 	void GetNewPath();
-	void GetNextWaypoint(bool);
+	void GetNextWaypoint();
 
 	float BreakingDistance(float speed);
 	float3 Here();
 
-	float MinDistanceToWaypoint();
-	float MaxDistanceToWaypoint();
-
 	void StartEngine();
 	void StopEngine();
 
