@@ -10,8 +10,11 @@
 
 #include "lib/minizip/zip.h"
 #include "mmgr.h"
+
+#include "PathAllocator.h"
 #include "PathCache.h"
 #include "PathFinder.h"
+#include "PathFinderDef.h"
 #include "Map/ReadMap.h"
 #include "Sim/MoveTypes/MoveInfo.h"
 #include "Sim/MoveTypes/MoveMath/MoveMath.h"
@@ -25,50 +28,28 @@
 
 #define PATHDEBUG false
 
-
-const unsigned int PATHDIR_LEFT = 0;		// +x
-const unsigned int PATHDIR_LEFT_UP = 1;		// +x+z
-const unsigned int PATHDIR_UP = 2;			// +z
-const unsigned int PATHDIR_RIGHT_UP = 3;	// -x+z
-const unsigned int PATHDIR_RIGHT = 4;		// -x
-const unsigned int PATHDIR_RIGHT_DOWN = 5;	// -x-z
-const unsigned int PATHDIR_DOWN = 6;		// -z
-const unsigned int PATHDIR_LEFT_DOWN = 7;	// +x-z
-
-const unsigned int PATHOPT_OPEN = 8;
-const unsigned int PATHOPT_CLOSED = 16;
-const unsigned int PATHOPT_FORBIDDEN = 32;
-const unsigned int PATHOPT_BLOCKED = 64;
-const unsigned int PATHOPT_SEARCHRELATED = (PATHOPT_OPEN | PATHOPT_CLOSED | PATHOPT_FORBIDDEN | PATHOPT_BLOCKED);
-const unsigned int PATHOPT_OBSOLETE = 128;
-
-const unsigned int PATHESTIMATOR_VERSION = 44;
-const float PATHCOST_INFINITY = 10000000;
-const int SQUARES_TO_UPDATE = 600;
-
 const std::string pathDir = "cache/paths/";
 
 #if !defined(USE_MMGR)
-void* CPathEstimator::operator new(size_t size) { return ::PF_ALLOC(size); }
-void CPathEstimator::operator delete(void* p, size_t size) { ::PF_FREE(p, size); }
+void* CPathEstimator::operator new(size_t size) { return PathAllocator::Alloc(size); }
+void CPathEstimator::operator delete(void* p, size_t size) { PathAllocator::Free(p, size); }
 #endif
 
-/**
- * Constructor, loads precalculated data if it exists
- */
-CPathEstimator::CPathEstimator(CPathFinder* pf, unsigned int BSIZE, unsigned int mmOpt, std::string cacheFileName, const std::string& map):
+
+
+CPathEstimator::CPathEstimator(CPathFinder* pf, unsigned int BSIZE, const std::string& cacheFileName, const std::string& map):
 	BLOCK_SIZE(BSIZE),
 	BLOCK_PIXEL_SIZE(BSIZE * SQUARE_SIZE),
 	BLOCKS_TO_UPDATE(SQUARES_TO_UPDATE / (BLOCK_SIZE * BLOCK_SIZE) + 1),
 	pathFinder(pf),
 	nbrOfBlocksX(gs->mapx / BLOCK_SIZE),
 	nbrOfBlocksZ(gs->mapy / BLOCK_SIZE),
-	nbrOfBlocks(nbrOfBlocksX * nbrOfBlocksZ),
-	openBlockBufferIndex(0),
-	moveMathOptions(mmOpt),
 	pathChecksum(0),
-	offsetBlockNum(nbrOfBlocks),costBlockNum(nbrOfBlocks),
-	lastOffsetMessage(-1),lastCostMessage(-1)
+	offsetBlockNum(nbrOfBlocksX * nbrOfBlocksZ),
+	costBlockNum(nbrOfBlocksX * nbrOfBlocksZ),
+	nextOffsetMessage(-1),
+	nextCostMessage(-1),
+	blockStates(int2(nbrOfBlocksX, nbrOfBlocksZ), int2(gs->mapx, gs->mapy))
 {
 	// these give the changes in (x, z) coors
 	// when moving one step in given direction
@@ -92,10 +73,9 @@ CPathEstimator::CPathEstimator(CPathFinder* pf, unsigned int BSIZE, unsigned int
 	goalSqrOffset.x = BLOCK_SIZE / 2;
 	goalSqrOffset.y = BLOCK_SIZE / 2;
 
-	blockState = new BlockInfo[nbrOfBlocks];
-	nbrOfVertices = moveinfo->moveData.size() * nbrOfBlocks * PATH_DIRECTION_VERTICES;
-	vertex = new float[nbrOfVertices];
+	vertices.resize(moveinfo->moveData.size() * blockStates.GetSize() * PATH_DIRECTION_VERTICES, 0.0f);
 
+	// load precalculated data if it exists
 	InitEstimator(cacheFileName, map);
 
 	// As all vertexes are bidirectional and have equal values
@@ -106,26 +86,23 @@ CPathEstimator::CPathEstimator(CPathFinder* pf, unsigned int BSIZE, unsigned int
 	directionVertex[PATHDIR_LEFT_UP   ] = PATHDIR_LEFT_UP;
 	directionVertex[PATHDIR_UP        ] = PATHDIR_UP;
 	directionVertex[PATHDIR_RIGHT_UP  ] = PATHDIR_RIGHT_UP;
-	directionVertex[PATHDIR_RIGHT     ] = int(PATHDIR_LEFT) - PATH_DIRECTION_VERTICES;
-	directionVertex[PATHDIR_RIGHT_DOWN] = int(PATHDIR_LEFT_UP) - (nbrOfBlocksX * PATH_DIRECTION_VERTICES) - PATH_DIRECTION_VERTICES;
-	directionVertex[PATHDIR_DOWN      ] = int(PATHDIR_UP) - (nbrOfBlocksX * PATH_DIRECTION_VERTICES);
+	directionVertex[PATHDIR_RIGHT     ] = int(PATHDIR_LEFT    ) - PATH_DIRECTION_VERTICES;
+	directionVertex[PATHDIR_RIGHT_DOWN] = int(PATHDIR_LEFT_UP ) - (nbrOfBlocksX * PATH_DIRECTION_VERTICES) - PATH_DIRECTION_VERTICES;
+	directionVertex[PATHDIR_DOWN      ] = int(PATHDIR_UP      ) - (nbrOfBlocksX * PATH_DIRECTION_VERTICES);
 	directionVertex[PATHDIR_LEFT_DOWN ] = int(PATHDIR_RIGHT_UP) - (nbrOfBlocksX * PATH_DIRECTION_VERTICES) + PATH_DIRECTION_VERTICES;
 
 	pathCache = new CPathCache(nbrOfBlocksX, nbrOfBlocksZ);
 }
 
-
-/**
- * Free all used memory
- */
 CPathEstimator::~CPathEstimator()
 {
-	for (int i = 0; i < nbrOfBlocks; i++)
-		delete[] blockState[i].sqrCenter;
+	for (int i = 0; i < blockStates.GetSize(); i++)
+		blockStates[i].nodeOffsets.clear();
 
-	delete[] blockState;
-	delete[] vertex;
 	delete pathCache;
+
+	blockStates.Clear();
+	vertices.clear();
 }
 
 
@@ -154,14 +131,12 @@ void CPathEstimator::InitEstimator(const std::string& cacheFileName, const std::
 	InitVertices();
 	InitBlocks();
 
-	PrintLoadMsg("Reading estimate path costs");
+	char calcMsg[512];
+	sprintf(calcMsg, "Reading Estimate PathCosts [%d]", BLOCK_SIZE);
+	PrintLoadMsg(calcMsg);
 
 	if (!ReadFile(cacheFileName, map)) {
-		char calcMsg[512];
-		sprintf(calcMsg, "Analyzing map accessibility [%d]", BLOCK_SIZE);
-		PrintLoadMsg(calcMsg);
-
-		pathBarrier=new boost::barrier(numThreads);
+		pathBarrier = new boost::barrier(numThreads);
 
 		// Start threads if applicable
 		for(size_t i=1; i<numThreads; ++i) {
@@ -180,40 +155,40 @@ void CPathEstimator::InitEstimator(const std::string& cacheFileName, const std::
 
 		delete pathBarrier;
 
-		PrintLoadMsg("Writing path data file...");
+		PrintLoadMsg("PathCosts: writing", true);
 		WriteFile(cacheFileName, map);
+		PrintLoadMsg("PathCosts: written", true);
 	}
 }
 
 
 void CPathEstimator::InitVertices() {
-	for (unsigned int i = 0; i < nbrOfVertices; i++)
-		vertex[i] = PATHCOST_INFINITY;
+	for (unsigned int i = 0; i < vertices.size(); i++)
+		vertices[i] = PATHCOST_INFINITY;
 }
 
 void CPathEstimator::InitBlocks() {
-	for (int idx = 0; idx < nbrOfBlocks; idx++) {
-		int x = idx % nbrOfBlocksX;
-		int z = idx / nbrOfBlocksX;
-		int blockNr = z * nbrOfBlocksX + x;
+	for (int idx = 0; idx < blockStates.GetSize(); idx++) {
+		const int x = idx % nbrOfBlocksX;
+		const int z = idx / nbrOfBlocksX;
+		const int blockNr = z * nbrOfBlocksX + x;
 
-		blockState[blockNr].cost = PATHCOST_INFINITY;
-		blockState[blockNr].options = 0;
-		blockState[blockNr].parentBlock.x = -1;
-		blockState[blockNr].parentBlock.y = -1;
-		blockState[blockNr].sqrCenter = new int2[moveinfo->moveData.size()];
+		blockStates[blockNr].nodeOffsets.resize(moveinfo->moveData.size());
 	}
 }
 
 
 void CPathEstimator::CalcOffsetsAndPathCosts(int thread) {
+	//! reset FPU state for synced computations
 	streflop_init<streflop::Simple>();
+
 	// NOTE: EstimatePathCosts() [B] is temporally dependent on CalculateBlockOffsets() [A],
 	// A must be completely finished before B_i can be safely called. This means we cannot
 	// let thread i execute (A_i, B_i), but instead have to split the work such that every
 	// thread finishes its part of A before any starts B_i.
-	int nbr = nbrOfBlocks - 1;
+	const int nbr = blockStates.GetSize() - 1;
 	int i;
+
 	while((i = --offsetBlockNum) >= 0)
 		CalculateBlockOffsets(nbr - i, thread);
 
@@ -226,15 +201,12 @@ void CPathEstimator::CalcOffsetsAndPathCosts(int thread) {
 
 void CPathEstimator::CalculateBlockOffsets(int idx, int thread)
 {
-	int x = idx % nbrOfBlocksX;
-	int z = idx / nbrOfBlocksX;
+	const int x = idx % nbrOfBlocksX;
+	const int z = idx / nbrOfBlocksX;
 
-	if (thread == 0 && (idx/1000)!=lastOffsetMessage) {
-		lastOffsetMessage=idx/1000;
-		char calcMsg[128];
-		sprintf(calcMsg, "Block offset: %d of %d (size %d)", lastOffsetMessage*1000, nbrOfBlocks, BLOCK_SIZE);
-		net->Send(CBaseNetProtocol::Get().SendCPUUsage(BLOCK_SIZE | (lastOffsetMessage<<8)));
-		PrintLoadMsg(calcMsg);
+	if (thread == 0 && idx >= nextOffsetMessage) {
+		nextOffsetMessage = idx + blockStates.GetSize() / 16;
+		net->Send(CBaseNetProtocol::Get().SendCPUUsage(BLOCK_SIZE | (idx<<8)));
 	}
 
 	for (vector<MoveData*>::iterator mi = moveinfo->moveData.begin(); mi != moveinfo->moveData.end(); mi++)
@@ -242,15 +214,15 @@ void CPathEstimator::CalculateBlockOffsets(int idx, int thread)
 }
 
 void CPathEstimator::EstimatePathCosts(int idx, int thread) {
-	int x = idx % nbrOfBlocksX;
-	int z = idx / nbrOfBlocksX;
+	const int x = idx % nbrOfBlocksX;
+	const int z = idx / nbrOfBlocksX;
 
-	if (thread == 0 && (idx/1000)!=lastCostMessage) {
-		lastCostMessage=idx/1000;
+	if (thread == 0 && idx >= nextCostMessage) {
+		nextCostMessage = idx + blockStates.GetSize() / 16;
 		char calcMsg[128];
-		sprintf(calcMsg, "Path cost: %d of %d (size %d)", lastCostMessage*1000, nbrOfBlocks, BLOCK_SIZE);
-		net->Send(CBaseNetProtocol::Get().SendCPUUsage(0x1 | BLOCK_SIZE | (lastCostMessage<<8)));
-		PrintLoadMsg(calcMsg);
+		sprintf(calcMsg, "PathCosts: precached %d of %d", idx, blockStates.GetSize());
+		net->Send(CBaseNetProtocol::Get().SendCPUUsage(0x1 | BLOCK_SIZE | (idx<<8)));
+		PrintLoadMsg(calcMsg, (idx != 0));
 	}
 
 	for (vector<MoveData*>::iterator mi = moveinfo->moveData.begin(); mi != moveinfo->moveData.end(); mi++)
@@ -267,36 +239,39 @@ void CPathEstimator::EstimatePathCosts(int idx, int thread) {
  */
 void CPathEstimator::FindOffset(const MoveData& moveData, int blockX, int blockZ) {
 	// lower corner position of block
-	int lowerX = blockX * BLOCK_SIZE;
-	int lowerZ = blockZ * BLOCK_SIZE;
+	const int lowerX = blockX * BLOCK_SIZE;
+	const int lowerZ = blockZ * BLOCK_SIZE;
+	const unsigned int blockArea = (BLOCK_SIZE * BLOCK_SIZE) / SQUARE_SIZE;
 
-	float best = 100000000.0f;
-	unsigned int bestX = BLOCK_SIZE >> 1;
-	unsigned int bestZ = BLOCK_SIZE >> 1;
-	unsigned int num = (BLOCK_SIZE * BLOCK_SIZE) >> 3;
+	unsigned int bestPosX = BLOCK_SIZE >> 1;
+	unsigned int bestPosZ = BLOCK_SIZE >> 1;
+
+	float bestCost = std::numeric_limits<float>::max();
 
 	// search for an accessible position
 	for (unsigned int z = 1; z < BLOCK_SIZE; z += 2) {
 		for (unsigned int x = 1; x < BLOCK_SIZE; x += 2) {
-			int dx = x - (BLOCK_SIZE >> 1);
-			int dz = z - (BLOCK_SIZE >> 1);
-			float cost = (dx * dx + dz * dz) + num / (0.001f + moveData.moveMath->SpeedMod(moveData, (int)(lowerX + x), (int)(lowerZ + z)));
-			int mask = CMoveMath::BLOCK_STRUCTURE | CMoveMath::BLOCK_TERRAIN;
+			const int dx = x - (BLOCK_SIZE >> 1);
+			const int dz = z - (BLOCK_SIZE >> 1);
+			const float speedMod = moveData.moveMath->SpeedMod(moveData, int(lowerX + x), int(lowerZ + z));
 
-			if (moveData.moveMath->IsBlocked2(moveData, lowerX + x, lowerZ + z) & mask) {
-				cost += 1000000.0f;
+			float cost = (dx * dx + dz * dz) + (blockArea / (0.001f + speedMod));
+
+			if (moveData.moveMath->IsBlocked2(moveData, lowerX + x, lowerZ + z) & CMoveMath::BLOCK_STRUCTURE) {
+				cost = std::numeric_limits<float>::infinity();
 			}
-			if (cost < best) {
-				best = cost;
-				bestX = x;
-				bestZ = z;
+
+			if (cost < bestCost) {
+				bestCost = cost;
+				bestPosX = x;
+				bestPosZ = z;
 			}
 		}
 	}
 
 	// store the offset found
-	blockState[blockZ * nbrOfBlocksX + blockX].sqrCenter[moveData.pathType].x = blockX * BLOCK_SIZE + bestX;
-	blockState[blockZ * nbrOfBlocksX + blockX].sqrCenter[moveData.pathType].y = blockZ * BLOCK_SIZE + bestZ;
+	blockStates[blockZ * nbrOfBlocksX + blockX].nodeOffsets[moveData.pathType].x = blockX * BLOCK_SIZE + bestPosX;
+	blockStates[blockZ * nbrOfBlocksX + blockX].nodeOffsets[moveData.pathType].y = blockZ * BLOCK_SIZE + bestPosZ;
 }
 
 
@@ -315,47 +290,47 @@ void CPathEstimator::CalculateVertices(const MoveData& moveData, int blockX, int
  */
 void CPathEstimator::CalculateVertex(const MoveData& moveData, int parentBlockX, int parentBlockZ, unsigned int direction, int thread) {
 	// initial calculations
-	int parentBlocknr = parentBlockZ * nbrOfBlocksX + parentBlockX;
-	int childBlockX = parentBlockX + directionVector[direction].x;
-	int childBlockZ = parentBlockZ + directionVector[direction].y;
-	int vertexNbr = moveData.pathType * nbrOfBlocks * PATH_DIRECTION_VERTICES + parentBlocknr * PATH_DIRECTION_VERTICES + direction;
+	const int parentBlocknr = parentBlockZ * nbrOfBlocksX + parentBlockX;
+	const int childBlockX = parentBlockX + directionVector[direction].x;
+	const int childBlockZ = parentBlockZ + directionVector[direction].y;
+	const int vertexNbr = moveData.pathType * blockStates.GetSize() * PATH_DIRECTION_VERTICES + parentBlocknr * PATH_DIRECTION_VERTICES + direction;
 
 	// outside map?
 	if (childBlockX < 0 || childBlockZ < 0 ||
 		childBlockX >= nbrOfBlocksX || childBlockZ >= nbrOfBlocksZ) {
-		vertex[vertexNbr] = PATHCOST_INFINITY;
+		vertices[vertexNbr] = PATHCOST_INFINITY;
 		return;
 	}
 
 	// start position
-	int parentXSquare = blockState[parentBlocknr].sqrCenter[moveData.pathType].x;
-	int parentZSquare = blockState[parentBlocknr].sqrCenter[moveData.pathType].y;
-	float3 startPos = SquareToFloat3(parentXSquare, parentZSquare);
+	const int parentXSquare = blockStates[parentBlocknr].nodeOffsets[moveData.pathType].x;
+	const int parentZSquare = blockStates[parentBlocknr].nodeOffsets[moveData.pathType].y;
+	const float3 startPos = SquareToFloat3(parentXSquare, parentZSquare);
 
 	// goal position
-	int childBlocknr = childBlockZ * nbrOfBlocksX + childBlockX;
-	int childXSquare = blockState[childBlocknr].sqrCenter[moveData.pathType].x;
-	int childZSquare = blockState[childBlocknr].sqrCenter[moveData.pathType].y;
-	float3 goalPos = SquareToFloat3(childXSquare, childZSquare);
+	const int childBlocknr = childBlockZ * nbrOfBlocksX + childBlockX;
+	const int childXSquare = blockStates[childBlocknr].nodeOffsets[moveData.pathType].x;
+	const int childZSquare = blockStates[childBlocknr].nodeOffsets[moveData.pathType].y;
+	const float3 goalPos = SquareToFloat3(childXSquare, childZSquare);
 
 	// PathFinder definition
 	CRangedGoalWithCircularConstraint pfDef(startPos, goalPos, 0, 1.1f, 2);
 
 	// the path to find
-	Path path;
-	SearchResult result;
+	IPath::Path path;
+	IPath::SearchResult result;
 
 	// since CPathFinder::GetPath() is not thread-safe,
 	// use this thread's "private" CPathFinder instance
 	// (rather than locking pathFinder->GetPath()) if we
 	// are in one
-	result = pathFinders[thread]->GetPath(moveData, startPos, pfDef, path, false, true, CPathFinder::MAX_SEARCHED_SQUARES, false);
+	result = pathFinders[thread]->GetPath(moveData, startPos, pfDef, path, false, true, MAX_SEARCHED_NODES_PE, false, 0, true);
 
 	// store the result
-	if (result == Ok)
-		vertex[vertexNbr] = path.pathCost;
+	if (result == IPath::Ok)
+		vertices[vertexNbr] = path.pathCost;
 	else
-		vertex[vertexNbr] = PATHCOST_INFINITY;
+		vertices[vertexNbr] = PATHCOST_INFINITY;
 }
 
 
@@ -392,16 +367,16 @@ void CPathEstimator::MapChanged(unsigned int x1, unsigned int z1, unsigned int x
 	// bi-directional vertices
 	for (int z = upperZ; z >= lowerZ; z--) {
 		for (int x = upperX; x >= lowerX; x--) {
-			if (!(blockState[z * nbrOfBlocksX + x].options & PATHOPT_OBSOLETE)) {
+			if (!(blockStates[z * nbrOfBlocksX + x].nodeMask & PATHOPT_OBSOLETE)) {
 				vector<MoveData*>::iterator mi;
 
 				for (mi = moveinfo->moveData.begin(); mi < moveinfo->moveData.end(); mi++) {
 					SingleBlock sb;
-					sb.block.x = x;
-					sb.block.y = z;
-					sb.moveData = *mi;
+						sb.block.x = x;
+						sb.block.y = z;
+						sb.moveData = *mi;
 					needUpdate.push_back(sb);
-					blockState[z * nbrOfBlocksX + x].options |= PATHOPT_OBSOLETE;
+					blockStates[z * nbrOfBlocksX + x].nodeMask |= PATHOPT_OBSOLETE;
 				}
 			}
 		}
@@ -420,10 +395,11 @@ void CPathEstimator::Update() {
 		// next block in line
 		SingleBlock sb = needUpdate.front();
 		needUpdate.pop_front();
-		int blocknr = sb.block.y * nbrOfBlocksX + sb.block.x;
+
+		const int blocknr = sb.block.y * nbrOfBlocksX + sb.block.x;
 
 		// check if it's already updated
-		if (!(blockState[blocknr].options & PATHOPT_OBSOLETE))
+		if (!(blockStates[blocknr].nodeMask & PATHOPT_OBSOLETE))
 			continue;
 
 		// no, update the block
@@ -432,7 +408,7 @@ void CPathEstimator::Update() {
 
 		// mark it as updated
 		if (sb.moveData == moveinfo->moveData.back()) {
-			blockState[blocknr].options &= ~PATHOPT_OBSOLETE;
+			blockStates[blocknr].nodeMask &= ~PATHOPT_OBSOLETE;
 		}
 
 		// one block updated
@@ -448,7 +424,7 @@ IPath::SearchResult CPathEstimator::GetPath(
 	const MoveData& moveData,
 	float3 start,
 	const CPathFinderDef& peDef,
-	Path& path,
+	IPath::Path& path,
 	unsigned int maxSearchedBlocks,
 	bool synced
 ) {
@@ -459,7 +435,7 @@ IPath::SearchResult CPathEstimator::GetPath(
 	path.pathCost = PATHCOST_INFINITY;
 
 	// initial calculations
-	maxBlocksToBeSearched = std::min(maxSearchedBlocks, MAX_SEARCHED_BLOCKS - 8U);
+	maxBlocksToBeSearched = std::min(maxSearchedBlocks, MAX_SEARCHED_NODES_PE - 8U);
 
 	startBlock.x = (int)(start.x / BLOCK_PIXEL_SIZE);
 	startBlock.y = (int)(start.z / BLOCK_PIXEL_SIZE);
@@ -478,13 +454,13 @@ IPath::SearchResult CPathEstimator::GetPath(
 	}
 
 	// oterhwise search
-	SearchResult result = InitSearch(moveData, peDef);
+	IPath::SearchResult result = InitSearch(moveData, peDef, synced);
 
 	// if search successful, generate new path
-	if (result == Ok || result == GoalOutOfRange) {
+	if (result == IPath::Ok || result == IPath::GoalOutOfRange) {
 		FinishSearch(moveData, path);
 
-		if (synced) {
+		if (synced && result == IPath::Ok) {
 			// add succesful paths to the cache (NOTE: only when in synced context)
 			pathCache->AddPath(&path, result, startBlock, goalBlock, peDef.sqGoalRadius, moveData.pathType);
 		}
@@ -492,15 +468,15 @@ IPath::SearchResult CPathEstimator::GetPath(
 		if (PATHDEBUG) {
 			LogObject() << "PE: Search completed.\n";
 			LogObject() << "Tested blocks: " << testedBlocks << "\n";
-			LogObject() << "Open blocks: " << openBlockBufferIndex << "\n";
-			LogObject() << "Path length: " << (int)(path.path.size()) << "\n";
+			LogObject() << "Open blocks: " << openBlockBuffer.GetSize() << "\n";
+			LogObject() << "Path length: " << path.path.size() << "\n";
 			LogObject() << "Path cost: " << path.pathCost << "\n";
 		}
 	} else {
 		if (PATHDEBUG) {
 			LogObject() << "PE: Search failed!\n";
 			LogObject() << "Tested blocks: " << testedBlocks << "\n";
-			LogObject() << "Open blocks: " << openBlockBufferIndex << "\n";
+			LogObject() << "Open blocks: " << openBlockBuffer.GetSize() << "\n";
 		}
 	}
 
@@ -508,31 +484,34 @@ IPath::SearchResult CPathEstimator::GetPath(
 }
 
 
-/**
- * Make some initial calculations and preparations
- */
-IPath::SearchResult CPathEstimator::InitSearch(const MoveData& moveData, const CPathFinderDef& peDef) {
+// set up the starting point of the search
+IPath::SearchResult CPathEstimator::InitSearch(const MoveData& moveData, const CPathFinderDef& peDef, bool synced) {
 	// is starting square inside goal area?
-	int xSquare = blockState[startBlocknr].sqrCenter[moveData.pathType].x;
-	int zSquare = blockState[startBlocknr].sqrCenter[moveData.pathType].y;
+	const int xSquare = blockStates[startBlocknr].nodeOffsets[moveData.pathType].x;
+	const int zSquare = blockStates[startBlocknr].nodeOffsets[moveData.pathType].y;
+
 	if (peDef.IsGoal(xSquare, zSquare))
-		return CantGetCloser;
+		return IPath::CantGetCloser;
 
 	// no, clean the system from last search
 	ResetSearch();
 
 	// mark and store the start-block
-	blockState[startBlocknr].options |= PATHOPT_OPEN;
-	blockState[startBlocknr].cost = 0;
+	blockStates[startBlocknr].nodeMask |= PATHOPT_OPEN;
+	blockStates[startBlocknr].fCost = 0.0f;
+	blockStates[startBlocknr].gCost = 0.0f;
+	blockStates.SetMaxFCost(0.0f);
+	blockStates.SetMaxGCost(0.0f);
+
 	dirtyBlocks.push_back(startBlocknr);
 
-	openBlockBufferIndex = 0;
+	openBlockBuffer.SetSize(0);
 	// add the starting block to the open-blocks-queue
-	OpenBlock* ob = &openBlockBuffer[openBlockBufferIndex];
-		ob->cost = 0;
-		ob->currentCost = 0;
-		ob->block = startBlock;
-		ob->blocknr = startBlocknr;
+	PathNode* ob = openBlockBuffer.GetNode(openBlockBuffer.GetSize());
+		ob->fCost   = 0.0f;
+		ob->gCost   = 0.0f;
+		ob->nodePos = startBlock;
+		ob->nodeNum = startBlocknr;
 	openBlocks.push(ob);
 
 	// mark starting point as best found position
@@ -543,75 +522,76 @@ IPath::SearchResult CPathEstimator::InitSearch(const MoveData& moveData, const C
 	goalSqrOffset = peDef.GoalSquareOffset(BLOCK_SIZE);
 
 	// perform the search
-	SearchResult result = DoSearch(moveData, peDef);
+	IPath::SearchResult result = DoSearch(moveData, peDef, synced);
 
 	// if no improvements are found, then return CantGetCloser instead
-	if (goalBlock.x == startBlock.x && goalBlock.y == startBlock.y)
-		return CantGetCloser;
-	else
-		return result;
+	if (goalBlock.x == startBlock.x && goalBlock.y == startBlock.y) {
+		return IPath::CantGetCloser;
+	}
+
+	return result;
 }
 
 
 /**
  * Performs the actual search.
  */
-IPath::SearchResult CPathEstimator::DoSearch(const MoveData& moveData, const CPathFinderDef& peDef) {
+IPath::SearchResult CPathEstimator::DoSearch(const MoveData& moveData, const CPathFinderDef& peDef, bool synced) {
 	bool foundGoal = false;
 
-	while (!openBlocks.empty() && (openBlockBufferIndex < maxBlocksToBeSearched)) {
+	while (!openBlocks.empty() && (openBlockBuffer.GetSize() < maxBlocksToBeSearched)) {
 		// get the open block with lowest cost
-		OpenBlock* ob = openBlocks.top();
+		PathNode* ob = const_cast<PathNode*>(openBlocks.top());
 		openBlocks.pop();
 
 		// check if the block has been marked as unaccessible during its time in the queue
-		if (blockState[ob->blocknr].options & (PATHOPT_BLOCKED | PATHOPT_CLOSED | PATHOPT_FORBIDDEN))
+		if (blockStates[ob->nodeNum].nodeMask & (PATHOPT_BLOCKED | PATHOPT_CLOSED | PATHOPT_FORBIDDEN))
 			continue;
 
 		// no, check if the goal is already reached
-		int xBSquare = blockState[ob->blocknr].sqrCenter[moveData.pathType].x;
-		int zBSquare = blockState[ob->blocknr].sqrCenter[moveData.pathType].y;
-		int xGSquare = ob->block.x * BLOCK_SIZE + goalSqrOffset.x;
-		int zGSquare = ob->block.y * BLOCK_SIZE + goalSqrOffset.y;
+		const int xBSquare = blockStates[ob->nodeNum].nodeOffsets[moveData.pathType].x;
+		const int zBSquare = blockStates[ob->nodeNum].nodeOffsets[moveData.pathType].y;
+		const int xGSquare = ob->nodePos.x * BLOCK_SIZE + goalSqrOffset.x;
+		const int zGSquare = ob->nodePos.y * BLOCK_SIZE + goalSqrOffset.y;
 
 		if (peDef.IsGoal(xBSquare, zBSquare) || peDef.IsGoal(xGSquare, zGSquare)) {
-			goalBlock = ob->block;
+			goalBlock = ob->nodePos;
 			goalHeuristic = 0;
 			foundGoal = true;
 			break;
 		}
 
 		// no, test the 8 surrounding blocks
-		// NOTE: each of these calls increments openBlockBufferIndex by 1, so
-		// maxBlocksToBeSearched is always less than <MAX_SEARCHED_BLOCKS - 8>
-		TestBlock(moveData, peDef, *ob, PATHDIR_LEFT);
-		TestBlock(moveData, peDef, *ob, PATHDIR_LEFT_UP);
-		TestBlock(moveData, peDef, *ob, PATHDIR_UP);
-		TestBlock(moveData, peDef, *ob, PATHDIR_RIGHT_UP);
-		TestBlock(moveData, peDef, *ob, PATHDIR_RIGHT);
-		TestBlock(moveData, peDef, *ob, PATHDIR_RIGHT_DOWN);
-		TestBlock(moveData, peDef, *ob, PATHDIR_DOWN);
-		TestBlock(moveData, peDef, *ob, PATHDIR_LEFT_DOWN);
+		// NOTE: each of these calls increments openBlockBuffer.idx by 1, so
+		// maxBlocksToBeSearched is always less than <MAX_SEARCHED_NODES_PE - 8>
+		TestBlock(moveData, peDef, *ob, PATHDIR_LEFT,       synced);
+		TestBlock(moveData, peDef, *ob, PATHDIR_LEFT_UP,    synced);
+		TestBlock(moveData, peDef, *ob, PATHDIR_UP,         synced);
+		TestBlock(moveData, peDef, *ob, PATHDIR_RIGHT_UP,   synced);
+		TestBlock(moveData, peDef, *ob, PATHDIR_RIGHT,      synced);
+		TestBlock(moveData, peDef, *ob, PATHDIR_RIGHT_DOWN, synced);
+		TestBlock(moveData, peDef, *ob, PATHDIR_DOWN,       synced);
+		TestBlock(moveData, peDef, *ob, PATHDIR_LEFT_DOWN,  synced);
 
 		// mark this block as closed
-		blockState[ob->blocknr].options |= PATHOPT_CLOSED;
+		blockStates[ob->nodeNum].nodeMask |= PATHOPT_CLOSED;
 	}
 
 	// we found our goal
 	if (foundGoal)
-		return Ok;
+		return IPath::Ok;
 
 	// we could not reach the goal
-	if (openBlockBufferIndex >= maxBlocksToBeSearched)
-		return GoalOutOfRange;
+	if (openBlockBuffer.GetSize() >= maxBlocksToBeSearched)
+		return IPath::GoalOutOfRange;
 
 	// search could not reach the goal due to the unit being locked in
 	if (openBlocks.empty())
-		return GoalOutOfRange;
+		return IPath::GoalOutOfRange;
 
 	// should never happen
 	LogObject() << "ERROR: CPathEstimator::DoSearch() - Unhandled end of search!\n";
-	return Error;
+	return IPath::Error;
 }
 
 
@@ -619,113 +599,125 @@ IPath::SearchResult CPathEstimator::DoSearch(const MoveData& moveData, const CPa
  * Test the accessability of a block and its value,
  * possibly also add it to the open-blocks pqueue.
  */
-void CPathEstimator::TestBlock(const MoveData& moveData, const CPathFinderDef &peDef, OpenBlock& parentOpenBlock, unsigned int direction) {
+void CPathEstimator::TestBlock(
+	const MoveData& moveData,
+	const CPathFinderDef& peDef,
+	PathNode& parentOpenBlock,
+	unsigned int direction,
+	bool synced
+) {
 	testedBlocks++;
 
 	// initial calculations of the new block
 	int2 block;
-	block.x = parentOpenBlock.block.x + directionVector[direction].x;
-	block.y = parentOpenBlock.block.y + directionVector[direction].y;
-	int vertexNbr = moveData.pathType * nbrOfBlocks * PATH_DIRECTION_VERTICES + parentOpenBlock.blocknr * PATH_DIRECTION_VERTICES + directionVertex[direction];
+		block.x = parentOpenBlock.nodePos.x + directionVector[direction].x;
+		block.y = parentOpenBlock.nodePos.y + directionVector[direction].y;
+	const int vertexIdx =
+		moveData.pathType * blockStates.GetSize() * PATH_DIRECTION_VERTICES +
+		parentOpenBlock.nodeNum * PATH_DIRECTION_VERTICES +
+		directionVertex[direction];
+	const int blockIdx = block.y * nbrOfBlocksX + block.x;
 
-	/*
 	if (block.x < 0 || block.x >= nbrOfBlocksX || block.y < 0 || block.y >= nbrOfBlocksZ) {
 		// blocks should never be able to lie outside map to the infinite vertices at the edges
 		return;
 	}
-	*/
 
-	if (vertexNbr < 0 || (unsigned int)vertexNbr >= nbrOfVertices)
+	if (vertexIdx < 0 || (unsigned int)vertexIdx >= vertices.size())
 		return;
 
-	int blocknr = block.y * nbrOfBlocksX + block.x;
-	float blockCost = vertex[vertexNbr];
-	if (blockCost >= PATHCOST_INFINITY)
+	if (vertices[vertexIdx] >= PATHCOST_INFINITY)
 		return;
 
 	// check if the block is unavailable
-	if (blockState[blocknr].options & (PATHOPT_FORBIDDEN | PATHOPT_BLOCKED | PATHOPT_CLOSED))
+	if (blockStates[blockIdx].nodeMask & (PATHOPT_FORBIDDEN | PATHOPT_BLOCKED | PATHOPT_CLOSED))
 		return;
 
-	int xSquare = blockState[blocknr].sqrCenter[moveData.pathType].x;
-	int zSquare = blockState[blocknr].sqrCenter[moveData.pathType].y;
+	const int xSquare = blockStates[blockIdx].nodeOffsets[moveData.pathType].x;
+	const int zSquare = blockStates[blockIdx].nodeOffsets[moveData.pathType].y;
 
 	// check if the block is blocked or out of constraints
 	if (!peDef.WithinConstraints(xSquare, zSquare)) {
-		blockState[blocknr].options |= PATHOPT_BLOCKED;
-		dirtyBlocks.push_back(blocknr);
+		blockStates[blockIdx].nodeMask |= PATHOPT_BLOCKED;
+		dirtyBlocks.push_back(blockIdx);
 		return;
 	}
 
-	// evaluate this node
-	float heuristicCost = peDef.Heuristic(xSquare, zSquare);
-	float currentCost = parentOpenBlock.currentCost + blockCost;
-	float cost = currentCost + heuristicCost;
+	// evaluate this node (NOTE the max-res. indexing for extraCost)
+	const float extraCost = blockStates.GetNodeExtraCost(xSquare, zSquare, synced);
+	const float nodeCost = vertices[vertexIdx] + extraCost;
 
-	// check if the block is already in queue and keep it if it's better
-	if (blockState[blocknr].options & PATHOPT_OPEN) {
-		if (blockState[blocknr].cost <= cost)
+	const float gCost = parentOpenBlock.gCost + nodeCost;  // g
+	const float hCost = peDef.Heuristic(xSquare, zSquare); // h
+	const float fCost = gCost + hCost;                     // f
+
+
+	if (blockStates[blockIdx].nodeMask & PATHOPT_OPEN) {
+		// already in the open set
+		if (blockStates[blockIdx].fCost <= fCost)
 			return;
-		blockState[blocknr].options &= 255 - 7;
+
+		blockStates[blockIdx].nodeMask &= ~PATHOPT_DIRECTION;
 	}
 
 	// look for improvements
-	if (heuristicCost < goalHeuristic) {
+	if (hCost < goalHeuristic) {
 		goalBlock = block;
-		goalHeuristic = heuristicCost;
+		goalHeuristic = hCost;
 	}
 
 	// store this block as open.
-	++openBlockBufferIndex;
-	assert(openBlockBufferIndex < MAX_SEARCHED_BLOCKS);
+	openBlockBuffer.SetSize(openBlockBuffer.GetSize() + 1);
+	assert(openBlockBuffer.GetSize() < MAX_SEARCHED_NODES_PE);
 
-	OpenBlock* ob = &openBlockBuffer[openBlockBufferIndex];
-		ob->block = block;
-		ob->blocknr = blocknr;
-		ob->cost = cost;
-		ob->currentCost = currentCost;
+	PathNode* ob = openBlockBuffer.GetNode(openBlockBuffer.GetSize());
+		ob->fCost   = fCost;
+		ob->gCost   = gCost;
+		ob->nodePos = block;
+		ob->nodeNum = blockIdx;
 	openBlocks.push(ob);
 
-	// Mark the block as open, and its parent.
-	blockState[blocknr].cost = cost;
-	blockState[blocknr].options |= (direction | PATHOPT_OPEN);
-	blockState[blocknr].parentBlock = parentOpenBlock.block;
-	dirtyBlocks.push_back(blocknr);
+	blockStates.SetMaxFCost(std::max(blockStates.GetMaxFCost(), fCost));
+	blockStates.SetMaxGCost(std::max(blockStates.GetMaxGCost(), gCost));
+
+	// mark this block as open
+	blockStates[blockIdx].fCost = fCost;
+	blockStates[blockIdx].gCost = gCost;
+	blockStates[blockIdx].nodeMask |= (direction | PATHOPT_OPEN);
+	blockStates[blockIdx].parentNodePos = parentOpenBlock.nodePos;
+
+	dirtyBlocks.push_back(blockIdx);
 }
 
 
 /**
  * Recreate the path taken to the goal
  */
-void CPathEstimator::FinishSearch(const MoveData& moveData, Path& path) {
+void CPathEstimator::FinishSearch(const MoveData& moveData, IPath::Path& foundPath) {
 	int2 block = goalBlock;
-	while (block.x != startBlock.x || block.y != startBlock.y) {
-		int blocknr = block.y * nbrOfBlocksX + block.x;
 
-		/*
-		int xGSquare = block.x * BLOCK_SIZE + goalSqrOffset.x;
-		int zGSquare = block.y * BLOCK_SIZE + goalSqrOffset.y;
-		// in first case try to use an offset defined by goal...
-		if (!moveData.moveMath->IsBlocced(moveData, moveMathOptions, xGSquare, zGSquare)) {
-			float3 pos = SquareToFloat3(xGSquare, zGSquare);
-			pos.y = moveData.moveMath->yLevel(xGSquare, zGSquare);
-			path.path.push_back(pos);
-		}
-		else */{
-			// ...if not possible, then use offset defined by the block
-			int xBSquare = blockState[blocknr].sqrCenter[moveData.pathType].x;
-			int zBSquare = blockState[blocknr].sqrCenter[moveData.pathType].y;
-			float3 pos = SquareToFloat3(xBSquare, zBSquare);
-			path.path.push_back(pos);
+	while (block.x != startBlock.x || block.y != startBlock.y) {
+		const int blockIdx = block.y * nbrOfBlocksX + block.x;
+
+		{
+			// use offset defined by the block
+			const int xBSquare = blockStates[blockIdx].nodeOffsets[moveData.pathType].x;
+			const int zBSquare = blockStates[blockIdx].nodeOffsets[moveData.pathType].y;
+			const float3& pos = SquareToFloat3(xBSquare, zBSquare);
+
+			foundPath.path.push_back(pos);
 		}
 
 		// next step backwards
-		block = blockState[blocknr].parentBlock;
+		block = blockStates[blockIdx].parentNodePos;
+	}
+
+	if (!foundPath.path.empty()) {
+		foundPath.pathGoal = foundPath.path.front();
 	}
 
 	// set some additional information
-	path.pathCost = blockState[goalBlock.y * nbrOfBlocksX + goalBlock.x].cost - goalHeuristic;
-	path.pathGoal = path.path.front();
+	foundPath.pathCost = blockStates[goalBlock.y * nbrOfBlocksX + goalBlock.x].fCost - goalHeuristic;
 }
 
 
@@ -733,15 +725,18 @@ void CPathEstimator::FinishSearch(const MoveData& moveData, Path& path) {
  * Clean lists from last search
  */
 void CPathEstimator::ResetSearch() {
-	while (!openBlocks.empty())
-		openBlocks.pop();
+	openBlocks.Clear();
+
 	while (!dirtyBlocks.empty()) {
-		blockState[dirtyBlocks.back()].cost = PATHCOST_INFINITY;
-		blockState[dirtyBlocks.back()].parentBlock.x = -1;
-		blockState[dirtyBlocks.back()].parentBlock.y = -1;
-		blockState[dirtyBlocks.back()].options &= PATHOPT_OBSOLETE;
+		PathNodeState& ns = blockStates[dirtyBlocks.back()];
+			ns.fCost = PATHCOST_INFINITY;
+			ns.gCost = PATHCOST_INFINITY;
+			ns.nodeMask &= PATHOPT_OBSOLETE;
+			ns.parentNodePos.x = -1;
+			ns.parentNodePos.y = -1;
 		dirtyBlocks.pop_back();
 	}
+
 	testedBlocks = 0;
 }
 
@@ -750,9 +745,9 @@ void CPathEstimator::ResetSearch() {
  * Try to read offset and vertices data from file, return false on failure
  * TODO: Read-error-check.
  */
-bool CPathEstimator::ReadFile(std::string cacheFileName, const std::string& map)
+bool CPathEstimator::ReadFile(const std::string& cacheFileName, const std::string& map)
 {
-	unsigned int hash = Hash();
+	const unsigned int hash = Hash();
 	char hashString[50];
 	sprintf(hashString, "%u", hash);
 
@@ -770,8 +765,8 @@ bool CPathEstimator::ReadFile(std::string cacheFileName, const std::string& map)
 	CArchiveZip& file(*pfile);
 
 	const unsigned fid = file.FindFile("pathinfo");
-	if (fid < file.NumFiles())
-	{
+
+	if (fid < file.NumFiles()) {
 		pathChecksum = file.GetCrc32(fid);
 
 		std::vector<boost::uint8_t> buffer;
@@ -788,18 +783,18 @@ bool CPathEstimator::ReadFile(std::string cacheFileName, const std::string& map)
 
 		// Read block-center-offset data.
 		const unsigned blockSize = moveinfo->moveData.size() * sizeof(int2);
-		if (buffer.size() < pos + blockSize*nbrOfBlocks)
+		if (buffer.size() < pos + blockSize * blockStates.GetSize())
 			return false;
-		for (int blocknr = 0; blocknr < nbrOfBlocks; blocknr++)
-		{
-			std::memcpy(blockState[blocknr].sqrCenter, &buffer[pos], blockSize);
+
+		for (int blocknr = 0; blocknr < blockStates.GetSize(); blocknr++) {
+			std::memcpy(&blockStates[blocknr].nodeOffsets[0], &buffer[pos], blockSize);
 			pos += blockSize;
 		}
 
 		// Read vertices data.
-		if (buffer.size() < pos + nbrOfVertices * sizeof(float))
+		if (buffer.size() < pos + vertices.size() * sizeof(float))
 			return false;
-		std::memcpy(vertex, &buffer[pos], nbrOfVertices * sizeof(float));
+		std::memcpy(&vertices[0], &buffer[pos], vertices.size() * sizeof(float));
 
 		// File read successful.
 		return true;
@@ -812,17 +807,18 @@ bool CPathEstimator::ReadFile(std::string cacheFileName, const std::string& map)
 /**
  * Try to write offset and vertex data to file.
  */
-void CPathEstimator::WriteFile(std::string cacheFileName, const std::string& map)
+void CPathEstimator::WriteFile(const std::string& cacheFileName, const std::string& map)
 {
 	// We need this directory to exist
 	if (!filesystem.CreateDirectory(pathDir))
 		return;
 
-	unsigned int hash = Hash();
-	char hashString[50];
-	sprintf(hashString,"%u",hash);
+	const unsigned int hash = Hash();
+	char hashString[64] = {0};
 
-	std::string filename = std::string(pathDir) + map + hashString + "." + cacheFileName + ".zip";
+	sprintf(hashString, "%u", hash);
+
+	const std::string filename = std::string(pathDir) + map + hashString + "." + cacheFileName + ".zip";
 	zipFile file;
 
 	// open file for writing in a suitable location
@@ -835,11 +831,11 @@ void CPathEstimator::WriteFile(std::string cacheFileName, const std::string& map
 		zipWriteInFileInZip(file, (void*) &hash, 4);
 
 		// Write block-center-offsets.
-		for (int blocknr = 0; blocknr < nbrOfBlocks; blocknr++)
-			zipWriteInFileInZip(file, (void*) blockState[blocknr].sqrCenter, moveinfo->moveData.size() * sizeof(int2));
+		for (int blocknr = 0; blocknr < blockStates.GetSize(); blocknr++)
+			zipWriteInFileInZip(file, (void*) &blockStates[blocknr].nodeOffsets[0], moveinfo->moveData.size() * sizeof(int2));
 
 		// Write vertices.
-		zipWriteInFileInZip(file, (void*) vertex, nbrOfVertices * sizeof(float));
+		zipWriteInFileInZip(file, &vertices[0], vertices.size() * sizeof(float));
 
 		zipCloseFileInZip(file);
 		zipClose(file, NULL);
@@ -868,32 +864,5 @@ void CPathEstimator::WriteFile(std::string cacheFileName, const std::string& map
  */
 unsigned int CPathEstimator::Hash() const
 {
-	return (readmap->mapChecksum + moveinfo->moveInfoChecksum + BLOCK_SIZE + moveMathOptions + PATHESTIMATOR_VERSION);
-}
-
-
-
-float3 CPathEstimator::FindBestBlockCenter(const MoveData* moveData, float3 pos)
-{
-	int pathType = moveData->pathType;
-	CRangedGoalWithCircularConstraint rangedGoal(pos, pos, 0, 0, SQUARE_SIZE * BLOCK_SIZE * SQUARE_SIZE * BLOCK_SIZE * 4);
-	IPath::Path path;
-
-	std::vector<float3> startPositions;
-
-	int xm = (int) (pos.x / (SQUARE_SIZE * BLOCK_SIZE));
-	int ym = (int) (pos.z / (SQUARE_SIZE * BLOCK_SIZE));
-
-	for (int y = std::max(0, ym - 1); y <= std::min(nbrOfBlocksZ - 1, ym + 1); ++y) {
-		for (int x = std::max(0, xm - 1); x <= std::min(nbrOfBlocksX - 1, xm + 1); ++x) {
-			startPositions.push_back(float3(blockState[y * nbrOfBlocksX + x].sqrCenter[pathType].x * SQUARE_SIZE, 0, blockState[y * nbrOfBlocksX+x].sqrCenter[pathType].y * SQUARE_SIZE));
-		}
-	}
-
-	IPath::SearchResult result = pathFinder->GetPath(*moveData, startPositions, rangedGoal, path);
-
-	if (result == IPath::Ok && !path.path.empty()) {
-		return path.path.back();
-	}
-	return ZeroVector;
+	return (readmap->mapChecksum + moveinfo->moveInfoChecksum + BLOCK_SIZE + PATHESTIMATOR_VERSION);
 }
