@@ -1,3 +1,5 @@
+/* This file is part of the Spring engine (GPL v2 or later), see LICENSE.html */
+
 #ifndef FLAREPROJECTILE_H
 #define FLAREPROJECTILE_H
 
@@ -9,7 +11,7 @@ class CFlareProjectile :
 {
 public:
 	CR_DECLARE(CFlareProjectile);
-	CFlareProjectile(const float3& pos,const float3& speed,CUnit* owner,int activateFrame GML_PARG_H);
+	CFlareProjectile(const float3& pos,const float3& speed,CUnit* owner,int activateFrame);
 	~CFlareProjectile();
 	void Update(void);
 	void Draw(void);
