@@ -10,10 +10,9 @@ class CBeamLaserProjectile: public CWeaponProjectile
 	CR_DECLARE(CBeamLaserProjectile);
 public:
 	CBeamLaserProjectile(const float3& startPos, const float3& endPos,
-		float startAlpha, float endAlpha, const float3& color, const float3& color2,
-		CUnit* owner, float thickness, float corethickness, float flaresize,
-		const WeaponDef* weaponDef, int ttl, float decay);
-	~CBeamLaserProjectile(void);
+		float startAlpha, float endAlpha, const float3& color,
+		CUnit* owner, const WeaponDef* weaponDef);
+	~CBeamLaserProjectile(void) {}
 
 	float3 startPos;
 	float3 endPos;
