@@ -10,10 +10,11 @@ class CExplosiveProjectile : public CWeaponProjectile
 {
 	CR_DECLARE(CExplosiveProjectile);
 public:
-	CExplosiveProjectile(const float3& pos, const float3& speed, CUnit* owner,
-		const WeaponDef *weaponDef, int ttl = 100000, float areaOfEffect = 8,
-		float gravity = 0);
-	virtual ~CExplosiveProjectile();
+	CExplosiveProjectile(const float3& pos, const float3& speed,
+		CUnit* owner, const WeaponDef* weaponDef,
+		int ttl = 100000, float areaOfEffect = 8.0f,
+		float gravity = 0.0f);
+
 	void Update();
 	void Draw(void);
 	void Collision(CUnit* unit);
