@@ -1,3 +1,5 @@
+/* This file is part of the Spring engine (GPL v2 or later), see LICENSE.html */
+
 #ifndef LIGHTNINGPROJECTILE_H
 #define LIGHTNINGPROJECTILE_H
 
@@ -11,7 +13,7 @@ class CLightningProjectile :
 	CR_DECLARE(CLightningProjectile);
 public:
 	CLightningProjectile(const float3& pos, const float3& end, CUnit* owner, const float3& color,
-		const WeaponDef *weaponDef, int ttl = 10, CWeapon* weap = 0 GML_PARG_H);
+		const WeaponDef *weaponDef, int ttl = 10, CWeapon* weap = 0);
 	~CLightningProjectile(void);
 
 	float3 color;
