@@ -1,26 +1,27 @@
+/* This file is part of the Spring engine (GPL v2 or later), see LICENSE.html */
+
 #include "StdAfx.h"
 #include "mmgr.h"
 
 #include "Game/Camera.h"
 #include "Game/GameHelper.h"
-#include "LogOutput.h"
 #include "Map/Ground.h"
-#include "Matrix44f.h"
 #include "MissileProjectile.h"
-#include "myMath.h"
-#include "Sim/Projectiles/ProjectileHandler.h"
+#include "Rendering/GlobalRendering.h"
+#include "Rendering/ProjectileDrawer.hpp"
 #include "Rendering/GL/myGL.h"
 #include "Rendering/GL/VertexArray.h"
+#include "Rendering/Models/3DModel.h"
+#include "Rendering/Textures/TextureAtlas.h"
 #include "Sim/Misc/GeometricObjects.h"
+#include "Sim/Projectiles/ProjectileHandler.h"
 #include "Sim/Projectiles/Unsynced/SmokeTrailProjectile.h"
 #include "Sim/Units/Unit.h"
 #include "Sim/Weapons/WeaponDefHandler.h"
-#include "Sync/SyncTracer.h"
-#include "Rendering/UnitModels/IModelParser.h"
-#include "Rendering/UnitModels/UnitDrawer.h"
-#include "Rendering/UnitModels/s3oParser.h"
-#include "Rendering/UnitModels/3DOParser.h"
-#include "GlobalUnsynced.h"
+#include "System/Matrix44f.h"
+#include "System/myMath.h"
+#include "System/Sync/SyncTracer.h"
+#include "System/GlobalUnsynced.h"
 
 static const float Smoke_Time=60;
 
@@ -55,11 +56,14 @@ CR_REG_METADATA(CMissileProjectile,(
 	CR_RESERVED(16)
 	));
 
-CMissileProjectile::CMissileProjectile(const float3& pos, const float3& speed, CUnit* owner,
-		float areaOfEffect, float maxSpeed, int ttl, CUnit* target, const WeaponDef *weaponDef,
-		float3 targetPos GML_PARG_C):
-	CWeaponProjectile(pos, speed, owner, target, targetPos, weaponDef, 0, ttl GML_PARG_P),
-	dir(speed),
+CMissileProjectile::CMissileProjectile(
+	const float3& pos, const float3& speed,
+	CUnit* owner,
+	float areaOfEffect, float maxSpeed, int ttl,
+	CUnit* target, const WeaponDef* weaponDef,
+	float3 targetPos):
+
+	CWeaponProjectile(pos, speed, owner, target, targetPos, weaponDef, 0, ttl),
 	maxSpeed(maxSpeed),
 	areaOfEffect(areaOfEffect),
 	age(0),
@@ -79,8 +83,9 @@ CMissileProjectile::CMissileProjectile(const float3& pos, const float3& speed, C
 	danceCenter(0, 0, 0),
 	extraHeightTime(0)
 {
+	projectileType = WEAPON_MISSILE_PROJECTILE;
 	curSpeed = speed.Length();
-	dir.Normalize();
+	dir = speed / curSpeed;
 	oldDir = dir;
 
 	if (target) {
@@ -90,9 +95,9 @@ CMissileProjectile::CMissileProjectile(const float3& pos, const float3& speed, C
 	SetRadius(0.0f);
 
 	if (weaponDef) {
-		s3domodel = LoadModel(weaponDef);
-		if (s3domodel) {
-			SetRadius(s3domodel->radius);
+		model = LoadModel(weaponDef);
+		if (model) {
+			SetRadius(model->radius);
 		}
 	}
 
@@ -126,7 +131,7 @@ CMissileProjectile::CMissileProjectile(const float3& pos, const float3& speed, C
 	}
 
 
-	if (cegTag.size() > 0) {
+	if (!cegTag.empty()) {
 		ceg.Load(explGenHandler, cegTag);
 	}
 }
@@ -178,98 +183,99 @@ void CMissileProjectile::Collision(CUnit *unit)
 
 void CMissileProjectile::Update(void)
 {
-	ttl--;
-	if (ttl > 0) {
-		if (curSpeed < maxSpeed)
-			curSpeed += weaponDef->weaponacceleration;
+	if (--ttl > 0) {
+		if (!luaMoveCtrl) {
+			if (curSpeed < maxSpeed) {
+				curSpeed += weaponDef->weaponacceleration;
+			}
 
-		float3 targSpeed(0, 0, 0);
+			float3 targSpeed(0, 0, 0);
 
-		if (weaponDef->tracks && (decoyTarget || target)) {
-			if (decoyTarget) {
-				targPos = decoyTarget->pos;
-				targSpeed = decoyTarget->speed;
+			if (weaponDef->tracks && (decoyTarget || target)) {
+				if (decoyTarget) {
+					targPos = decoyTarget->pos;
+					targSpeed = decoyTarget->speed;
+				} else {
+					targSpeed = target->speed;
+
+					if ((target->physicalState == CSolidObject::Flying && (target->midPos - pos).SqLength() < 150 * 150) || !owner()) {
+						targPos = target->midPos;
+					} else {
+						targPos = helper->GetUnitErrorPos(target, owner()->allyteam);
+					}
+				}
+			}
+
+
+			if (isWobbling) {
+				--wobbleTime;
+				if (wobbleTime == 0) {
+					float3 newWob = gs->randVector();
+					wobbleDif = (newWob - wobbleDir) * (1.0f / 16);
+					wobbleTime = 16;
+				}
+
+				wobbleDir += wobbleDif;
+
+				dir += wobbleDir * weaponDef->wobble * (owner()? (1.0f - owner()->limExperience * 0.5f): 1);
+				dir.Normalize();
+			}
+
+			if (isDancing) {
+				--danceTime;
+				if (danceTime <= 0) {
+					danceMove = gs->randVector() * weaponDef->dance - danceCenter;
+					danceCenter += danceMove;
+					danceTime = 8;
+				}
+
+				pos += danceMove;
+			}
+
+			float3 orgTargPos = targPos;
+			float3 targetDir = (targPos - pos).Normalize();
+			float dist = targPos.distance(pos) + 0.1f;
+
+			if (extraHeightTime > 0) {
+				extraHeight -= extraHeightDecay;
+				--extraHeightTime;
+
+				targPos.y += extraHeight;
+
+				if (dir.y <= 0.0f) {
+					// missile has reached apex, smoothly transition
+					// to targetDir (can still overshoot when target
+					// is too close or height difference too large)
+					const float horDiff = (targPos - pos).Length2D() + 0.01f;
+					const float verDiff = (targPos.y - pos.y) + 0.01f;
+					const float dirDiff = fabs(targetDir.y - dir.y);
+					const float ratio = fabs(verDiff / horDiff);
+
+					dir.y -= (dirDiff * ratio);
+				} else {
+					// missile is still ascending
+					dir.y -= (extraHeightDecay / dist);
+				}
+			}
+
+
+			float3 dif = (targPos + targSpeed * (dist / maxSpeed) * 0.7f - pos).Normalize();
+			float3 dif2 = dif - dir;
+
+			if (dif2.SqLength() < Square(weaponDef->turnrate)) {
+				dir = dif;
 			} else {
-				targSpeed = target->speed;
-				if ((target->physicalState == CSolidObject::Flying && (target->midPos-pos).SqLength() < 150 * 150) || !owner())
-					targPos = target->midPos;
-				else
-					targPos = helper->GetUnitErrorPos(target, owner()->allyteam);
+				dif2 -= (dir * (dif2.dot(dir)));
+				dif2.SafeNormalize();
+				dir += (dif2 * weaponDef->turnrate);
+				dir.SafeNormalize();
 			}
+
+			targPos = orgTargPos;
+			speed = dir * curSpeed;
 		}
 
-
-		if (isWobbling) {
-			--wobbleTime;
-			if (wobbleTime == 0) {
-				float3 newWob = gs->randVector();
-				wobbleDif = (newWob - wobbleDir) * (1.0f / 16);
-				wobbleTime = 16;
-			}
-			wobbleDir += wobbleDif;
-			dir += wobbleDir * weaponDef->wobble * (owner()? (1 - owner()->limExperience * 0.5f): 1);
-			dir.Normalize();
-		}
-
-		if (isDancing) {
-			--danceTime;
-			if (danceTime <= 0) {
-				danceMove = gs->randVector() * weaponDef->dance - danceCenter;
-				danceCenter += danceMove;
-				danceTime = 8;
-			}
-			pos += danceMove;
-		}
-
-
-		float3 orgTargPos(targPos);
-		float3 targetDir = (targPos - pos).Normalize();
-		float dist = targPos.distance(pos);
-
-		if (dist == 0) {
-			dist = 0.1f;
-		}
-
-		if (extraHeightTime > 0) {
-			extraHeight -= extraHeightDecay;
-			--extraHeightTime;
-
-			targPos.y += extraHeight;
-
-			if (dir.y <= 0.0f) {
-				// missile has reached apex, smoothly transition
-				// to targetDir (can still overshoot when target
-				// is too close or height difference too large)
-				const float horDiff = (targPos - pos).Length2D() + 0.01f;
-				const float verDiff = (targPos.y - pos.y) + 0.01f;
-				const float dirDiff = fabs(targetDir.y - dir.y);
-				const float ratio = fabs(verDiff / horDiff);
-				dir.y -= (dirDiff * ratio);
-			} else {
-				// missile is still ascending
-				dir.y -= (extraHeightDecay / dist);
-			}
-		}
-
-
-		float3 dif(targPos + targSpeed * (dist / maxSpeed) * 0.7f - pos);
-		dif.Normalize();
-		float3 dif2 = dif - dir;
-		float tracking = weaponDef->turnrate;
-
-		if (dif2.SqLength() < Square(tracking)) {
-			dir = dif;
-		} else {
-			dif2 -= (dir * (dif2.dot(dir)));
-			dif2.SafeNormalize();
-			dir += (dif2 * tracking);
-			dir.SafeNormalize();
-		}
-
-		speed = dir * curSpeed;
-		targPos = orgTargPos;
-
-		if (cegTag.size() > 0) {
+		if (!cegTag.empty()) {
 			ceg.Explosion(pos, ttl, areaOfEffect, 0x0, 0.0f, 0x0, dir);
 		}
 	} else {
@@ -278,31 +284,41 @@ void CMissileProjectile::Update(void)
 		} else {
 			// only when TTL <= 0 do we (missiles)
 			// get influenced by gravity and drag
-			speed *= 0.98f;
-			speed.y += mygravity;
-			dir = speed;
-			dir.Normalize();
+			if (!luaMoveCtrl) {
+				speed *= 0.98f;
+				speed.y += mygravity;
+				dir = speed;
+				dir.Normalize();
+			}
 		}
 	}
 
-	pos += speed;
+	if (!luaMoveCtrl) {
+		pos += speed;
+	}
+
 	age++;
 	numParts++;
 
 
 	if (weaponDef->visuals.smokeTrail && !(age & 7)) {
-		CSmokeTrailProjectile* tp = new CSmokeTrailProjectile(pos, oldSmoke,
+		CSmokeTrailProjectile* tp = new CSmokeTrailProjectile(
+			pos, oldSmoke,
 			dir, oldDir, owner(), age == 8, false, 7, Smoke_Time, 0.6f, drawTrail, 0,
-			weaponDef->visuals.texture2);
+			weaponDef->visuals.texture2
+		);
+
 		oldSmoke = pos;
 		oldDir = dir;
 		numParts = 0;
 		useAirLos = tp->useAirLos;
 
 		if (!drawTrail) {
-			float3 camDir = (pos - camera->pos).Normalize();
-			if ((camera->pos.distance(pos) * 0.2f + (1 - fabs(camDir.dot(dir))) * 3000) > 300)
+			const float3 camDir = (pos - camera->pos).Normalize();
+
+			if ((camera->pos.distance(pos) * 0.2f + (1 - fabs(camDir.dot(dir))) * 3000) > 300) {
 				drawTrail = true;
+			}
 		}
 	}
 
@@ -310,7 +326,11 @@ void CMissileProjectile::Update(void)
 }
 
 void CMissileProjectile::UpdateGroundBounce() {
-	float3 tempSpeed = speed;
+	if (luaMoveCtrl) {
+		return;
+	}
+
+	const float3 tempSpeed = speed;
 	CWeaponProjectile::UpdateGroundBounce();
 
 	if (tempSpeed != speed) {
@@ -320,16 +340,18 @@ void CMissileProjectile::UpdateGroundBounce() {
 	}
 }
 
+
+
 void CMissileProjectile::Draw(void)
 {
 	inArray = true;
-	float age2 = (age & 7) + gu->timeOffset;
+	const float age2 = (age & 7) + globalRendering->timeOffset;
 
-	float color = 0.6f;
 	unsigned char col[4];
 
 	va->EnlargeArrays(8+4*numParts,0,VA_SIZE_TC);
 	if (weaponDef->visuals.smokeTrail) {
+		const float color = 0.6f;
 		if (drawTrail) {
 			// draw the trail as a single quad
 			float3 dif(drawPos - camera->pos);
@@ -343,11 +365,11 @@ void CMissileProjectile::Draw(void)
 
 			float a1 = (1.0f / (Smoke_Time)) * 255;
 			a1 *= 0.7f + fabs(dif.dot(dir));
-			float alpha = std::min(255.0f, std::max(0.0f, a1));
-			col[0] = (unsigned char) (color * alpha);
-			col[1] = (unsigned char) (color * alpha);
-			col[2] = (unsigned char) (color * alpha);
-			col[3] = (unsigned char) alpha;
+			const float alpha1 = std::min(255.0f, std::max(0.0f, a1));
+			col[0] = (unsigned char) (color * alpha1);
+			col[1] = (unsigned char) (color * alpha1);
+			col[2] = (unsigned char) (color * alpha1);
+			col[3] = (unsigned char) alpha1;
 
 			unsigned char col2[4];
 			float a2 = (1 - float(age2) / (Smoke_Time)) * 255;
@@ -356,40 +378,42 @@ void CMissileProjectile::Draw(void)
 				a2 = 0;
 
 			a2 *= 0.7f + fabs(dif2.dot(oldDir));
-			alpha = std::min(255.0f, std::max(0.0f, a2));
-			col2[0] = (unsigned char) (color * alpha);
-			col2[1] = (unsigned char) (color * alpha);
-			col2[2] = (unsigned char) (color * alpha);
-			col2[3] = (unsigned char) alpha;
+			const float alpha2 = std::min(255.0f, std::max(0.0f, a2));
+			col2[0] = (unsigned char) (color * alpha2);
+			col2[1] = (unsigned char) (color * alpha2);
+			col2[2] = (unsigned char) (color * alpha2);
+			col2[3] = (unsigned char) alpha2;
 
-			float size = 1.0f;
-			float size2 = (1 + (age2 * (1 / Smoke_Time)) * 7);
-			float txs = weaponDef->visuals.texture2->xend - (weaponDef->visuals.texture2->xend - weaponDef->visuals.texture2->xstart) * (age2 / 8.0f);
+			const float size = 1.0f;
+			const float size2 = (1 + (age2 * (1 / Smoke_Time)) * 7);
+			const float txs = weaponDef->visuals.texture2->xend - (weaponDef->visuals.texture2->xend - weaponDef->visuals.texture2->xstart) * (age2 / 8.0f);
 
-			va->AddVertexQTC(drawPos - dir1 * size,  txs,                               weaponDef->visuals.texture2->ystart, col);
-			va->AddVertexQTC(drawPos + dir1 * size,  txs,                               weaponDef->visuals.texture2->yend,   col);
+			va->AddVertexQTC(drawPos  - dir1 * size,  txs,                               weaponDef->visuals.texture2->ystart, col);
+			va->AddVertexQTC(drawPos  + dir1 * size,  txs,                               weaponDef->visuals.texture2->yend,   col);
 			va->AddVertexQTC(oldSmoke + dir2 * size2, weaponDef->visuals.texture2->xend, weaponDef->visuals.texture2->yend,   col2);
 			va->AddVertexQTC(oldSmoke - dir2 * size2, weaponDef->visuals.texture2->xend, weaponDef->visuals.texture2->ystart, col2);
 		} else {
 			// draw the trail as particles
-			float dist = pos.distance(oldSmoke);
-			float3 dirpos1 = pos - dir * dist * 0.33f;
-			float3 dirpos2 = oldSmoke + oldDir * dist * 0.33f;
+			const float dist = pos.distance(oldSmoke);
+			const float3 dirpos1 = pos - dir * dist * 0.33f;
+			const float3 dirpos2 = oldSmoke + oldDir * dist * 0.33f;
 
 			for (int a = 0; a < numParts; ++a) { //! CAUTION: loop count must match EnlargeArrays above
-				float alpha = 255.0f;
+				const float alpha = 255.0f;
 				col[0] = (unsigned char) (color * alpha);
 				col[1] = (unsigned char) (color * alpha);
 				col[2] = (unsigned char) (color * alpha);
 				col[3] = (unsigned char) alpha;
 
-				float size = (1 + (a * (1 / Smoke_Time)) * 7);
+				const float size = (1 + (a * (1 / Smoke_Time)) * 7);
 				float3 pos1 = CalcBeizer(float(a) / (numParts), pos, dirpos1, dirpos2, oldSmoke);
 
-				va->AddVertexQTC(pos1 + ( camera->up + camera->right) * size, ph->smoketex[0].xstart, ph->smoketex[0].ystart, col);
-				va->AddVertexQTC(pos1 + ( camera->up - camera->right) * size, ph->smoketex[0].xend,   ph->smoketex[0].ystart, col);
-				va->AddVertexQTC(pos1 + (-camera->up - camera->right) * size, ph->smoketex[0].xend,   ph->smoketex[0].ystart, col);
-				va->AddVertexQTC(pos1 + (-camera->up + camera->right) * size, ph->smoketex[0].xstart, ph->smoketex[0].ystart, col);
+				#define st projectileDrawer->smoketex[0]
+				va->AddVertexQTC(pos1 + ( camera->up + camera->right) * size, st->xstart, st->ystart, col);
+				va->AddVertexQTC(pos1 + ( camera->up - camera->right) * size, st->xend,   st->ystart, col);
+				va->AddVertexQTC(pos1 + (-camera->up - camera->right) * size, st->xend,   st->ystart, col);
+				va->AddVertexQTC(pos1 + (-camera->up + camera->right) * size, st->xstart, st->ystart, col);
+				#undef st
 			}
 		}
 	}
@@ -399,78 +423,34 @@ void CMissileProjectile::Draw(void)
 	col[1] = 210;
 	col[2] = 180;
 	col[3] = 1;
-	float fsize = radius * 0.4f;
+	const float fsize = radius * 0.4f;
 	va->AddVertexQTC(drawPos - camera->right * fsize-camera->up * fsize, weaponDef->visuals.texture1->xstart, weaponDef->visuals.texture1->ystart, col);
 	va->AddVertexQTC(drawPos + camera->right * fsize-camera->up * fsize, weaponDef->visuals.texture1->xend,   weaponDef->visuals.texture1->ystart, col);
 	va->AddVertexQTC(drawPos + camera->right * fsize+camera->up * fsize, weaponDef->visuals.texture1->xend,   weaponDef->visuals.texture1->yend,   col);
 	va->AddVertexQTC(drawPos - camera->right * fsize+camera->up * fsize, weaponDef->visuals.texture1->xstart, weaponDef->visuals.texture1->yend,   col);
-
-/*	col[0]=200;
-	col[1]=200;
-	col[2]=200;
-	col[3]=255;
-	float3 r=dir.cross(UpVector);
-	r.Normalize();
-	float3 u=dir.cross(r);
-	va->AddVertexTC(interPos+r*1.0f,1.0f/16,1.0f/16,col);
-	va->AddVertexTC(interPos-r*1.0f,1.0f/16,1.0f/16,col);
-	va->AddVertexTC(interPos+dir*9,1.0f/16,1.0f/16,col);
-	va->AddVertexTC(interPos+dir*9,1.0f/16,1.0f/16,col);
-
-	va->AddVertexTC(interPos+u*1.0f,1.0f/16,1.0f/16,col);
-	va->AddVertexTC(interPos-u*1.0f,1.0f/16,1.0f/16,col);
-	va->AddVertexTC(interPos+dir*9,1.0f/16,1.0f/16,col);
-	va->AddVertexTC(interPos+dir*9,1.0f/16,1.0f/16,col);*/
 }
 
-void CMissileProjectile::DrawUnitPart(void)
+int CMissileProjectile::ShieldRepulse(CPlasmaRepulser* shield, float3 shieldPos, float shieldForce, float shieldMaxSpeed)
 {
-	float3 rightdir, updir;
+	if (!luaMoveCtrl) {
+		if (ttl > 0) {
+			const float3 sdir = (pos - shieldPos).Normalize();
+			// steer away twice as fast as we can steer toward target
+			float3 dif2 = sdir - dir;
+			float tracking = std::max(shieldForce * 0.05f, weaponDef->turnrate * 2);
 
-	if (fabs(dir.y) < 0.95f) {
-		rightdir = dir.cross(UpVector);
-		rightdir.SafeNormalize();
-	} else {
-		rightdir = float3(1.0f, 0.0f, 0.0f);
-	}
+			if (dif2.SqLength() < Square(tracking)) {
+				dir = sdir;
+			} else {
+				dif2 -= dir * (dif2.dot(dir));
+				dif2.Normalize();
+				dir += dif2 * tracking;
+				dir.Normalize();
+			}
 
-	updir = rightdir.cross(dir);
-
-	CMatrix44f transMatrix(drawPos + dir * radius * 0.9f, -rightdir, updir, dir);
-
-	glPushMatrix();
-		glMultMatrixf(transMatrix);
-		glCallList(s3domodel->rootobject->displist); // dont cache displists because of delayed loading (GML)
-	glPopMatrix();
-}
-
-int CMissileProjectile::ShieldRepulse(CPlasmaRepulser* shield,float3 shieldPos, float shieldForce, float shieldMaxSpeed)
-{
-	float3 sdir = pos - shieldPos;
-	sdir.Normalize();
-
-	if (ttl > 0) {
-		// steer away twice as fast as we can steer toward target
-		float3 dif2 = sdir - dir;
-		float tracking = std::max(shieldForce * 0.05f, weaponDef->turnrate * 2);
-
-		if (dif2.SqLength() < Square(tracking)) {
-			dir = sdir;
-		} else {
-			dif2 -= dir * (dif2.dot(dir));
-			dif2.Normalize();
-			dir += dif2 * tracking;
-			dir.Normalize();
+			return 2;
 		}
-
-		return 2;
 	}
 
 	return 0;
 }
-
-void CMissileProjectile::DrawS3O(void)
-{
-	unitDrawer->SetTeamColour(colorTeam);
-	DrawUnitPart();
-}
