@@ -137,7 +137,6 @@ CUnit::CUnit():
 	los(0),
 	tempNum(0),
 	lastSlowUpdate(0),
-	controlRadius(2),
 	losRadius(0),
 	airLosRadius(0),
 	losHeight(0),
@@ -427,8 +426,10 @@ void CUnit::SetRight(const SyncedFloat3& newDir)
 
 void CUnit::UpdateMidPos()
 {
-	midPos = pos + (frontdir * relMidPos.z) + (updir * relMidPos.y)
-		+ (rightdir * relMidPos.x);
+	midPos = pos +
+		(frontdir * relMidPos.z) +
+		(updir    * relMidPos.y) +
+		(rightdir * relMidPos.x);
 }
 
 
@@ -685,6 +686,8 @@ void CUnit::SlowUpdate()
 		}
 	}
 
+	UpdateResources();
+
 	if (stunned) {
 		// de-stun only if we are not (still) inside a non-firebase transport
 		if ((paralyzeDamage <= (modInfo.paralyzeOnMaxHealth? maxHealth: health)) &&
@@ -693,7 +696,6 @@ void CUnit::SlowUpdate()
 		}
 
 		SlowUpdateCloak(true);
-		UpdateResources();
 		return;
 	}
 
@@ -723,9 +725,7 @@ void CUnit::SlowUpdate()
 			if (health < 0.0f) {
 				KillUnit(false, true, NULL);
 			}
-			UpdateResources();
 		}
-
 		ScriptDecloak(false);
 		return;
 	}
@@ -737,8 +737,6 @@ void CUnit::SlowUpdate()
 	commandAI->SlowUpdate();
 	moveType->SlowUpdate();
 
-	UpdateResources();
-
 	// FIXME: scriptMakeMetal ...?
 	AddMetal(uncondMakeMetal);
 	AddEnergy(uncondMakeEnergy);
@@ -1430,8 +1428,8 @@ void CUnit::SetLastAttacker(CUnit* attacker)
 	if (teamHandler->AlliedTeams(team, attacker->team)) {
 		return;
 	}
-	if (lastAttacker && lastAttacker != userTarget)
-		DeleteDeathDependence(lastAttacker);
+	if (lastAttacker)
+		DeleteDeathDependence(lastAttacker, DEPENDENCE_ATTACKER);
 
 	lastAttack = gs->frameNum;
 	lastAttacker = attacker;
@@ -1455,9 +1453,8 @@ void CUnit::DependentDied(CObject* o)
 
 void CUnit::SetUserTarget(CUnit* target)
 {
-	if (userTarget && (lastAttacker != userTarget)) {
-		DeleteDeathDependence(userTarget);
-	}
+	if (userTarget)
+		DeleteDeathDependence(userTarget, DEPENDENCE_TARGET);
 
 	userTarget=target;
 	for(vector<CWeapon*>::iterator wi=weapons.begin();wi!=weapons.end();++wi) {
@@ -1506,6 +1503,9 @@ void CUnit::Init(const CUnit* builder)
 		Block();
 	}
 
+	if (unitDef->windGenerator > 0.0f) {
+		wind.AddUnit(this);
+	}
 
 	isUnderWater = ((pos.y + model->height) < 0.0f);
 	UpdateTerrainType();
@@ -1733,7 +1733,7 @@ void CUnit::FinishedBuilding(void)
 	buildProgress = 1.0f;
 
 	if (soloBuilder) {
-		DeleteDeathDependence(soloBuilder);
+		DeleteDeathDependence(soloBuilder, DEPENDENCE_BUILDER);
 		soloBuilder = NULL;
 	}
 
@@ -1743,10 +1743,6 @@ void CUnit::FinishedBuilding(void)
 
 	ChangeLos(realLosRadius, realAirLosRadius);
 
-	if (unitDef->windGenerator > 0.0f) {
-		wind.AddUnit(this);
-	}
-
 	if (unitDef->activateWhenBuilt) {
 		Activate();
 	}
@@ -2219,6 +2215,33 @@ void CUnit::ScriptDecloak(bool updateCloakTimeOut)
 }
 
 
+void CUnit::DeleteDeathDependence(CObject* o, DependenceType dep) {
+	switch(dep) {
+		case DEPENDENCE_BUILD:
+		case DEPENDENCE_CAPTURE:
+		case DEPENDENCE_RECLAIM:
+		case DEPENDENCE_TERRAFORM:
+		case DEPENDENCE_TRANSPORTEE:
+		case DEPENDENCE_TRANSPORTER:
+			if(o == lastAttacker || o == soloBuilder || o == userTarget) return;
+			break;
+		case DEPENDENCE_ATTACKER:
+			if(o == soloBuilder || o == userTarget) return;
+			break;
+		case DEPENDENCE_BUILDER:
+			if(o == lastAttacker || o == userTarget) return;
+			break;
+		case DEPENDENCE_RESURRECT:
+			// feature
+			break;
+		case DEPENDENCE_TARGET:
+			if(o == lastAttacker || o == soloBuilder) return;
+			break;
+	}
+	CObject::DeleteDeathDependence(o);
+}
+
+
 // Member bindings
 CR_REG_METADATA(CUnit, (
 	//CR_MEMBER(unitDef),
@@ -2278,7 +2301,6 @@ CR_REG_METADATA(CUnit, (
 	CR_MEMBER(tempNum),
 	CR_MEMBER(lastSlowUpdate),
 	CR_MEMBER(mapSquare),
-	CR_MEMBER(controlRadius),
 	CR_MEMBER(losRadius),
 	CR_MEMBER(airLosRadius),
 	CR_MEMBER(losHeight),
