@@ -121,6 +121,10 @@ public:
 	}
 
 	void DependentDied(CObject* o);
+	enum DependenceType { DEPENDENCE_ATTACKER, DEPENDENCE_BUILD, DEPENDENCE_BUILDER, DEPENDENCE_CAPTURE, DEPENDENCE_RECLAIM, 
+						DEPENDENCE_RESURRECT, DEPENDENCE_TARGET, DEPENDENCE_TERRAFORM, DEPENDENCE_TRANSPORTEE, DEPENDENCE_TRANSPORTER };
+	virtual void DeleteDeathDependence(CObject* o, DependenceType dep);
+
 	void SetUserTarget(CUnit* target);
 	virtual void Init(const CUnit* builder);
 	bool SetGroup(CGroup* group);
@@ -294,7 +298,6 @@ public:
 
 	int mapSquare;
 
-	int controlRadius;
 	int losRadius;
 	int airLosRadius;
 	float losHeight;
