@@ -14,7 +14,7 @@ namespace netcode
 }
 struct PlayerStatistics;
 
-const unsigned char NETWORK_VERSION = 1;
+const unsigned short NETWORK_VERSION = 2;
 
 /*
 Comment behind NETMSG enumeration constant gives the extra data belonging to
@@ -57,7 +57,6 @@ enum NETMSG {
 	NETMSG_CPU_USAGE        = 21, // float cpuUsage;
 	NETMSG_DIRECT_CONTROL   = 22, // uchar myPlayerNum;
 	NETMSG_DC_UPDATE        = 23, // uchar myPlayerNum, status; short heading, pitch;
-	NETMSG_ATTEMPTCONNECT   = 25, // ushort msgsize, string playername, string passwd, string VERSION_STRING_DETAILED
 	NETMSG_SHARE            = 26, // uchar myPlayerNum, shareTeam, bShareUnits; float shareMetal, shareEnergy;
 	NETMSG_SETSHARE         = 27, // uchar myPlayerNum, uchar myTeam; float metalShareFraction, energyShareFraction;
 	NETMSG_SENDPLAYERSTAT   = 28, //
@@ -89,14 +88,13 @@ enum NETMSG {
 	NETMSG_CUSTOM_DATA      = 55, // uchar myPlayerNum, uchar dataType, uchar dataValue
 	NETMSG_TEAMSTAT         = 60, // uchar teamNum, struct TeamStatistics statistics      # used by LadderBot #
 
+	NETMSG_ATTEMPTCONNECT   = 65, // ushort msgsize, ushort netversion, string playername, string passwd, string VERSION_STRING_DETAILED
+
 	NETMSG_AI_CREATED       = 70, // /* uchar messageSize */, uchar myPlayerNum, uint whichSkirmishAI, uchar team, std::string name (ends with \0)
 	NETMSG_AI_STATE_CHANGED = 71, // uchar myPlayerNum, uint whichSkirmishAI, uchar newState
 
 	NETMSG_REQUEST_TEAMSTAT = 72, // uchar teamNum, ushort statFrameNum                   # used by LadderBot #
 
-	NETMSG_REGISTER_NETMSG	= 73, // uchar myPlayerNum, uchar NETMSG
-	NETMSG_UNREGISTER_NETMSG= 74, // uchar myPlayerNum, uchar NETMSG
-
 	NETMSG_CREATE_NEWPLAYER = 75 // uchar playerNum, uchar spectator, uchar teamNum, std::string playerName #used for players not preset in script.txt#
 };
 
@@ -201,9 +199,6 @@ public:
 
 	PacketType SendSetAllied(uchar myPlayerNum, uchar whichAllyTeam, uchar state);
 
-	PacketType SendRegisterNetMsg( uchar myPlayerNum, NETMSG msgID );
-	PacketType SendUnRegisterNetMsg( uchar myPlayerNum, NETMSG msgID );
-
 	PacketType SendCreateNewPlayer( uchar playerNum, bool spectator, uchar teamNum, std::string playerName );
 
 #ifdef SYNCDEBUG
@@ -219,3 +214,4 @@ private:
 };
 
 #endif // BASENETPROTOCOL_H
+
