@@ -1,37 +1,43 @@
+/* This file is part of the Spring engine (GPL v2 or later), see LICENSE.html */
+
 #ifndef __ARCHIVE_7ZIP_H
 #define __ARCHIVE_7ZIP_H
 
-#include "ArchiveBuffered.h"
-
+#include <boost/thread/mutex.hpp>
 extern "C" {
 #include "lib/7z/7zFile.h"
 #include "lib/7z/Archive/7z/7zIn.h"
 };
 
-class CArchive7Zip : public CArchiveBuffered
+#include "ArchiveBase.h"
+
+class CArchive7Zip : public CArchiveBase
 {
 public:
 	CArchive7Zip(const std::string& name);
 	virtual ~CArchive7Zip(void);
+
 	virtual bool IsOpen();
-	virtual int FindFiles(int cur, std::string* name, int* size);
-	virtual unsigned int GetCrc32 (const std::string& fileName);
+	
+	virtual unsigned NumFiles() const;
+	virtual bool GetFile(unsigned fid, std::vector<boost::uint8_t>& buffer);
+	virtual void FileInfo(unsigned fid, std::string& name, int& size) const;
+	virtual unsigned GetCrc32(unsigned fid);
 
 private:
+	boost::mutex archiveLock;
 	UInt32 blockIndex;
 	Byte *outBuffer;
 	size_t outBufferSize;
 
-	struct FileData {
+	struct FileData
+	{
 		int fp;
 		int size;
 		std::string origName;
 		unsigned int crc;
 	};
-	std::map<std::string, FileData> fileData;
-
-	int curSearchHandle;
-	std::map<int, std::map<std::string, FileData>::iterator> searchHandles;
+	std::vector<FileData> fileData;
 
 	CFileInStream archiveStream;
 	CSzArEx db;
@@ -40,7 +46,6 @@ private:
 	ISzAlloc allocTempImp;
 
 	bool isOpen;
-	virtual FileBuffer* GetEntireFileImpl(const std::string& fName);
 };
 
 #endif
