@@ -60,6 +60,7 @@ CArchiveScanner::CArchiveScanner()
 	for (std::vector<std::string>::const_reverse_iterator d = datadirs.rbegin(); d != datadirs.rend(); ++d) {
 		scanDirs.push_back(*d + "maps");
 		scanDirs.push_back(*d + "base");
+		scanDirs.push_back(*d + "games");
 		scanDirs.push_back(*d + "mods");
 		scanDirs.push_back(*d + "packages");
 	}
@@ -439,7 +440,8 @@ void CArchiveScanner::ReadCacheData(const std::string& filename)
 	LuaParser p(filename, SPRING_VFS_RAW, SPRING_VFS_BASE);
 
 	if (!p.Execute()) {
-		logOutput.Print("ERROR in " + filename + ": " + p.GetErrorLog());
+		logOutput.Print("Warning: Failed to read archive cache: " + p.GetErrorLog());
+		return;
 	}
 	const LuaTable archiveCache = p.GetRoot();
 	const LuaTable archives = archiveCache.SubTable("archives");
@@ -451,7 +453,7 @@ void CArchiveScanner::ReadCacheData(const std::string& filename)
 	}
 
 	for (int i = 1; archives.KeyExists(i); ++i) {
-	  const LuaTable curArchive = archives.SubTable(i);
+		const LuaTable curArchive = archives.SubTable(i);
 		const LuaTable archived = curArchive.SubTable("archivedata");
 		ArchiveInfo ai;
 
