@@ -1,3 +1,5 @@
+/* This file is part of the Spring engine (GPL v2 or later), see LICENSE.html */
+
 #include "StdAfx.h"
 
 #include "FileHandler.h"
@@ -22,7 +24,7 @@ using namespace std;
 /******************************************************************************/
 
 CFileHandler::CFileHandler(const char* filename, const char* modes)
-: ifs(NULL), hpiFileBuffer(NULL), hpiOffset(0), filesize(-1)
+: ifs(NULL), filePos(0), fileSize(-1)
 {
 	GML_RECMUTEX_LOCK(file); // CFileHandler
 
@@ -31,7 +33,7 @@ CFileHandler::CFileHandler(const char* filename, const char* modes)
 
 
 CFileHandler::CFileHandler(const string& filename, const string& modes)
-: ifs(NULL), hpiFileBuffer(NULL), hpiOffset(0), filesize(-1)
+: ifs(NULL), filePos(0), fileSize(-1)
 {
 	GML_RECMUTEX_LOCK(file); // CFileHandler
 
@@ -43,12 +45,7 @@ CFileHandler::~CFileHandler(void)
 {
 	GML_RECMUTEX_LOCK(file); // ~CFileHandler
 
-	if (ifs) {
-		delete ifs;
-	}
-	if (hpiFileBuffer) {
-		delete[] hpiFileBuffer;
-	}
+	delete ifs;
 }
 
 
@@ -60,7 +57,7 @@ bool CFileHandler::TryRawFS(const string& filename)
 	ifs = new ifstream(rawpath.c_str(), ios::in | ios::binary);
 	if (ifs && !ifs->bad() && ifs->is_open()) {
 		ifs->seekg(0, ios_base::end);
-		filesize = ifs->tellg();
+		fileSize = ifs->tellg();
 		ifs->seekg(0, ios_base::beg);
 		return true;
 	}
@@ -77,20 +74,12 @@ bool CFileHandler::TryModFS(const string& filename)
 	}
 
 	const string file = StringToLower(filename);
-
-	hpiLength = vfsHandler->GetFileSize(file);
-	if (hpiLength != -1) {
-		hpiFileBuffer = new unsigned char[hpiLength];
-		if (vfsHandler->LoadFile(file, hpiFileBuffer) < 0) {
-			delete[] hpiFileBuffer;
-			hpiFileBuffer = NULL;
-		}
-		else {
-			filesize = hpiLength;
-			return true;
-		}
+	if (vfsHandler->LoadFile(file, fileBuffer)) {
+		fileSize = fileBuffer.size();
+		return true;
 	}
-	return false;
+	else
+		return false;
 }
 
 
@@ -124,7 +113,7 @@ void CFileHandler::Init(const string& _filename, const string& modes)
 
 bool CFileHandler::FileExists() const
 {
-	return (ifs || hpiFileBuffer);
+	return (fileSize >= 0);
 }
 
 
@@ -136,13 +125,14 @@ int CFileHandler::Read(void* buf,int length)
 		ifs->read((char*)buf, length);
 		return ifs->gcount ();
 	}
-	else if (hpiFileBuffer) {
-		if ((length + hpiOffset) > hpiLength) {
-			length = hpiLength - hpiOffset;
+	else if (!fileBuffer.empty()) {
+		if ((length + filePos) > fileSize) {
+			length = fileSize - filePos;
 		}
 		if (length > 0) {
-			memcpy(buf, &hpiFileBuffer[hpiOffset], length);
-			hpiOffset += length;
+			assert(fileBuffer.size() >= filePos+length);
+			memcpy(buf, &fileBuffer[filePos], length);
+			filePos += length;
 		}
 		return length;
 	}
@@ -161,19 +151,19 @@ void CFileHandler::Seek(int length, ios_base::seekdir where)
 		ifs->clear();
 		ifs->seekg(length, where);
 	}
-	else if (hpiFileBuffer)
+	else if (!fileBuffer.empty())
 	{
 		if (where == std::ios_base::beg)
 		{
-			hpiOffset = length;
+			filePos = length;
 		}
 		else if (where == std::ios_base::cur)
 		{
-			hpiOffset += length;
+			filePos += length;
 		}
 		else if (where == std::ios_base::end)
 		{
-			hpiOffset = hpiLength + length;
+			filePos = fileSize + length;
 		}
 	}
 }
@@ -186,9 +176,9 @@ int CFileHandler::Peek() const
 	if (ifs) {
 		return ifs->peek();
 	}
-	else if (hpiFileBuffer){
-		if (hpiOffset < hpiLength) {
-			return hpiFileBuffer[hpiOffset];
+	else if (!fileBuffer.empty()){
+		if (filePos < fileSize) {
+			return fileBuffer.at(filePos);
 		} else {
 			return EOF;
 		}
@@ -204,8 +194,8 @@ bool CFileHandler::Eof() const
 	if (ifs) {
 		return ifs->eof();
 	}
-	if (hpiFileBuffer) {
-		return (hpiOffset >= hpiLength);
+	if (!fileBuffer.empty()) {
+		return (filePos >= fileSize);
 	}
 	return true;
 }
@@ -213,7 +203,7 @@ bool CFileHandler::Eof() const
 
 int CFileHandler::FileSize() const
 {
-   return filesize;
+   return fileSize;
 }
 
 
@@ -224,7 +214,7 @@ int CFileHandler::GetPos() const
 	if (ifs) {
 		return ifs->tellg();
 	} else {
-		return hpiOffset;
+		return filePos;
 	}
 }
 
@@ -236,9 +226,9 @@ bool CFileHandler::LoadStringData(string& data)
 	if (!FileExists()) {
 		return false;
 	}
-	char* buf = new char[filesize];
-	Read(buf, filesize);
-	data.append(buf, filesize);
+	char* buf = new char[fileSize];
+	Read(buf, fileSize);
+	data.append(buf, fileSize);
 	delete[] buf;
 	return true;
 }
