@@ -1,3 +1,5 @@
+/* This file is part of the Spring engine (GPL v2 or later), see LICENSE.html */
+
 #ifndef MATRIX44F_H
 #define MATRIX44F_H
 
@@ -9,6 +11,14 @@ public:
 	CR_DECLARE_STRUCT(CMatrix44f);
 
 	CMatrix44f();
+
+	template<typename T> CMatrix44f(const T* m44) {
+		m[ 0] = float(m44[ 0]); m[ 1] = float(m44[ 1]); m[ 2] = float(m44[ 2]); m[ 3] = float(m44[ 3]);
+		m[ 4] = float(m44[ 4]); m[ 5] = float(m44[ 5]); m[ 6] = float(m44[ 6]); m[ 7] = float(m44[ 7]);
+		m[ 8] = float(m44[ 8]); m[ 9] = float(m44[ 9]); m[10] = float(m44[10]); m[11] = float(m44[11]);
+		m[12] = float(m44[12]); m[13] = float(m44[13]); m[14] = float(m44[14]); m[15] = float(m44[15]);
+	}
+
 	CMatrix44f(const float3& pos, const float3& x, const float3& y, const float3& z);
 	CMatrix44f(const float& rotX, const float& rotY, const float& rotZ);
 	explicit CMatrix44f(const float3& pos);
@@ -24,13 +34,17 @@ public:
 	void RotateX(float rad);
 	void RotateY(float rad);
 	void RotateZ(float rad);
-	void Rotate(float rad, const float3& axis);	//axis assumed normalized
+	void Rotate(float rad, const float3& axis); // axis is assumed to be normalized
 	void Translate(float x, float y, float z);
 	CMatrix44f Mul(const CMatrix44f& other) const;
 
 	CMatrix44f& InvertInPlace();
 	CMatrix44f Invert() const;
 
+	static double CalculateCofactor(const double m[4][4], int ei, int ej);
+	static bool Invert(const double m[4][4], double mInv[4][4]);
+
+
 	float3 Mul(const float3& vect) const;
 	float3 GetPos(void) const { return float3(m[12], m[13], m[14]); }
 
