@@ -44,9 +44,8 @@ CNetProtocol::~CNetProtocol()
 void CNetProtocol::InitClient(const char *server_addr, unsigned portnum, const std::string& myName, const std::string& myPasswd, const std::string& myVersion)
 {
 	GML_STDMUTEX_LOCK(net); // InitClient
-	int sourceport = configHandler->Get("SourcePort", 0);
 	
-	netcode::UDPConnection* conn = new netcode::UDPConnection(sourceport, server_addr, portnum);
+	netcode::UDPConnection* conn = new netcode::UDPConnection(configHandler->Get("SourcePort", 0), server_addr, portnum);
 	serverConn.reset(conn);
 	serverConn->SendData(CBaseNetProtocol::Get().SendAttemptConnect(myName, myPasswd, myVersion));
 	serverConn->Flush(true);
