@@ -317,7 +317,8 @@ static bool MultisampleTest(void)
  */
 static bool MultisampleVerify(void)
 {
-	GLint buffers, samples;
+	GLint buffers = 0;
+	GLint samples = 0;
 	glGetIntegerv(GL_SAMPLE_BUFFERS_ARB, &buffers);
 	glGetIntegerv(GL_SAMPLES_ARB, &samples);
 	if (buffers && samples) {
@@ -532,9 +533,7 @@ bool SpringApp::GetDisplayGeometry()
 			case SW_SHOWMAXIMIZED:
 				windowState = 1;
 				break;
-			case SW_SHOWMINIMIZED:
-				windowState = 2;
-				break;
+			case SW_SHOWMINIMIZED: //minimized startup breaks init stuff, so don't store it
 			default:
 				windowState = 0;
 		}
@@ -607,7 +606,8 @@ void SpringApp::RestoreWindowPosition()
 				int wState;
 				switch (windowState) {
 					case 1: wState = SW_SHOWMAXIMIZED; break;
-					case 2: wState = SW_SHOWMINIMIZED; break;
+					//Setting the main-window minimized breaks initialization
+					case 2: // wState = SW_SHOWMINIMIZED; break;
 					default: stateChanged = false;
 				}
 				if (stateChanged) {
@@ -1001,11 +1001,10 @@ int SpringApp::Sim()
 			gmlProcessor->ExpandAuxQueue();
 
 			{
-				GML_RECMUTEX_LOCK(sim); // Sim
+				GML_MSTMUTEX_LOCK(sim); // Sim
 
-				if (!activeController->Update()) {
+				if(!activeController->Update())
 					return 0;
-				}
 			}
 
 			gmlProcessor->GetQueue();
@@ -1036,7 +1035,7 @@ int SpringApp::Update()
 #if defined(USE_GML) && GML_ENABLE_SIM
 			if (gmlMultiThreadSim) {
 				if (!gs->frameNum) {
-					GML_RECMUTEX_LOCK(sim); // Update
+					GML_MSTMUTEX_LOCK(sim); // Update
 
 					activeController->Update();
 					if (gs->frameNum) {
@@ -1044,7 +1043,7 @@ int SpringApp::Update()
 					}
 				}
 			} else {
-				GML_RECMUTEX_LOCK(sim); // Update
+				GML_MSTMUTEX_LOCK(sim); // Update
 
 				activeController->Update();
 			}
@@ -1237,7 +1236,7 @@ bool SpringApp::MainEventHandler(const SDL_Event& event)
 	switch (event.type) {
 		case SDL_VIDEORESIZE: {
 
-			GML_RECMUTEX_LOCK(sim); // Run
+			GML_MSTMUTEX_LOCK(sim); // MainEventHandler
 
 			CrashHandler::ClearDrawWDT(true);
 			screenWidth = event.resize.w;
@@ -1249,11 +1248,12 @@ bool SpringApp::MainEventHandler(const SDL_Event& event)
 #endif
 			InitOpenGL();
 			activeController->ResizeEvent();
+
 			break;
 		}
 		case SDL_VIDEOEXPOSE: {
 
-			GML_RECMUTEX_LOCK(sim); // Run
+			GML_MSTMUTEX_LOCK(sim); // MainEventHandler
 
 			CrashHandler::ClearDrawWDT(true);
 			// re-initialize the stencil
@@ -1261,6 +1261,7 @@ bool SpringApp::MainEventHandler(const SDL_Event& event)
 			glClear(GL_STENCIL_BUFFER_BIT); SDL_GL_SwapBuffers();
 			glClear(GL_STENCIL_BUFFER_BIT); SDL_GL_SwapBuffers();
 			SetupViewportGeometry();
+
 			break;
 		}
 		case SDL_QUIT: {
