@@ -1,7 +1,8 @@
+/* This file is part of the Spring engine (GPL v2 or later), see LICENSE.html */
+
 #ifndef UNSYNCEDRNG_H_
 #define UNSYNCEDRNG_H_
 
-
 class UnsyncedRNG
 {
 public:
@@ -23,4 +24,5 @@ private:
 	unsigned randSeed;
 };
 
-#endif /*UNSYNCEDRNG_H_*/
+#endif // UNSYNCEDRNG_H_
+
