@@ -1,3 +1,5 @@
+/* This file is part of the Spring engine (GPL v2 or later), see LICENSE.html */
+
 #ifndef LIST_H
 #define LIST_H
 
@@ -69,6 +71,10 @@ private:
 	bool hasFocus;
 	int topIndex;
 
+	GuiElement scrollbar;
+	bool activeScrollbar;
+	float scrollbarGrabPos;
+
 	// for filtering
 	std::string query;
 	std::vector<std::string>* filteredItems;
