@@ -231,6 +231,7 @@ boost::mutex rprojmutex;
 boost::mutex rflashmutex;
 boost::mutex rpiecemutex;
 boost::mutex rfeatmutex;
+boost::mutex drawmutex;
 
 #include <boost/thread/recursive_mutex.hpp>
 boost::recursive_mutex unitmutex;
@@ -244,8 +245,9 @@ boost::recursive_mutex filemutex;
 boost::recursive_mutex &qnummutex=quadmutex;
 boost::recursive_mutex &groupmutex=selmutex;
 boost::recursive_mutex &grpselmutex=selmutex;
-boost::recursive_mutex simmutex;
 boost::recursive_mutex laycmdmutex;
+
+gmlMutex simmutex;
 #endif
 
 // GMLqueue implementation
